@@ -1,4 +1,4 @@
-/* $Id: connection.h,v 1.6 2005/12/05 21:28:35 kuhlmann Exp $ */
+/* $Id: connection.h,v 1.14 2007-02-27 22:42:52 kuhlmann Exp $ */
 
 #ifndef MICQ_UTIL_CONNECTION_H
 #define MICQ_UTIL_CONNECTION_H
@@ -31,18 +31,20 @@ struct Connection_s
     UBYTE     version;        /* protocol version in this session         */
     UDWORD    uin;            /* the uin of this server connection        */
     char     *screen;
-    UDWORD    pref_status;
+    status_t  pref_status;
     char     *pref_server;
     UDWORD    pref_port;
     char     *pref_passwd;
 
     Contact  *cont;           /* the user this connection is for          */
-    UDWORD    status;         /* status of uin                            */
+    status_t  status;         /* own status                               */
+    UDWORD    nativestatus;   /* own ICQ extended status                  */
     char     *server;         /* the remote server name                   */
     UDWORD    port;           /* the port the server is listening on      */
     char     *passwd;         /* the password for this user               */
     UDWORD    ip;             /* the remote ip (host byte order)          */
-    void     *tlv;            /* temporary during v8 connect              */
+    
+    idleflag_t idle_flag;     /* the idle status                          */
 
     SOK_T     sok;            /* socket for connection to server          */
     UWORD     connect;        /* connection setup status                  */
@@ -50,6 +52,10 @@ struct Connection_s
     Packet   *outgoing;       /* packet we're sending                     */
     
     ContactGroup *contacts;   /* The contacts for this connection         */
+    
+    void     *xmpp_private;   /* private data for XMPP connections        */
+
+    void     *tlv;            /* temporary during v8 connect              */
 
 #if ENABLE_SSL
 #if ENABLE_GNUTLS
@@ -103,10 +109,11 @@ Connection    *ConnectionClone   (Connection *conn, UWORD type DEBUGPARAM);
 void           ConnectionD       (Connection *conn DEBUGPARAM);
 Connection    *ConnectionNr      (int i);
 Connection    *ConnectionFind    (UWORD type, const Contact *cont, const Connection *parent);
-Connection    *ConnectionFindUIN (UWORD type, UDWORD uin);
+Connection    *ConnectionFindScreen (UWORD type, const char *screen);
 UDWORD         ConnectionFindNr  (Connection *conn);
 const char    *ConnectionType    (Connection *conn);
-const char    *ConnectionServerType (Connection *conn);
+const char    *ConnectionServerType  (UWORD type);
+UWORD          ConnectionServerNType (const char *type, char del);
 val_t          ConnectionPrefVal (Connection *conn, UDWORD flag);
 
 #define ConnectionC(t)       ConnectionC (t DEBUGARGS)
@@ -144,23 +151,26 @@ val_t          ConnectionPrefVal (Connection *conn, UDWORD flag);
 #define TYPEF_REMOTE     1024  /* remote control (socket)*/
 #define TYPEF_MSN        2048  /* MSN: connection        */
 #define TYPEF_MSN_CHAT   4096  /* MSN: chat connection   */
+#define TYPEF_XMPP       8192  /* XMPP(Jabber) connection*/
+#define TYPEF_HAVEUIN   16384  /* server session uses numeric UINs */
 
 /* any conn->type may be only any of those values:
  * do not use the flags above unless you _really_ REALLY know what you're doing
  */
-#define TYPE_SERVER_OLD   (TYPEF_ANY_SERVER | TYPEF_SERVER_OLD)
-#define TYPE_SERVER       (TYPEF_ANY_SERVER | TYPEF_SERVER)
-#define TYPE_MSGLISTEN    (TYPEF_ANY_PEER | TYPEF_ANY_MSG  | TYPEF_ANY_LISTEN)
-#define TYPE_MSGDIRECT    (TYPEF_ANY_PEER | TYPEF_ANY_MSG  | TYPEF_ANY_DIRECT)
-#define TYPE_FILELISTEN   (TYPEF_ANY_PEER | TYPEF_ANY_FILE | TYPEF_ANY_LISTEN)
-#define TYPE_FILEDIRECT   (TYPEF_ANY_PEER | TYPEF_ANY_FILE | TYPEF_ANY_DIRECT)
-#define TYPE_CHATLISTEN   (TYPEF_ANY_PEER | TYPEF_ANY_CHAT | TYPEF_ANY_LISTEN)
-#define TYPE_CHATDIRECT   (TYPEF_ANY_PEER | TYPEF_ANY_CHAT | TYPEF_ANY_DIRECT)
-#define TYPE_FILE         TYPEF_FILE
-#define TYPE_REMOTE       TYPEF_REMOTE
-#define TYPE_MSN_TEMP     TYPEF_MSN
-#define TYPE_MSN_SERVER   (TYPEF_ANY_SERVER | TYPEF_MSN)
-#define TYPE_MSN_CHAT     (TYPEF_ANY_SERVER | TYPEF_MSN | TYPEF_MSN_CHAT)
+#define TYPE_SERVER_OLD    (TYPEF_ANY_SERVER | TYPEF_SERVER_OLD | TYPEF_HAVEUIN)
+#define TYPE_SERVER        (TYPEF_ANY_SERVER | TYPEF_SERVER     | TYPEF_HAVEUIN)
+#define TYPE_MSGLISTEN     (TYPEF_ANY_PEER | TYPEF_ANY_MSG  | TYPEF_ANY_LISTEN | TYPEF_HAVEUIN)
+#define TYPE_MSGDIRECT     (TYPEF_ANY_PEER | TYPEF_ANY_MSG  | TYPEF_ANY_DIRECT | TYPEF_HAVEUIN)
+#define TYPE_FILELISTEN    (TYPEF_ANY_PEER | TYPEF_ANY_FILE | TYPEF_ANY_LISTEN | TYPEF_HAVEUIN)
+#define TYPE_FILEDIRECT    (TYPEF_ANY_PEER | TYPEF_ANY_FILE | TYPEF_ANY_DIRECT | TYPEF_HAVEUIN)
+#define TYPE_CHATLISTEN    (TYPEF_ANY_PEER | TYPEF_ANY_CHAT | TYPEF_ANY_LISTEN | TYPEF_HAVEUIN)
+#define TYPE_CHATDIRECT    (TYPEF_ANY_PEER | TYPEF_ANY_CHAT | TYPEF_ANY_DIRECT | TYPEF_HAVEUIN)
+#define TYPE_FILE          TYPEF_FILE
+#define TYPE_REMOTE        TYPEF_REMOTE
+#define TYPE_MSN_TEMP      TYPEF_MSN
+#define TYPE_MSN_SERVER    (TYPEF_ANY_SERVER | TYPEF_MSN)
+#define TYPE_MSN_CHAT      (TYPEF_ANY_SERVER | TYPEF_MSN | TYPEF_MSN_CHAT)
+#define TYPE_XMPP_SERVER   (TYPEF_ANY_SERVER | TYPEF_XMPP)
 
 #define ASSERT_ANY_SERVER(s)  (assert (s), assert ((s)->type & TYPEF_ANY_SERVER))
 #define ASSERT_SERVER(s)      (assert (s), assert ((s)->type == TYPE_SERVER))
