@@ -1,4 +1,4 @@
-/* $Id: file_util.h,v 1.21 2004/07/04 13:17:10 kuhlmann Exp $ */
+/* $Id: file_util.h,v 1.22 2007-04-07 13:50:30 kuhlmann Exp $ */
 
 int  Save_RC(void);
 void Initialize_RC_File (void);
@@ -7,4 +7,4 @@ void PrefReadStat (FILE *stf);
 int PrefWriteStatusFile (void);
 int PrefWriteConfFile (void);
 
-Connection *PrefNewConnection (UDWORD uin, const char *passwd);
+Connection *PrefNewConnection (UDWORD servertype, const char *user, const char *passwd);
