@@ -1,12 +1,10 @@
-/* If this is defined words will not be broken up at the screen's edge
-**  You should leave this defined unless something is broken or you like
-**  the ugly display  */
-#define WORD_WRAP
-
 /* Set this if your terminal is an ANSI terminal. Note: you can use color even
    if this is unset. */
 #define ANSI_TERM
 
+/* Use the arrow keys on ansi terminals.  Very good to have if it works */
+#define USE_MREADLINE
+
 /* remove this line to disable shell commands from within mICQ */
 #define SHELL_COMMANDS
 
@@ -18,9 +16,9 @@
 ** increasing this *MAY* help.  It might do nothing.
 **  The real icq uses a value of 6  */
 #define MAX_RETRY_ATTEMPTS 8
-#define MAX_RETRY_P2P_ATTEMPTS 2
+#define MAX_RETRY_P2P_ATTEMPTS 2    /* _no_ retry */
 #define MAX_RETRY_P2PFILE_ATTEMPTS 6
-#define MAX_RETRY_TYPE2_ATTEMPTS 3
+#define MAX_RETRY_TYPE2_ATTEMPTS 2  /* _no_ retry */
 
 /* This is the time in seconds mICQ waits before retrying resp. before
 ** giving the user feedback about the delay. */
@@ -41,10 +39,6 @@
 ** footprint slightly. */
 #define MAX_SEQ_DEPTH 50
 
-/* This is the maximum number of people you can have in your list.  This is 
-** not only true contacts but aliases to your contacts also. */
-#define MAX_CONTACTS 1024
-
 /* If you change this things will probably come out formated badly.
 ** But you can change the tab stops if you so desire */
 #define TAB_STOP 8
