@@ -1,4 +1,4 @@
-/* $Id: msg_queue.h,v 1.54 2005/02/26 21:08:07 kuhlmann Exp $ */
+/* $Id: msg_queue.h,v 1.55 2006-09-22 09:42:09 kuhlmann Exp $ */
 
 #ifndef MICQ_MSG_QUEUE_H
 #define MICQ_MSG_QUEUE_H
@@ -84,7 +84,7 @@ void        QueuePrint (void);
 #define QUEUE_USERFILEACK       81
 #define QUEUE_ACKNOWLEDGE       80
 #define QUEUE_MICQ_COMMAND      11
-#define QUEUE_DEP_OSCARLOGIN    12
+#define QUEUE_DEP_WAITLOGIN     12
 #define QUEUE_CACHE_MSG         100
 
 #define QUEUE_FLAG_CONSIDERED     1 /* this event has been considered and won't
