@@ -1,4 +1,4 @@
-/* $Id: oscar_base.h,v 1.2 2004/07/24 09:10:05 kuhlmann Exp $ */
+/* $Id: oscar_base.h,v 1.4 2006-09-20 15:54:33 kuhlmann Exp $ */
 
 #ifndef MICQ_ICQV8_FLAP_H
 #define MICQ_ICQV8_FLAP_H
@@ -27,5 +27,4 @@ void    FlapSave (Packet *pak, BOOL in);
 Event *ConnectionInitServer (Connection *serv);
 Connection *SrvRegisterUIN (Connection *serv, const char *pass);
 
-
-#endif /* MICQ_ICQV8_FLAP_H */
+#endif
