@@ -10,7 +10,7 @@ UBYTE SnacCliSendmsg (Connection *serv, Contact *cont, const char *text, UDWORD
 UBYTE SnacCliSendmsg2 (Connection *serv, Contact *cont, Opt *opt);
 
 void SrvMsgAdvanced (Packet *pak, UDWORD seq, UWORD type,
-                     UWORD status, UDWORD deststatus, UWORD flags, const char *msg);
+                     status_t status, status_t deststatus, UWORD flags, const char *msg);
 void SrvMsgGreet (Packet *pak, UWORD cmd, const char *reason, UWORD port, UDWORD len, const char *msg);
 void SrvReceiveAdvanced (Connection *serv, Event *inc_event, Packet *inc_pak, Event *ack_event);
 
