@@ -1,11 +1,11 @@
-/* $Id: oscar_tlv.h,v 1.1 2003/10/11 02:44:03 kuhlmann Exp $ */
+/* $Id: oscar_tlv.h,v 1.2 2006-11-05 18:19:13 kuhlmann Exp $ */
 
 #ifndef MICQ_OSCAR_TLV_H
 #define MICQ_OSCAR_TLV_H
 
 typedef struct { UWORD tag; UDWORD nr; str_s str; } TLV;
 
-TLV  *TLVRead (Packet *pak, UDWORD TLVlen);
+TLV  *TLVRead (Packet *pak, UDWORD TLVlen, UDWORD TLVCount);
 UWORD TLVGet  (TLV *tlv, UWORD nr);
 void  TLVDone (TLV *tlv, UWORD nr);
 void  TLVD    (TLV *tlv);
