@@ -1,4 +1,4 @@
-/* $Id: preferences.h,v 1.57 2005/12/21 14:57:19 kuhlmann Exp $ */
+/* $Id: preferences.h,v 1.59 2007-04-09 14:33:54 kuhlmann Exp $ */
 
 #ifndef MICQ_PREFERENCES_H
 #define MICQ_PREFERENCES_H
@@ -25,7 +25,7 @@ struct Preferences_s
 
     UDWORD verbose;     /* verbosity to use on startup */
     UWORD  sound;       /* how to beep */
-    UDWORD status;      /* status to use when logging in */
+    status_t status;    /* status to use when logging in */
     UWORD  screen;      /* manual maximum screen width; 0 = auto */
     UDWORD flags;       /* flags for output */
     UDWORD away_time;   /* time after which to be away automatically; 0 = disable */
@@ -42,6 +42,9 @@ struct Preferences_s
     SBYTE  chat;
     SBYTE  autoupdate;
 
+    char  *prompt_strftime;    /* user define prompt */
+    char  *prompt;             /* user define prompt */
+
 #ifdef ENABLE_TCL
     tcl_pref_p tclscript;
     tcl_callback *tclout;
@@ -82,6 +85,7 @@ const char *PrefLogNameReal (Preferences *pref);
 #define FLAG_DEP_ULTRAQUIET (1 << 13)
 #define FLAG_AUTOSAVE   (1 << 14)
 #define FLAG_AUTOFINGER (1 << 15)
+#define FLAG_USERPROMPT (1 << 16)
 /*      FLAG_S5
  *      FLAG_S5_USE
  */
