@@ -1,4 +1,4 @@
-/* $Id: tcp.h,v 1.34 2003/10/10 19:53:03 kuhlmann Exp $ */
+/* $Id: tcp.h,v 1.35 2006-09-20 01:04:26 kuhlmann Exp $ */
 
 #ifndef MICQ_PEER_H
 #define MICQ_PEER_H
@@ -21,7 +21,7 @@ BOOL TCPSendFiles      (Connection *list, Contact *cont, const char *description
 BOOL TCPGetAuto        (Connection *list, Contact *cont, UWORD which);
 
 Connection *PeerFileCreate    (Connection *serv);
-BOOL        PeerFileAccept    (Connection *peer, UWORD status, UDWORD port);
+BOOL        PeerFileAccept    (Connection *peer, UWORD ackstatus, UDWORD port);
 UBYTE       PeerFileIncAccept (Connection *list, Event *event);
 
 void PeerFileResend (Event *event);
