@@ -81,6 +81,7 @@ extern struct OptEntry_s OptList[];
 #define CO_DCAUTH        (COF_BOOL    | CO_GROUP   | 0x001003UL) /* this connection requires auth for dc  */
 #define CO_DCCONT        (COF_BOOL    | CO_GROUP   | 0x004003UL) /* this connection: dc only for contacts */
 #define CO_OBEYSBL       (COF_BOOL    | CO_GROUP   | 0x010003UL) /* this connection obeys the sbl         */
+#define CO_AWAYCOUNT     (COF_BOOL    | CO_GROUP   | 0x100003UL) /* this connection counts msgs even if _manual_ na/away/... */
 
 #define CO_ENCODING      (COF_NUMERIC | CO_CONTACT | 0x04UL) /* the default encoding for this contact */
 #define CO_ENCODINGSTR   (COF_STRING  | CO_CONTACT | 0x05UL) /* the default encoding for this contact */
@@ -134,16 +135,18 @@ extern struct OptEntry_s OptList[];
 #define CO_FILENAME      (COF_STRING               | 0x29UL) /* a filename */
 #define CO_FILEACCEPT    (COF_NUMERIC              | 0x2aUL) /* accept a file transfer */
 #define CO_REFUSE        (COF_STRING               | 0x2bUL) /* refuse message */
+#define CO_SUBJECT       (COF_STRING               | 0x2cUL) /* message with subject */
 
 #define CV_ORIGIN_v5  8
 #define CV_ORIGIN_v8  2
 #define CV_ORIGIN_dc  1
 #define CV_ORIGIN_ssl 16
 
-#define CV_MSGTRANS_DC     1
-#define CV_MSGTRANS_TYPE2  2
-#define CV_MSGTRANS_ICQv8  4
-#define CV_MSGTRANS_ICQv5  8
-#define CV_MSGTRANS_ANY   15
+#define CV_MSGTRANS_DC      1
+#define CV_MSGTRANS_TYPE2   2
+#define CV_MSGTRANS_ICQv8   4
+#define CV_MSGTRANS_ICQv5   8
+#define CV_MSGTRANS_XMPP   16
+#define CV_MSGTRANS_ANY    31
 
 #endif
