@@ -1,4 +1,4 @@
-/* $Id: util_str.h,v 1.27 2003/10/10 18:12:44 kuhlmann Exp $ */
+/* $Id: util_str.h,v 1.32 2007-04-17 21:13:42 kuhlmann Exp $ */
 
 #ifndef MICQ_UTIL_STR_H
 #define MICQ_UTIL_STR_H
@@ -37,8 +37,10 @@ void        s_done    (str_t str);
 
 const char *s_sprintf (const char *fmt, ...) __attribute__ ((format (__printf__, 1, 2)));
 const char *s_ip      (UDWORD ip);
-const char *s_status  (UDWORD status);
+const char *s_status  (status_t status, UDWORD nativestatus);
+const char *s_status_short  (status_t status);
 const char *s_time    (time_t *stamp);
+const char *s_strftime (time_t *stamp, const char *fmt, char as_gmt);
 const char *s_msgtok  (char *);
 
 const char *s_dump    (const UBYTE *data, UWORD len);
