@@ -22,11 +22,11 @@ typedef void (tcl_callback)(const char *s);
 
 void TCLInit ();
 void TCLPrefAppend (Tcl_type type, char *file);
-void TCLEvent (const char *type, const char *data);
+void TCLEvent (Contact *from, const char *type, const char *data);
 void TCLMessage (Contact *from, const char *text);
 
 #include "cmd_user.h"
 jump_f CmdUserTclScript;
 
 #endif /* MICQ_UTIL_TCL_H */
-#endif /* ENABLE_TCL */
+#endif
