@@ -1,4 +1,4 @@
-/* $Id: util_ui.h,v 1.43 2003/10/09 20:47:31 kuhlmann Exp $ */
+/* $Id: util_ui.h,v 1.45 2007-02-27 21:56:35 kuhlmann Exp $ */
 
 #ifndef MICQ_UTIL_UI_H
 #define MICQ_UTIL_UI_H
@@ -24,6 +24,9 @@ BOOL DebugReal (UDWORD level, const char *str, ...) __attribute__ ((format (__pr
 #define DEB_TCP           0x00200000L
 #define DEB_IO            0x00400000L
 #define DEB_SSL           0x00800000L
+#define DEB_XMPPIN        0x01000000L
+#define DEB_XMPPOUT       0x02000000L
+#define DEB_XMPPOTHER     0x04000000L
 
 #define AVPFMT "%s%-15s%s %s\n"
 
