@@ -26,10 +26,11 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: buildmark.c,v 1.76 2006/01/16 18:47:30 kuhlmann Exp $
+ * $Id: buildmark.c,v 1.81 2007-04-18 00:05:35 kuhlmann Exp $
  */
 
-#define MICQ_BUILD_NUM 0x00050100
+/*                        0.5.3 */
+#define MICQ_BUILD_NUM 0x00050300
 
 #include "micq.h"
 #include "buildmark.h"
@@ -116,9 +117,22 @@
 #define EV_SSL
 #endif
 
-#define EV EV_AP EV_SSL EV_TCL EV_P2P
+#ifdef ENABLE_XMPP
+#define EV_XMPP "XMPP "
+#else
+#define EV_XMPP
+#endif
+
+#ifdef ENABLE_MSN
+#define EV_MSN "MSN "
+#else
+#define EV_MSN
+#endif
+
+#define EV EV_AP EV_SSL EV_TCL EV_P2P EV_XMPP EV_MSN
 
 const UDWORD BuildPlatformID = BUILD_PLATFORM;
+const char  *BuildPlatformStr = EXTRAVERSION_DEF;
 
 const char *BuildVersion (void)
 {
@@ -142,7 +156,8 @@ const char *BuildAttribution (void)
 }                  
 
 const UDWORD BuildVersionNum = MICQ_BUILD_NUM;
-const char  *BuildVersionText = "$VER: mICQ " VERSION " " EV EXTRAVERSION "\n(" CVSUPDATE " build " BUILDDATE ")";
+const char *BuildVersionText = "$VER: mICQ " VERSION " " EV EXTRAVERSION "\n(" CVSUPDATE " build " BUILDDATE ")";
+const char *BuildVersionStr  = VERSION MICQ_IS_CVS;
 
 /*
  i18n (1000, "UTF-8")            charset used
