@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: connection.c,v 1.13 2006/01/16 21:52:12 kuhlmann Exp $
+ * $Id: connection.c,v 1.20 2007-02-27 22:42:51 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -88,7 +88,7 @@ Connection *ConnectionC (UWORD type DEBUGPARAM)
     
     conn->our_local_ip   = 0x7f000001;
     conn->our_outside_ip = 0x7f000001;
-    conn->status = STATUS_ICQOFFLINE;
+    conn->status = ims_offline;
     conn->sok = -1;
     conn->type = type;
     conn->flags = CONN_CONFIGURED;
@@ -116,7 +116,8 @@ Connection *ConnectionClone (Connection *conn, UWORD type DEBUGPARAM)
     child->flags  = 0;
     child->screen = strdup (conn->screen ? conn->screen : "child");
     
-    Debug (DEB_CONNECT, "<=*= %p (%s) clone from %p (%s)", child, ConnectionType (child), conn, ConnectionType (conn));
+    Debug (DEB_CONNECT, "<=*= %p (%s) clone from %p (%s)",
+        child, ConnectionType (child), conn, ConnectionType (conn));
 
     return child;
 }
@@ -188,21 +189,22 @@ Connection *ConnectionFind (UWORD type, const Contact *cont, const Connection *p
 }
 
 /*
- * Finds a session of given type and/or given uin.
+ * Finds a session of given type and given screen.
  * Actually, you may specify TYPEF_* here that must all be set.
- * The parent is the session this one has to have as parent.
  */
-Connection *ConnectionFindUIN (UWORD type, UDWORD uin)
+Connection *ConnectionFindScreen (UWORD type, const char *screen)
 {
     ConnectionList *cl;
     Connection *conn;
     int i;
-    
-    assert (type && uin);
 
+    assert (type);    
+    assert (screen);
+    
     for (cl = &slist; cl; cl = cl->more)
         for (i = 0; i < ConnectionListLen; i++)
-            if ((conn = cl->conn[i]) && (conn->type & type) == type && conn->uin == uin)
+            if ((conn = cl->conn[i]) && (conn->type & type) == type
+                && !strcmp (conn->screen, screen))
                 return conn;
 
     return NULL;
@@ -302,6 +304,8 @@ void ConnectionD (Connection *conn DEBUGPARAM)
 const char *ConnectionType (Connection *conn)
 {
     switch (conn->type) {
+        case TYPE_XMPP_SERVER:
+            return i18n (2604, "xmpp");
         case TYPE_MSN_TEMP:
             return i18n (2584, "msn temp");
         case TYPE_MSN_SERVER:
@@ -329,9 +333,11 @@ const char *ConnectionType (Connection *conn)
     }
 }
 
-const char *ConnectionServerType (Connection *conn)
+const char *ConnectionServerType (UWORD type)
 {
-    switch (conn->type) {
+    switch (type) {
+        case TYPE_XMPP_SERVER:
+            return "xmpp";
         case TYPE_MSN_SERVER:
             return "msn";
         case TYPE_SERVER:
@@ -347,6 +353,19 @@ const char *ConnectionServerType (Connection *conn)
     }
 }
 
+UWORD ConnectionServerNType (const char *type, char del)
+{
+    if (!strncmp (type, "icq8", 4)   && type[4] == del) return TYPE_SERVER;
+    if (!strncmp (type, "icq5", 4)   && type[4] == del) return TYPE_SERVER_OLD;
+    if (!strncmp (type, "peer", 4)   && type[4] == del) return TYPE_MSGLISTEN;
+    if (!strncmp (type, "remote", 6) && type[6] == del) return TYPE_REMOTE;
+    if (!strncmp (type, "msn", 3)    && type[3] == del) return TYPE_MSN_SERVER;
+    if (!strncmp (type, "jabber", 6) && type[6] == del) return TYPE_XMPP_SERVER;
+    if (!strncmp (type, "xmpp", 4)   && type[4] == del) return TYPE_XMPP_SERVER;
+    return 0;
+}
+
+
 /*
  * Query an option for a contact group
  */
