@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: contact.c,v 1.133 2005/12/22 23:08:21 kuhlmann Exp $
+ * $Id: contact.c,v 1.150 2007-04-17 22:10:13 kuhlmann Exp $
  */
 
 #include <ctype.h>
@@ -37,12 +37,13 @@
 #include "micq.h"
 #include "contact.h"
 #include "connection.h"
-#include "util_ui.h"     /* for Debug() and DEB_CONTACT */
-#include "conv.h"        /* for meta data file recoding */
-#include "util_io.h"     /* for UtilIOReadline() */
-#include "packet.h"      /* for capabilities */
-#include "buildmark.h"   /* for versioning */
-#include "preferences.h" /* for BASEDIR */
+#include "util_ui.h"       /* for Debug() and DEB_CONTACT */
+#include "conv.h"          /* for meta data file recoding */
+#include "util_io.h"       /* for UtilIOReadline() */
+#include "packet.h"        /* for capabilities */
+#include "buildmark.h"     /* for versioning */
+#include "preferences.h"   /* for BASEDIR */
+#include "oldicq_compat.h" /* for IcqIsUIN */
 #include "util_parse.h"
 
 static ContactGroup **cnt_groups = NULL;
@@ -193,6 +194,56 @@ void ContactGroupD (ContactGroup *group DEBUGPARAM)
         }
 }
 
+void ContactGroupSort (ContactGroup *group, contact_sort_func_t sort, int mode)
+{
+    ContactGroup *orig;
+    int i, j;
+    int found = 1, where = 0, res;
+    Contact **a = NULL, **b = NULL, *m;
+    
+    if (!group)
+    {
+        if (!cnt_groups)
+            ContactGroupInit ();
+        group = CONTACTGROUP_GLOBAL;
+    }
+    orig = group;
+    while (found)
+    {
+        found = 0;
+        j = 0;
+        i = 0;
+        b = NULL;
+        group = orig;
+        while (group)
+        {
+            if (i >= group->used)
+            {
+                i = 0;
+                group = group->more;
+                continue;
+            }
+            if (where && j > where)
+                break;
+            a = b;
+            b = &group->contacts[i];
+            i++;
+            j++;
+            if (!a)
+                continue;
+            res = sort (*a, *b, mode);
+            if (res < 0)
+            {
+                m = *b;
+                *b = *a;
+                *a = m;
+                found = j;
+            }
+        }
+        where = found;
+    }
+}
+
 /*
  * Iterate through contacts on a contact group
  */
@@ -230,25 +281,202 @@ Contact *ContactIndex (ContactGroup *group, int i)
     return group->contacts[i];
 }
 
-static Contact *ContactC (Connection *serv, UDWORD uin, const char *nick DEBUGPARAM)
+#undef ContactCUIN
+static Contact *ContactCUIN (Connection *serv, UDWORD uin DEBUGPARAM)
 {
     Contact *cont;
 
-    if (!(cont = calloc (1, sizeof (Contact))))
+    assert (uin);
+    cont = calloc (1, sizeof (Contact));
+    if (!cont)
         return NULL;
+    
+    cont->ids = NULL;
+    cont->status = ims_offline;
+    cont->serv = serv;
 
     cont->uin = uin;
-    cont->ids = NULL;
-    cont->status = STATUS_ICQOFFLINE;
-    
-    s_repl (&cont->nick, nick ? nick : s_sprintf ("%ld", uin));
-    s_repl (&cont->screen, nick && serv->type != TYPE_SERVER ? nick : s_sprintf ("%ld", uin));
+    s_repl (&cont->screen, s_sprintf ("%ld", uin));
+    s_repl (&cont->nick, cont->screen);
 
-    Debug (DEB_CONTACT, "new  %p %ld '%s' '%s' %p", cont, uin, cont->screen, cont->nick, cont);
+    Debug (DEB_CONTACT, "new  %p UIN %s %p %p", cont, cont->screen, cont, serv);
+
+    if (!cnt_groups)
+        ContactGroupInit ();
     ContactAdd (CONTACTGROUP_GLOBAL, cont);
+
     return cont;
 }
 
+#undef ContactCScreen
+static Contact *ContactCScreen (Connection *serv, const char *screen DEBUGPARAM);
+static Contact *ContactCScreen (Connection *serv, const char *screen DEBUGPARAM)
+{
+    Contact *cont;
+    
+    cont = calloc (1, sizeof (Contact));
+    if (!cont)
+        return NULL;
+
+    cont->ids = NULL;
+    cont->status = ims_offline;
+    cont->serv = serv;
+
+    cont->uin = 0;
+    s_repl (&cont->screen, screen);
+    s_repl (&cont->nick, cont->screen);
+
+    Debug (DEB_CONTACT, "new  %p UIN '%s' %p %p", cont, cont->screen, cont, serv);
+
+    if (!cnt_groups)
+        ContactGroupInit ();
+    ContactAdd (CONTACTGROUP_GLOBAL, cont);
+
+    return cont;
+}
+
+/*
+ * Finds a contact on a contact group by UIN
+ */
+Contact *ContactFindUIN (ContactGroup *group, UDWORD uin)
+{
+    ContactGroup *tmp;
+    Contact *cont;
+    int i;
+    
+    for (tmp = group; tmp; tmp = tmp->more)
+    {
+        for (i = 0; i < tmp->used; i++)
+        {
+            cont = tmp->contacts[i];
+            if (uin == cont->uin)
+                return cont;
+        }
+    }
+    return NULL;
+}
+
+/*
+ * Finds a contact on a contact group by screen name
+ */
+Contact *ContactFindScreen (ContactGroup *group, const char *screen)
+{
+    ContactGroup *tmp;
+    Contact *cont;
+    int i;
+    
+    if (!screen)
+        return NULL;
+    
+    for (tmp = group; tmp; tmp = tmp->more)
+    {
+        for (i = 0; i < tmp->used; i++)
+        {
+            cont = tmp->contacts[i];
+            if (!strcasecmp (screen, cont->screen))
+                return cont;
+        }
+    }
+    return NULL;
+}
+
+#if ENABLE_CONT_HIER
+/*
+ * Finds a contact on a contact group by screen name under a parent
+ */
+Contact *ContactFindScreenP (ContactGroup *group, Contact *parent, const char *screen)
+{
+    ContactGroup *tmp;
+    Contact *cont;
+    int i;
+    
+    if (!screen)
+        return NULL;
+    
+    for (tmp = group; tmp; tmp = tmp->more)
+    {
+        for (i = 0; i < tmp->used; i++)
+        {
+            cont = tmp->contacts[i];
+            if (cont->parent == parent && !strcasecmp (screen, cont->screen))
+                return cont;
+        }
+    }
+    return NULL;
+}
+#endif
+
+/*
+ * Finds a contact on a contact group by UIN
+ */
+static Contact *ContactFindSUIN (ContactGroup *group, Connection *serv, UDWORD uin)
+{
+    ContactGroup *tmp;
+    Contact *cont;
+    int i;
+    
+    for (tmp = group; tmp; tmp = tmp->more)
+    {
+        for (i = 0; i < tmp->used; i++)
+        {
+            cont = tmp->contacts[i];
+            if (uin == cont->uin && serv == cont->serv)
+                return cont;
+        }
+    }
+    return NULL;
+}
+
+/*
+ * Finds a contact on a contact group by screen name
+ */
+static Contact *ContactFindSScreen (ContactGroup *group, Connection *serv, const char *screen)
+{
+    ContactGroup *tmp;
+    Contact *cont;
+    int i;
+    
+    if (!screen)
+        return NULL;
+    
+    for (tmp = group; tmp; tmp = tmp->more)
+    {
+        for (i = 0; i < tmp->used; i++)
+        {
+            cont = tmp->contacts[i];
+            if (serv == cont->serv && !strcasecmp (screen, cont->screen))
+                return cont;
+        }
+    }
+    return NULL;
+}
+
+#if ENABLE_CONT_HIER
+/*
+ * Finds a contact on a contact group by screen name under a parent
+ */
+static Contact *ContactFindSScreenP (ContactGroup *group, Connection *serv, Contact *parent, const char *screen)
+{
+    ContactGroup *tmp;
+    Contact *cont;
+    int i;
+    
+    if (!screen)
+        return NULL;
+    
+    for (tmp = group; tmp; tmp = tmp->more)
+    {
+        for (i = 0; i < tmp->used; i++)
+        {
+            cont = tmp->contacts[i];
+            if (serv == cont->serv && cont->parent == parent && !strcasecmp (screen, cont->screen))
+                return cont;
+        }
+    }
+    return NULL;
+}
+#endif
+
 /*
  * Finds a contact for a connection
  */
@@ -256,33 +484,70 @@ static Contact *ContactC (Connection *serv, UDWORD uin, const char *nick DEBUGPA
 Contact *ContactScreen (Connection *serv, const char *screen DEBUGPARAM)
 {
     Contact *cont;
-    UDWORD uin = atoi (screen);
+    UDWORD uin;
     
-    if (!serv || !serv->contacts || !uin)
+    if (!serv->contacts || !*screen)
         return NULL;
 
     if (!cnt_groups)
         ContactGroupInit ();
-
-    if (uin && (cont = ContactFind (serv->contacts, 0, uin, NULL)))
-        return cont;
     
-    if ((cont = ContactFind (serv->contacts, 0, uin, screen)))
+    uin = IcqIsUIN (screen);
+    if (uin && serv->type & TYPEF_HAVEUIN)
+        return ContactUIN (serv, uin);
+
+    if ((cont = ContactFindScreen (serv->contacts, screen)))
         return cont;
-    
-    if (uin && (cont = ContactFind (CONTACTGROUP_NONCONTACTS, 0, uin, NULL)))
+    if (!cnt_groups)
+        ContactGroupInit ();
+    if ((cont = ContactFindSScreen (CONTACTGROUP_NONCONTACTS, serv, screen)))
         return cont;
 
-    if ((cont = ContactFind (CONTACTGROUP_NONCONTACTS, 0, uin, screen)))
-        return cont;
-
-    if (!(cont = ContactC (serv, uin, screen DEBUGFOR)))
+    cont = ContactCScreen (serv, screen DEBUGFOR);
+    if (!cont)
         return NULL;
 
     ContactAdd (CONTACTGROUP_NONCONTACTS, cont);
     return cont;
 }
 
+#if ENABLE_CONT_HIER
+/*
+ * Finds a contact for a connection under a parent
+ */
+#undef ContactScreenP
+Contact *ContactScreenP (Connection *serv, Contact *parent, const char *screen DEBUGPARAM)
+{
+    Contact *cont;
+    UDWORD uin;
+    
+    if (!serv->contacts || !*screen)
+        return NULL;
+
+    if (!cnt_groups)
+        ContactGroupInit ();
+    
+    uin = IcqIsUIN (screen);
+    if (uin && serv->type & TYPEF_HAVEUIN)
+        return ContactUIN (serv, uin);
+
+    if ((cont = ContactFindScreenP (serv->contacts, parent, screen)))
+        return cont;
+    if (!cnt_groups)
+        ContactGroupInit ();
+    if ((cont = ContactFindSScreenP (CONTACTGROUP_NONCONTACTS, serv, parent, screen)))
+        return cont;
+
+    cont = ContactCScreen (serv, screen DEBUGFOR);
+    if (!cont)
+        return NULL;
+    cont->parent = parent;
+
+    ContactAdd (CONTACTGROUP_NONCONTACTS, cont);
+    return cont;
+}
+#endif
+
 /*
  * Finds a contact for a connection
  */
@@ -291,55 +556,60 @@ Contact *ContactUIN (Connection *serv, UDWORD uin DEBUGPARAM)
 {
     Contact *cont;
     
-    if (!serv || !serv->contacts || !uin)
-        return NULL;
+    assert (serv->contacts);
 
     if (!cnt_groups)
         ContactGroupInit ();
 
-    if ((cont = ContactFind (serv->contacts, 0, uin, NULL)))
+    if ((cont = ContactFindUIN (serv->contacts, uin)))
         return cont;
-    
-    if ((cont = ContactFind (CONTACTGROUP_NONCONTACTS, 0, uin, NULL)))
+    if (!cnt_groups)
+        ContactGroupInit ();
+    if ((cont = ContactFindSUIN (CONTACTGROUP_NONCONTACTS, serv, uin)))
         return cont;
 
-    if (!(cont = ContactC (serv, uin, NULL DEBUGFOR)))
-        return NULL;
+    cont = ContactCUIN (serv, uin DEBUGFOR);
+    if (!cont)
+        return cont;
 
     ContactAdd (CONTACTGROUP_NONCONTACTS, cont);
     return cont;
 }
 
 /*
- * Finds a contact on a contact group
+ * Finds an alias of a contact
  */
-Contact *ContactFind (ContactGroup *group, UWORD id, UDWORD uin, const char *nick)
+const char *ContactFindAlias (Contact *cont, const char *nick)
+{
+    ContactAlias *ca;
+
+    if (!strcasecmp (nick, cont->screen))
+        return cont->screen;
+    if (!strcasecmp (nick, cont->nick))
+        return cont->nick;
+    for (ca = cont->alias; ca; ca = ca->more)
+        if (!strcmp (nick, ca->alias))
+            return ca->alias;
+    return NULL;
+}
+
+
+/*
+ * Finds a contact by nick
+ */
+Contact *ContactFind (Connection *serv, const char *nick)
 {
     ContactGroup *tmp;
     ContactAlias *ca;
     Contact *cont;
     int i;
     
-    if (!group && !cnt_groups)
-        ContactGroupInit ();
-
-    for (tmp = group ? group : CONTACTGROUP_GLOBAL; tmp; tmp = tmp->more)
+    for (tmp = serv->contacts; tmp; tmp = tmp->more)
     {
         for (i = 0; i < tmp->used; i++)
         {
             cont = tmp->contacts[i];
-            if (uin && uin != cont->uin)
-                continue;
-            if (id)
-            {
-                ContactIDs *ids;
-                for (ids = cont->ids; ids; ids = ids->next)
-                    if (ids->id == id)
-                        break;
-                if (!ids)
-                    continue;
-            }
-            if (!nick || !strcmp (nick, cont->screen) || !strcmp (nick, cont->nick))
+            if (!strcasecmp (nick, cont->screen) || !strcasecmp (nick, cont->nick))
                 return cont;
             for (ca = cont->alias; ca; ca = ca->more)
                 if (!strcmp (nick, ca->alias))
@@ -352,65 +622,16 @@ Contact *ContactFind (ContactGroup *group, UWORD id, UDWORD uin, const char *nic
 #undef ContactCreate
 void ContactCreate (Connection *serv, Contact *cont DEBUGPARAM)
 {
-    if (!serv || !cont || cont->group)
+    assert (cont->serv == serv);
+    if (cont->group)
         return;
 
+    if (!cnt_groups)
+        ContactGroupInit ();
     ContactRem (CONTACTGROUP_NONCONTACTS, cont);
     ContactAdd (serv->contacts, cont);
     cont->group = serv->contacts;
-    Debug (DEB_CONTACT, "accc  #%d %ld '%s' %p in %p", 0, cont->uin, cont->nick, cont, serv->contacts);
-}
-
-
-/*
- * Finds a contact on a contact group, possibly creating it
- */
-#undef ContactFindCreate
-Contact *ContactFindCreate (ContactGroup *group, UDWORD uin, const char *nick DEBUGPARAM)
-{
-    Contact *cont;
-    
-    if (!group && !cnt_groups)
-        ContactGroupInit ();
-
-    assert (uin || (group->serv->type != TYPE_SERVER));
-    assert (group);
-
-    if ((cont = ContactFind (group, 0, uin, NULL)))
-    {
-        if (nick)
-            ContactAddAlias (cont, nick);
-        return cont;
-    }
-    
-    if ((cont = ContactFind (CONTACTGROUP_NONCONTACTS, 0, uin, NULL)))
-    {
-        if (!nick)
-            return cont;
-        ContactRem (CONTACTGROUP_NONCONTACTS, cont);
-        ContactAdd (group, cont);
-        cont->group = group;
-        s_repl (&cont->nick, nick);
-        Debug (DEB_CONTACT, "act   %ld '%s' %p in %p", uin, nick, cont, group);
-        return cont;
-    }
-    
-    if (!(cont = ContactC (group->serv, uin, nick DEBUGFOR)))
-        return NULL;
-
-    if (nick)
-    {
-        ContactAdd (group, cont);
-        cont->group = group;
-        Debug (DEB_CONTACT, "act   %ld '%s' %p", uin, nick, cont);
-    }
-    else
-    {
-        ContactAdd (CONTACTGROUP_NONCONTACTS, cont);
-        Debug (DEB_CONTACT, "temp  %ld %p", uin, cont);
-    }
-
-    return cont;
+    Debug (DEB_CONTACT, "accc  #%d %s '%s' %p in %p", 0, cont->screen, cont->nick, cont, serv->contacts);
 }
 
 /*
@@ -438,13 +659,15 @@ void ContactD (Contact *cont DEBUGPARAM)
     }
     cont->ids = NULL;
     cont->alias = NULL;
-    s_repl (&cont->nick, s_sprintf ("%ld", cont->uin));
+    s_repl (&cont->nick, cont->screen);
     
     for (i = 0; (cg = ContactGroupIndex (i)); i++)
         ContactRem (cg, cont);
+    if (!cnt_groups)
+        ContactGroupInit ();
     ContactAdd (CONTACTGROUP_NONCONTACTS, cont);
     cont->group = NULL;
-    Debug (DEB_CONTACT, "del   #%d %ld %p", 0, cont->uin, cont);
+    Debug (DEB_CONTACT, "del   #%d %s %p", 0, cont->screen, cont);
 }
 
 
@@ -455,14 +678,10 @@ void ContactD (Contact *cont DEBUGPARAM)
 BOOL ContactAdd (ContactGroup *group, Contact *cont DEBUGPARAM)
 {
     ContactGroup *orig = group;
-    if (!group)
-    {
-        if (!cnt_groups)
-            ContactGroupInit ();
-        group = CONTACTGROUP_GLOBAL;
-    }
-    if (!group || !cont)
-        return FALSE;
+    assert (group);
+    assert (cont);
+    assert (!group->serv || group->serv == cont->serv);
+
     while (group->used == MAX_ENTRIES && group->more)
         group = group->more;
     if (group->used == MAX_ENTRIES)
@@ -473,7 +692,7 @@ BOOL ContactAdd (ContactGroup *group, Contact *cont DEBUGPARAM)
         group = group->more;
     }
     group->contacts[group->used++] = cont;
-    Debug (DEB_CONTACT, "add   #%d %ld '%s' %p to %p", 0, cont->uin, cont->nick, cont, orig);
+    Debug (DEB_CONTACT, "add   #%d %s '%s' %p to %p", 0, cont->screen, cont->nick, cont, orig);
     return TRUE;
 }
 
@@ -484,14 +703,6 @@ BOOL ContactHas (ContactGroup *group, Contact *cont)
 {
     int i;
 
-    if (!group)
-    {
-        if (!cnt_groups)
-            ContactGroupInit ();
-        group = CONTACTGROUP_GLOBAL;
-    }
-    if (!group || !cont)
-        return FALSE;
     while (group)
     {
         for (i = 0; i < group->used; i++)
@@ -511,14 +722,6 @@ BOOL ContactRem (ContactGroup *group, Contact *cont DEBUGPARAM)
     ContactGroup *orig = group;
     int i;
 
-    if (!group)
-    {
-        if (!cnt_groups)
-            ContactGroupInit ();
-        group = CONTACTGROUP_GLOBAL;
-    }
-    if (!group || !cont)
-        return FALSE;
     while (group)
     {
         for (i = 0; i < group->used; i++)
@@ -538,7 +741,7 @@ BOOL ContactRem (ContactGroup *group, Contact *cont DEBUGPARAM)
                     else
                         cont->group = NULL;
                 }
-                Debug (DEB_CONTACT, "rem   #%d %ld '%s' %p to %p", 0, cont->uin, cont->nick, cont, orig);
+                Debug (DEB_CONTACT, "rem   #%d %s '%s' %p to %p", 0, cont->screen, cont->nick, cont, orig);
                 return TRUE;
             }
         group = group->more;
@@ -555,9 +758,6 @@ BOOL ContactAddAlias (Contact *cont, const char *nick DEBUGPARAM)
     ContactAlias **caref;
     ContactAlias *ca;
     
-    if (!nick)
-        return FALSE;
-
     if (!strcmp (cont->nick, nick))
         return TRUE;
     
@@ -565,13 +765,13 @@ BOOL ContactAddAlias (Contact *cont, const char *nick DEBUGPARAM)
         if (!strcmp ((*caref)->alias, nick))
             return TRUE;
     
-    if (!strcmp (cont->nick, s_sprintf ("%ld", cont->uin)))
+    if (!strcmp (cont->nick, cont->screen))
     {
         s_repl (&cont->nick, nick);
         return TRUE;
     }
     
-    if (!strcmp (nick, s_sprintf ("%ld", cont->uin)))
+    if (!strcmp (nick, cont->screen))
         return TRUE;
     
     ca = calloc (1, sizeof (ContactAlias));
@@ -585,7 +785,7 @@ BOOL ContactAddAlias (Contact *cont, const char *nick DEBUGPARAM)
         return FALSE;
     }
     *caref = ca;
-    Debug (DEB_CONTACT, "addal #%d %ld '%s' A'%s'", 0, cont->uin, cont->nick, nick);
+    Debug (DEB_CONTACT, "addal #%d %s '%s' A'%s'", 0, cont->screen, cont->nick, nick);
     return TRUE;
 }
 
@@ -613,7 +813,7 @@ BOOL ContactRemAlias (Contact *cont, const char *nick DEBUGPARAM)
             nn = strdup ("");
         free (cont->nick);
         cont->nick = nn;
-        Debug (DEB_CONTACT, "remal #%d %ld N'%s' '%s'", 0, cont->uin, cont->nick, nick);
+        Debug (DEB_CONTACT, "remal #%d %s N'%s' '%s'", 0, cont->screen, cont->nick, nick);
         return TRUE;
     }
 
@@ -624,12 +824,35 @@ BOOL ContactRemAlias (Contact *cont, const char *nick DEBUGPARAM)
             ca = *caref;
             *caref = (*caref)->more;
             free (ca);
-            Debug (DEB_CONTACT, "remal #%d %ld '%s' X'%s'", 0, cont->uin, cont->nick, nick);
+            Debug (DEB_CONTACT, "remal #%d %s '%s' X'%s'", 0, cont->screen, cont->nick, nick);
             return TRUE;
         }
 
     return FALSE;
 }
+                         /* -1   0   1 */
+int ContactStatusCmp (status_t a, status_t b)
+{
+    status_noi_t aa, bb;
+    aa = ContactClearInv (a);
+    bb = ContactClearInv (b);
+    if (aa == bb)
+    {
+        if (a == b)
+            return 0;
+        if (ContactIsInv (a) == ContactIsInv (b))
+            return 0;
+        return ContactIsInv (a) ? 1 : -1;
+    }
+    else
+    {
+        if (aa == imr_ffc)     return -1;
+        if (bb == imr_ffc)     return 1;
+        /* assumes further states are ordered by absentibility */
+        if (aa < bb)           return -1;
+        return 1;
+    }
+}
 
 /*
  * Returns the contact id for type, if available
@@ -722,16 +945,16 @@ BOOL ContactMetaSave (Contact *cont)
     ContactMeta *m;
     FILE *f;
     
-    if (!(f = fopen (s_sprintf ("%scontacts" _OS_PATHSEPSTR "icq-%ld", PrefUserDir (prG), cont->uin), "w")))
+    if (!(f = fopen (s_sprintf ("%scontacts" _OS_PATHSEPSTR "icq-%s", PrefUserDir (prG), cont->screen), "w")))
     {
         mkdir (s_sprintf ("%scontacts", PrefUserDir (prG)), 0700);
-        if (!(f = fopen (s_sprintf ("%scontacts" _OS_PATHSEPSTR "icq-%ld", PrefUserDir (prG), cont->uin), "w")))
+        if (!(f = fopen (s_sprintf ("%scontacts" _OS_PATHSEPSTR "icq-%s", PrefUserDir (prG), cont->screen), "w")))
             return FALSE;
     }
-    fprintf (f, "#\n# Meta data for contact %ld.\n#\n\n", cont->uin);
+    fprintf (f, "#\n# Meta data for contact %s.\n#\n\n", cont->screen);
     fprintf (f, "encoding UTF-8\n"); 
     fprintf (f, "format 1\n\n");
-    fprintf (f, "b_uin      %ld\n", cont->uin);
+    fprintf (f, "b_uin      %s\n", cont->screen);
     fprintf (f, "b_nick     %s\n", s_quote (cont->nick));
     if (cont->meta_about)
         fprintf (f, "b_about    %s\n", s_quote (cont->meta_about));
@@ -752,8 +975,7 @@ BOOL ContactMetaSave (Contact *cont)
         fprintf (f, "g_country  %u\n", mg->country);
         fprintf (f, "g_tz       %d\n", mg->tz);
         fprintf (f, "g_flags    %u\n", (mg->auth ? 1 : 0)
-                                    + (mg->webaware ? 2 : 0)
-                                    + (mg->hideip ? 4 : 0));
+                                    + (mg->webaware ? 2 : 0));
     }
     if (cont->meta_work)
     {
@@ -844,7 +1066,7 @@ BOOL ContactMetaLoad (Contact *cont)
     const char *line;
     UDWORD i;
     
-    if (!(f = fopen (s_sprintf ("%scontacts" _OS_PATHSEPSTR "icq-%ld", PrefUserDir (prG), cont->uin), "r")))
+    if (!(f = fopen (s_sprintf ("%scontacts" _OS_PATHSEPSTR "icq-%s", PrefUserDir (prG), cont->screen), "r")))
         return FALSE;
 
     cont->updated = 0;
@@ -909,7 +1131,6 @@ BOOL ContactMetaLoad (Contact *cont)
             {
                 mg->auth     = i & 1 ? 1 : 0;
                 mg->webaware = i & 2 ? 1 : 0;
-                mg->hideip   = i & 4 ? 1 : 0;
             }}
         }
         else if (!strncmp (cmd, "w_", 2))
@@ -1008,14 +1229,23 @@ val_t ContactGroupPrefVal (ContactGroup *cg, UDWORD flag)
 /*
  * Query an option for a contact.
  */
-val_t ContactPrefVal (Contact *cont, UDWORD flag)
+val_t ContactPrefVal (Contact *ocont, UDWORD flag)
 {
+    Contact *cont;
     val_t res = 0;
     
-    if (cont)
-    {
+#if ENABLE_CONT_HIER
+    for (cont = ocont; cont; cont = cont->parent)
+#else
+    for (cont = ocont; cont; cont = NULL)
+#endif
         if (OptGetVal (&cont->copts, flag, &res))
             return res;
+#if ENABLE_CONT_HIER
+    for (cont = ocont; cont; cont = cont->parent)
+#else
+    for (cont = ocont; cont; cont = NULL)
+#endif
         if (cont->group)
         {
             if (OptGetVal (&cont->group->copts, flag, &res))
@@ -1023,7 +1253,6 @@ val_t ContactPrefVal (Contact *cont, UDWORD flag)
             if (cont->group->serv && OptGetVal (&cont->group->serv->contacts->copts, flag, &res))
                 return res;
         }
-    }
     if (OptGetVal (&prG->copts, flag, &res))
         return res;
     return 0;
@@ -1032,14 +1261,23 @@ val_t ContactPrefVal (Contact *cont, UDWORD flag)
 /*
  * Query a string option for a contact.
  */
-const char *ContactPrefStr (Contact *cont, UDWORD flag)
+const char *ContactPrefStr (Contact *ocont, UDWORD flag)
 {
+    Contact *cont;
     const char *res = NULL;
-    
-    if (cont)
-    {
+
+#if ENABLE_CONT_HIER
+    for (cont = ocont; cont; cont = cont->parent)
+#else
+    for (cont = ocont; cont; cont = NULL)
+#endif
         if (OptGetStr (&cont->copts, flag, &res))
             return res;
+#if ENABLE_CONT_HIER
+    for (cont = ocont; cont; cont = cont->parent)
+#else
+    for (cont = ocont; cont; cont = NULL)
+#endif
         if (cont->group)
         {
             if (OptGetStr (&cont->group->copts, flag, &res))
@@ -1047,15 +1285,22 @@ const char *ContactPrefStr (Contact *cont, UDWORD flag)
             if (cont->group->serv && OptGetStr (&cont->group->serv->contacts->copts, flag, &res))
                 return res;
         }
-    }
     if (OptGetStr (&prG->copts, flag, &res))
         return res;
     if (~flag & COF_COLOR || flag == CO_COLORNONE)
         return "";
-    if (cont)
-    {
+#if ENABLE_CONT_HIER
+    for (cont = ocont; cont; cont = cont->parent)
+#else
+    for (cont = ocont; cont; cont = NULL)
+#endif
         if (OptGetStr (&cont->copts, CO_COLORNONE, &res))
             return res;
+#if ENABLE_CONT_HIER
+    for (cont = ocont; cont; cont = cont->parent)
+#else
+    for (cont = ocont; cont; cont = NULL)
+#endif
         if (cont->group)
         {
             if (OptGetStr (&cont->group->copts, CO_COLORNONE, &res))
@@ -1063,7 +1308,6 @@ const char *ContactPrefStr (Contact *cont, UDWORD flag)
             if (cont->group->serv && OptGetStr (&cont->group->serv->contacts->copts, CO_COLORNONE, &res))
                 return res;
         }
-    }
     if (OptGetStr (&prG->copts, CO_COLORNONE, &res))
         return res;
     return "";
@@ -1411,6 +1655,48 @@ void ContactSetVersion (Contact *cont)
     }
     else
         buf[0] = '\0';
-
     s_repl (&cont->version, strlen (buf) ? buf : NULL);
 }
+
+BOOL ContactStatus (const char **args, status_t *stat)
+{
+    if      (s_parsekey (args, "inv"))        *stat = ims_inv;
+    else if (s_parsekey (args, "inv online")) *stat = ims_inv;
+    else if (s_parsekey (args, "inv ffc"))    *stat = ims_inv_ffc;
+    else if (s_parsekey (args, "inv away"))   *stat = ims_inv_away;
+    else if (s_parsekey (args, "inv na"))     *stat = ims_inv_na;
+    else if (s_parsekey (args, "inv occ"))    *stat = ims_inv_occ;
+    else if (s_parsekey (args, "inv dnd"))    *stat = ims_inv_dnd;
+    else if (s_parsekey (args, "online"))     *stat = ims_online;
+    else if (s_parsekey (args, "ffc"))        *stat = ims_ffc;
+    else if (s_parsekey (args, "away"))       *stat = ims_away;
+    else if (s_parsekey (args, "na"))         *stat = ims_na;
+    else if (s_parsekey (args, "occ"))        *stat = ims_occ;
+    else if (s_parsekey (args, "dnd"))        *stat = ims_dnd;
+    else if (s_parsekey (args, "offline"))    *stat = ims_offline;
+    else if (s_parsekey (args, "online"))     *stat = ims_online;
+    else
+        return FALSE;
+    return TRUE;
+}
+
+const char *ContactStatusStr (status_t status)
+{
+    switch (status)
+    {
+        case ims_offline:   return "offline";
+        case ims_online:    return "online";
+        case ims_ffc:       return "ffc";
+        case ims_away:      return "away";
+        case ims_na:        return "na";
+        case ims_occ:       return "occ";
+        case ims_dnd:       return "dnd";
+        case ims_inv:       return "inv online";
+        case ims_inv_ffc:   return "inv ffc";
+        case ims_inv_away:  return "inv away";
+        case ims_inv_na:    return "inv na";
+        case ims_inv_occ:   return "inv occ";
+        case ims_inv_dnd:   return "inv dnd";
+    }
+    assert (0);
+}
