@@ -1,4 +1,4 @@
-/* $Id: file_util.c,v 1.215 2005/12/22 16:33:16 kuhlmann Exp $ */
+/* $Id: file_util.c,v 1.234 2007-04-17 22:03:26 kuhlmann Exp $ */
 /* Copyright: This file may be distributed under version 2 of the GPL licence. */
 
 #include "micq.h"
@@ -41,115 +41,207 @@
 #include "util_io.h"
 #include "remote.h"
 #include "oscar_base.h"
+#include "oldicq_compat.h"
 #include "connection.h"
 #include "util_parse.h"
 #include "util_rl.h"
 #include "msn_base.h"
+#include "jabber_base.h"
 
 /****/
+#define USER_PROMPT "%4c%b%p%d://%1c%b%n%3c/%2c%b%s%8c %t%7c%b>%6c%r%7c%b<%6c%b%a%7c%b>"
+/*#define USER_PROMPT "%n@%p:%s %t>%r<%a>"*/
+#define USER_PROMPT_STRFTIME "%c"
 
 #define WAND1 " \xb7 \xb7 "
 #define WAND2 "\xb7 o \xb7"
 #define WAND3 " \xb7 \\ "
 #define WAND4 "    \\"
 
-Connection *PrefNewConnection (UDWORD uin, const char *passwd)
+Connection *PrefNewConnection (UDWORD servertype, const char *user, const char *passwd)
 {
-    Connection *conn;
+    Connection *conn = NULL;
     Contact *cont;
     
-    conn = ConnectionC (TYPE_SERVER);
-    conn->open = &ConnectionInitServer;
-    
-    conn->flags |= CONN_AUTOLOGIN;
-    conn->pref_server = strdup ("login.icq.com");
-    conn->pref_port = 5190;
-    conn->pref_status = STATUS_ICQONLINE;
-    conn->version = 8;
-    conn->uin = uin;
-    s_repl (&conn->screen, s_sprintf ("%lu", uin));
-#ifdef __BEOS__
-    conn->pref_passwd = passwd ? strdup (passwd) : NULL;
-#endif
-    
-    conn->server  = strdup ("login.icq.com");
-    conn->port    = 5190;
-    conn->passwd  = passwd ? strdup (passwd) : NULL;
-    conn->status = STATUS_ICQONLINE;
+    if (servertype == TYPE_SERVER)
+    {
+        conn = ConnectionC (servertype);
+        conn->open = &ConnectionInitServer;
+        
+        conn->flags |= CONN_AUTOLOGIN;
+        conn->pref_server = strdup ("login.icq.com");
+        conn->pref_port = 5190;
+        conn->pref_status = ims_online;
+        conn->version = 8;
+        s_repl (&conn->screen, user);
+        conn->uin = atoi (user);
+        conn->pref_passwd = passwd ? strdup (passwd) : NULL;
+        
+        conn->server  = strdup (conn->pref_server);
+        conn->port    = conn->pref_port;
+        conn->passwd  = passwd ? strdup (passwd) : NULL;
+        conn->status  = ims_online;
 
-    conn->contacts = ContactGroupC (conn, 0, s_sprintf ("contacts-icq8-%ld", uin));
-    OptSetVal (&conn->contacts->copts, CO_IGNORE, 0);
-    cont = ContactFindCreate (conn->contacts, 82274703, "mICQ");
-    ContactFindCreate (conn->contacts, 82274703, "Tadu");
-    OptSetStr (&cont->copts, CO_COLORINCOMING, OptC2S ("red bold"));
-    OptSetStr (&cont->copts, CO_COLORMESSAGE, OptC2S ("red bold"));
-    rl_printf (i18n (2381, "I'll add the author of mICQ to your contact list for your convenience. Don't abuse this opportunity - please use the help command and make a serious attempt to read the man pages and the FAQ before asking questions.\n"));
+        conn->contacts = ContactGroupC (conn, 0, s_sprintf ("contacts-icq8-%s", user));
+        OptSetVal (&conn->contacts->copts, CO_IGNORE, 0);
+        
+        cont = ContactUIN (conn, 82274703);
+        ContactCreate (conn, cont);
+        ContactAddAlias (cont, "mICQ");
+        ContactAddAlias (cont, "Tadu");
+        OptSetStr (&cont->copts, CO_COLORINCOMING, OptC2S ("red bold"));
+        OptSetStr (&cont->copts, CO_COLORMESSAGE, OptC2S ("red bold"));
+        rl_print ("\n");
+        rl_printf (i18n (2381, "I'll add the author of mICQ to your contact list for your convenience. Don't abuse this opportunity - please use the help command and make a serious attempt to read the man pages and the FAQ before asking questions.\n"));
+        rl_print ("\n");
+    }
+#ifdef ENABLE_XMPP
+    else if (servertype == TYPE_XMPP_SERVER)
+    {
+        const char *serverpart = strchr (user, '@') + 1;
+    
+        conn = ConnectionC (servertype);
+        conn->open = &ConnectionInitXMPPServer;
+        
+        conn->flags |= CONN_AUTOLOGIN;
+        if (!strcmp (serverpart, "gmail.com") || !strcmp (serverpart, "gmail.com"))
+            conn->pref_server = strdup ("talk.google.com");
+        else
+            conn->pref_server = strdup (serverpart);
+        conn->pref_port = 5222;
+        conn->pref_status = ims_online;
+        conn->version = 8;
+        s_repl (&conn->screen, user);
+        conn->uin = 0;
+        conn->pref_passwd = passwd ? strdup (passwd) : NULL;
+        
+        conn->server  = strdup (conn->pref_server);
+        conn->port    = conn->pref_port;
+        conn->passwd  = passwd ? strdup (passwd) : NULL;
+        conn->status  = ims_online;
+
+        conn->contacts = ContactGroupC (conn, 0, s_sprintf ("contacts-xmpp-%s", user));
+        OptSetVal (&conn->contacts->copts, CO_IGNORE, 0);
+        
+        cont = ContactScreen (conn, "RKuhlmann@gmail.com");
+        ContactCreate (conn, cont);
+        ContactAddAlias (cont, "mICQ");
+        ContactAddAlias (cont, "Tadu");
+        OptSetStr (&cont->copts, CO_COLORINCOMING, OptC2S ("red bold"));
+        OptSetStr (&cont->copts, CO_COLORMESSAGE, OptC2S ("red bold"));
+        rl_print ("\n");
+        rl_printf (i18n (2381, "I'll add the author of mICQ to your contact list for your convenience. Don't abuse this opportunity - please use the help command and make a serious attempt to read the man pages and the FAQ before asking questions.\n"));
+        rl_print ("\n");
+    }
+#endif
+#ifdef ENABLE_MSN
+    else if (servertype == TYPE_MSN_SERVER)
+    {
+        conn = ConnectionC (servertype);
+        conn->open = &ConnectionInitMSNServer;
+        
+        conn->flags |= CONN_AUTOLOGIN;
+        conn->pref_server = strdup (strchr (user, '@') + 1);
+        conn->pref_port = 42;
+        conn->pref_status = ims_online;
+        conn->version = 8;
+        s_repl (&conn->screen, user);
+        conn->uin = 0;
+        conn->pref_passwd = passwd ? strdup (passwd) : NULL;
+        
+        conn->server  = strdup (conn->pref_server);
+        conn->port    = conn->pref_port;
+        conn->passwd  = passwd ? strdup (passwd) : NULL;
+        conn->status  = ims_online;
+
+        conn->contacts = ContactGroupC (conn, 0, s_sprintf ("contacts-msn-%s", user));
+        OptSetVal (&conn->contacts->copts, CO_IGNORE, 0);
+    }
+#endif
     return conn;
 }
 
 void Initialize_RC_File ()
 {
     strc_t line;
-    char *pwd;
     Connection *conn;
 #ifdef ENABLE_REMOTECONTROL
     Connection *conns;
 #endif
-    Contact *cont;
+    char *user = NULL;
     char *passwd;
     char *t;
-    UDWORD uin;
-    long tmpuin;
+    UDWORD servertype = 0;
     
     prG->away_time = default_away_time;
 
     rl_print ("\n");
-    rl_print (i18n (1793, "No valid UIN found. The setup wizard will guide you through the process of setting one up.\n"));
-    rl_print (i18n (1794, "If you already have an UIN, please enter it. Otherwise, enter 0, and I will request one for you.\n"));
-    rl_printf ("%s ", i18n (1618, "UIN:"));
-    fflush (stdout);
-    ReadLineTtyUnset ();
-    line = UtilIOReadline (stdin);
-    ReadLineTtySet ();
-    tmpuin = 0;
-    if (line)
-        sscanf (line->txt, "%ld", &tmpuin);
-    uin = tmpuin;
-
-    rl_print ("\n");
-    if (uin)
-        rl_printf (i18n (1781, "Your password for UIN %ld:\n"), uin);
-    else
-        rl_print (i18n (1782, "You need a password for your new UIN.\n"));
+    rl_print (i18n (2612, "No valid user account found. The setup wizard will guide you through the process of setting one up.\n"));
+    rl_print (i18n (2613, "You first need to enter a user account you want to use. This mICQ supports the following chat protocols:\n"));
+    rl_print ("  *  ");
+    rl_printf (i18n (2614, "%s, enter i.e. %s\n"), "ICQ", "12345678");
+#ifdef ENABLE_XMPP
+    rl_print ("  *  ");
+    rl_printf (i18n (2614, "%s, enter i.e. %s\n"), "XMPP (Jabber, Google Talk)", "example@gmail.com");
+#endif
+#ifdef ENABLE_MSN
+    rl_print ("  *  ");
+    rl_printf (i18n (2614, "%s, enter i.e. %s\n"), "MSN", "fool@hotmale.net");
+#endif
     while (1)
     {
-        rl_printf ("%s ", i18n (1795, "Password:"));
-        fflush (stdout);
-        line = UtilIOReadline (stdin);
-        if (!line)
-            continue;
-        rl_print ("\n");
-        if (uin)
+        servertype = 0;
+        while (!servertype)
+        {
+            rl_printf ("%s ", i18n (2615, "User account ID:"));
+            fflush (stdout);
+            ReadLineTtyUnset ();
+            line = UtilIOReadline (stdin);
+            ReadLineTtySet ();
+            
+            if (!line || !line->len)
+            {
+                if (!user)
+                    continue;
+                break;
+            }
+
+#ifdef ENABLE_XMPP
+            if (is_valid_xmpp_name (line->txt))
+                servertype = TYPE_XMPP_SERVER;
+            else
+#endif
+            if (is_valid_icq_name (line->txt))
+                servertype = TYPE_SERVER;
+            else
+#ifdef ENABLE_MSN
+            if (is_valid_msn_name (line->txt))
+                servertype = TYPE_MSN_SERVER;
+            else
+#endif
+                rl_printf (i18n (2616, "Cannot parse %s as a valid user ID. Try again!\n"), line->txt);
+        }
+        if (user && !servertype)
             break;
 
-        rl_print (i18n (1783, "To prevent typos, please enter your password again.\n"));
+        user = strdup (line->txt);
+        
+        rl_print ("\n");
+        rl_printf (i18n (2617, "Next you can allow mICQ to store your password for %s account %s. Enter nothing to not save the password.\n"),
+            ConnectionServerType (servertype), user);
+        
         rl_printf ("%s ", i18n (1795, "Password:"));
         fflush (stdout);
-        pwd = strdup (line->txt);
         line = UtilIOReadline (stdin);
-        if (!line)
-            continue;
         rl_print ("\n");
-        if (strcmp (pwd, line->txt))
-        {
-            rl_printf ("\n%s\n", i18n (1093, "Passwords did not match - please try again."));
-            free (pwd);
-            continue;
-        }
-        free (pwd);
-        break;
+
+        passwd = line && line->len ? strdup (c_out (line->txt)) : NULL;
+        conn = PrefNewConnection (servertype, user, passwd);
+        conn->flags |= CONN_WIZARD;
+        free (user);
+        s_free (passwd);
+        rl_print (i18n (2618, "You may add more user accounts now. Enter nothing to not add more accounts.\n"));
     }
-    passwd = strdup (c_out (line->txt));
 
     prG->s5Use = 0;
     prG->s5Port = 0;
@@ -206,23 +298,7 @@ void Initialize_RC_File ()
     rl_print ("\n");
     ReadLineTtySet ();
 
-    if (!uin)
-    {
-        rl_print (i18n (1796, "Setup wizard finished. Please wait until registration has finished.\n"));
-        conn = SrvRegisterUIN (NULL, passwd);
-        conn->open = &ConnectionInitServer;
-        conn->contacts = ContactGroupC (conn, 0, s_sprintf ("contacts-icq8-%ld", uin));
-        OptSetVal (&conn->contacts->copts, CO_IGNORE, 0);
-        cont = ContactFindCreate (conn->contacts, 82274703, "mICQ");
-        ContactFindCreate (conn->contacts, 82274703, "Tadu");
-        OptSetStr (&cont->copts, CO_COLORINCOMING, OptC2S ("red bold"));
-        OptSetStr (&cont->copts, CO_COLORMESSAGE, OptC2S ("red bold"));
-    }
-    else
-    {
-        conn = PrefNewConnection (uin, passwd);
-        rl_print (i18n (1791, "Setup wizard finished. Congratulations!\n"));
-    }
+    rl_print (i18n (1791, "Setup wizard finished. Congratulations!\n"));
     rl_print ("\n");
     
     {
@@ -232,7 +308,6 @@ void Initialize_RC_File ()
         mkdir (tmp, 0700);
         free (tmp);
     }
-    conn->flags |= CONN_WIZARD;
 #ifdef ENABLE_REMOTECONTROL
     conns = ConnectionC (TYPE_REMOTE);
     conns->open = &RemoteOpen;
@@ -241,10 +316,12 @@ void Initialize_RC_File ()
     conns->server = strdup (conns->pref_server);
 #endif
 
+    prG->prompt    = strdup (USER_PROMPT);
+    prG->prompt_strftime = strdup (USER_PROMPT_STRFTIME);
     prG->logplace  = strdup ("history" _OS_PATHSEPSTR);
     prG->chat      = 49;
     prG->autoupdate = AUTOUPDATE_CURRENT;
-    prG->status = STATUS_ICQONLINE;
+    prG->status = ims_online;
     prG->flags = FLAG_DELBS | FLAG_AUTOSAVE;
 #ifdef ANSI_TERM
     prG->flags |= FLAG_COLOR;
@@ -266,10 +343,7 @@ void Initialize_RC_File ()
     OptSetStr (&prG->copts, CO_AUTOOCC,  i18n (1012, "User is occupied [Auto-Message]"));
     OptSetStr (&prG->copts, CO_AUTOFFC,  i18n (2055, "User is ffc and wants to chat about everything."));
 
-    if (uin)
-        Save_RC ();
-    
-    free (passwd);
+    Save_RC ();
 }
 
 #define PrefParse(x)          switch (1) { case 1: if (!(par = s_parse (&args))) { rl_printf (i18n (2123, "%sSyntax error%s: Too few arguments: %s\n"), COLERROR, COLNONE, s_qquote (line->txt)); continue; } x = par->txt; }
@@ -510,6 +584,11 @@ int Read_RC_File (FILE *rcf)
                     PrefParseInt (i);
                     prG->chat = i;
                 }
+                else if (!strcasecmp (cmd, "prompt_strftime"))
+                {
+                    PrefParse (tmp);
+                    prG->prompt_strftime = strdup (tmp);
+                }
                 else if (!strcasecmp (cmd, "autoupdate"))
                 {
                     PrefParseInt (i);
@@ -657,6 +736,8 @@ int Read_RC_File (FILE *rcf)
                         which = FLAG_AUTOREPLY;
                     else if (!strcasecmp (cmd, "uinprompt"))
                         which = FLAG_UINPROMPT;
+                    else if (!strcasecmp (cmd, "prompt"))
+                        which = -9;
                     else if (!strcasecmp (cmd, "autosave"))
                         which = FLAG_AUTOSAVE;
                     else if (!strcasecmp (cmd, "autofinger"))
@@ -790,6 +871,20 @@ int Read_RC_File (FILE *rcf)
                         OptSetVal (&prG->copts, CO_DCCONT, 1);
                         dep = 8723;
                     }
+                    else if (which == -9)
+                    {
+                        PrefParse (cmd);
+
+                        prG->flags &= ~FLAG_USERPROMPT & ~FLAG_UINPROMPT;
+                        if (!strcasecmp (cmd, "user"))
+                            prG->flags |= FLAG_USERPROMPT;
+                        else if (!strcasecmp (cmd, "uin"))
+                            prG->flags |= FLAG_UINPROMPT;
+                        else if (!strcasecmp (cmd, "simple"))
+                            ;
+                        else
+                            ERROR;
+                    }
                 }
                 else if (!strcasecmp (cmd, "options"))
                 {
@@ -860,7 +955,7 @@ int Read_RC_File (FILE *rcf)
                     if (~tconn->type & TYPEF_ANY_SERVER)
                         continue;
 
-                    if ((cont = ContactFind (tconn->contacts, 0, uin, NULL)))
+                    if ((cont = ContactFindUIN (tconn->contacts, uin)))
                     {
                         j = 1;
                         ContactAddAlias (cont, cmd);
@@ -875,13 +970,15 @@ int Read_RC_File (FILE *rcf)
                             break;
                     if (!tconn)
                         break;
-                    if (!(cont = ContactFindCreate (tconn->contacts, uin, cmd)))
+                    if (!(cont = ContactFindUIN (tconn->contacts, uin)))
                     {
                         rl_printf ("%s%s%s %s\n", COLERROR, i18n (1619, "Warning:"), COLNONE,
-                                 i18n (1620, "maximal number of contacts reached. Ask a wizard to enlarge me!"));
+                                   i18n (2118, "Out of memory.\n"));
                         section = -1;
                         break;
                     }
+                    ContactCreate (tconn, cont);
+                    ContactAddAlias (cont, cmd);
                     OptSetVal (&cont->copts, flags, 1); /* FIXME */
                 }
                 break;
@@ -919,6 +1016,11 @@ int Read_RC_File (FILE *rcf)
                     free (tmp);
                     free (tmp2);
                 }
+                else if (!strcasecmp (cmd, "prompt"))
+                {
+                    PrefParse (cmd);
+                    prG->prompt = strdup (cmd);
+                }
                 else
                 {
                     rl_printf ("%s%s%s ", COLERROR, i18n (1619, "Warning:"), COLNONE);
@@ -940,15 +1042,11 @@ int Read_RC_File (FILE *rcf)
                                ? TYPE_SERVER : TYPE_SERVER_OLD) : conn->type);
                         dep = 20;
                     }
-                    else if (!strcasecmp (cmd, "icq8"))
-                        conn->type = TYPE_SERVER;
-                    else if (!strcasecmp (cmd, "icq5"))
-                        conn->type = TYPE_SERVER_OLD;
-                    else if (!strcasecmp (cmd, "msn"))
-                        conn->type = TYPE_MSN_SERVER;
-                    else if (!strcasecmp (cmd, "peer"))
+                    else if (!(conn->type = ConnectionServerNType (cmd, 0)))
+                        ERROR;
+                    
+                    if (conn->type == TYPE_MSGLISTEN)
                     {
-                        conn->type = TYPE_MSGLISTEN;
                         conn->pref_status = TCP_OK_FLAG;
                         if (oldconn->type == TYPE_SERVER || oldconn->type == TYPE_SERVER_OLD)
                         {
@@ -963,10 +1061,6 @@ int Read_RC_File (FILE *rcf)
                             section = -1;
                         }
                     }
-                    else if (!strcasecmp (cmd, "remote"))
-                        conn->type = TYPE_REMOTE;
-                    else
-                        ERROR;
 
                     if ((par = s_parse (&args)))
                     {
@@ -1002,11 +1096,13 @@ int Read_RC_File (FILE *rcf)
                     PrefParseInt (i);
                     conn->uin = i;
                     s_repl (&conn->screen, s_sprintf ("%lu", i));
+                    dep = 77;
                 }
                 else if (!strcasecmp (cmd, "screen"))
                 {
                     PrefParse (tmp);
                     s_repl (&conn->screen, tmp);
+                    conn->uin = IcqIsUIN (tmp);
                 }
                 else if (!strcasecmp (cmd, "password"))
                 {
@@ -1015,8 +1111,11 @@ int Read_RC_File (FILE *rcf)
                 }
                 else if (!strcasecmp (cmd, "status"))
                 {
-                    PrefParseInt (i);
-                    conn->pref_status = i;
+                    if (!ContactStatus (&args, &conn->pref_status))
+                    {
+                        PrefParseInt (i);
+                        conn->pref_status = IcqToStatus (i);
+                    }
                 }
                 else
                 {
@@ -1032,23 +1131,17 @@ int Read_RC_File (FILE *rcf)
                 PrefParse (cmd);
                 if (!strcasecmp (cmd, "label") && !cg->used)
                 {
-                    Connection *conn;
+                    Connection *conn = NULL;
                     PrefParse (tmp);
                     s_repl (&cg->name, tmp);
                     if (!strncmp (cg->name, "contacts-", 9))
                     {
-                        UWORD type = 0;
-                        UDWORD uin = 0;
-                        
-                        if      (!strncmp (cg->name + 9, "icq5-", 5)) type = TYPE_SERVER_OLD;
-                        else if (!strncmp (cg->name + 9, "icq8-", 5)) type = TYPE_SERVER;
-                        else if (!strncmp (cg->name + 9, "msn-", 4))  type = TYPE_MSN_SERVER;
-                        
-                        if (type)
+                        UWORD type;
+                        if ((type = ConnectionServerNType (cg->name + 9, '-')))
                         {
-                            uin = atoi (cg->name + 14);
-                        
-                            if ((conn = ConnectionFindUIN (type, uin)))
+                            int len = strlen (ConnectionServerType (type));
+                            conn = ConnectionFindScreen (type, cg->name + len + 10);
+                            if (conn)
                             {
                                 if (conn->contacts && conn->contacts != cg) /* Duplicate! */
                                 {
@@ -1075,19 +1168,15 @@ int Read_RC_File (FILE *rcf)
                 else if (!strcasecmp (cmd, "server") && !cg->used)
                 {
                     Connection *conn;
-                    UWORD type = 0;
-                    UDWORD uin = 0;
+                    UWORD type;
                     
                     PrefParse (tmp);
-                    if      (!strcasecmp (tmp, "icq5")) type = TYPE_SERVER_OLD;
-                    else if (!strcasecmp (tmp, "icq8")) type = TYPE_SERVER;
-                    else if (!strcasecmp (tmp, "msn"))  type = TYPE_MSN_SERVER;
-                    else
+                    if (!(type = ConnectionServerNType (tmp, 0)))
                         ERROR;
                     
-                    PrefParseInt (uin);
+                    PrefParse (tmp);
                     
-                    if ((conn = ConnectionFindUIN (type, uin)))
+                    if ((conn = ConnectionFindScreen (type, tmp)))
                         cg->serv = conn;
                 }
                 else if (!cg->serv)
@@ -1103,7 +1192,9 @@ int Read_RC_File (FILE *rcf)
                     PrefParseInt (i);
                     PrefParseInt (uin);
                     
-                    cont = ContactFindCreate (cg->serv->contacts, uin, s_sprintf ("%ld", uin));
+                    if (!(cont = ContactFindUIN (cg->serv->contacts, uin)))
+                        break;
+                    ContactCreate (cg->serv, cont);
                     if (cont && cg != cg->serv->contacts)
                         ContactAdd (cg, cont);
                 }
@@ -1123,6 +1214,12 @@ int Read_RC_File (FILE *rcf)
     if (!prG->chat)
         prG->chat = 49;
 
+    if (!prG->prompt)
+        prG->prompt = strdup (USER_PROMPT);
+
+    if (!prG->prompt_strftime)
+        prG->prompt_strftime = strdup (USER_PROMPT_STRFTIME);
+
     for (i = 0; (conn = ConnectionNr (i)); i++)
     {
         conn->port   = conn->pref_port;
@@ -1144,6 +1241,11 @@ int Read_RC_File (FILE *rcf)
                 conn->open = &ConnectionInitMSNServer;
                 break;
 #endif
+#ifdef ENABLE_XMPP
+            case TYPE_XMPP_SERVER:
+                conn->open = &ConnectionInitXMPPServer;
+                break;
+#endif
 #ifdef ENABLE_PEER2PEER
             case TYPE_MSGLISTEN:
                 conn->open = &ConnectionInitPeer;
@@ -1162,9 +1264,9 @@ int Read_RC_File (FILE *rcf)
         }
         if (format < 2 && !conn->contacts && conn->type & TYPEF_SERVER)
         {
-            conn->contacts = cg = ContactGroupC (conn, 0, s_sprintf ("contacts-%s-%ld", ConnectionServerType (conn), conn->uin));
+            conn->contacts = cg = ContactGroupC (conn, 0, s_sprintf ("contacts-%s-%s", ConnectionServerType (conn->type), conn->screen));
             for (i = 0; (cont = ContactIndex (NULL, i)); i++)
-                ContactFindCreate (cg, cont->uin, cont->nick);
+                ContactCreate (conn, cont);
             dep = 21;
         }
     }
@@ -1254,7 +1356,7 @@ void PrefReadStat (FILE *stf)
                 {
                     PrefParseInt (i);
                     format = i;
-                    if (format > 2)
+                    if (format > 3)
                     {
                         rl_printf ("%s%s%s ", COLERROR, i18n (1619, "Warning:"), COLNONE);
                         rl_printf (i18n (2457, "Unknown configuration file format version %d.\n"), format);
@@ -1276,17 +1378,14 @@ void PrefReadStat (FILE *stf)
                     s_repl (&cg->name, cmd);
                     if (!strncmp (cg->name, "contacts-", 9))
                     {
-                        UWORD type = 0;
-                        UDWORD uin = 0;
+                        UWORD type;
+                        int len;
                         
-                        if      (!strncmp (cg->name + 9, "icq5-", 5)) type = TYPE_SERVER_OLD;
-                        else if (!strncmp (cg->name + 9, "icq8-", 5)) type = TYPE_SERVER;
-                        else if (!strncmp (cg->name + 9, "msn-", 4))  type = TYPE_MSN_SERVER;
-                        else
+                        if (!(type = ConnectionServerNType (cg->name + 9, '-')))
                             break;
-                        
-                        uin = atoi (cg->name + 14);
-                        if ((conn = ConnectionFindUIN (type, uin)))
+
+                        len = strlen (ConnectionServerType (type));
+                        if ((conn = ConnectionFindScreen (type, cg->name + 10 + len)))
                         {
                             if (conn->contacts && conn->contacts != cg)
                             {
@@ -1310,18 +1409,14 @@ void PrefReadStat (FILE *stf)
                 }
                 else if (!strcasecmp (cmd, "server") && !cg->used)
                 {
-                    UWORD type = 0;
-                    UDWORD uin = 0;
+                    UWORD type;
                     
                     PrefParse (cmd);
-                    if      (!strcasecmp (cmd, "icq5")) type = TYPE_SERVER_OLD;
-                    else if (!strcasecmp (cmd, "icq8")) type = TYPE_SERVER;
-                    else if (!strcasecmp (cmd, "msn"))  type = TYPE_MSN_SERVER;
-                    else
+                    if (!(type = ConnectionServerNType (cmd, 0)))
                         ERROR;
                     
-                    PrefParseInt (uin);
-                    if ((conn = ConnectionFindUIN (type, uin)))
+                    PrefParse (cmd);
+                    if ((conn = ConnectionFindScreen (type, cmd)))
                         cg->serv = conn;
                 }
                 else if (!strcasecmp (cmd, "options"))
@@ -1331,28 +1426,29 @@ void PrefReadStat (FILE *stf)
                 }
                 else if (cg->serv && !strcasecmp (cmd, "entry"))
                 {
-                    UDWORD uin;
-                    
-                    PrefParseInt (i);
-                    PrefParseInt (uin);
+                    if (format < 3)
+                        PrefParseInt (i);
+                    PrefParse (cmd);
                     
                     if (!cg->serv->contacts)
                     {
-                        cg->serv->contacts = ContactGroupC (NULL, 0, s_sprintf ("contacts-icq8-%ld", uin));
+                        cg->serv->contacts = ContactGroupC (NULL, 0, s_sprintf ("contacts-%s-%s", ConnectionServerType (cg->serv->type), cmd));
                         OptSetVal (&cg->serv->contacts->copts, CO_IGNORE, 0);
                         cg->serv->contacts->serv = cg->serv;
                     }
                     
                     if (format == 1)
-                        cont = ContactFindCreate (cg->serv->contacts, uin, s_sprintf ("%ld", uin));
+                        cont = ContactFindUIN (cg->serv->contacts, atol (cmd));
                     else
-                        cont = ContactFind (cg->serv->contacts, 0, uin, NULL);
+                        cont = ContactScreen (cg->serv, cmd);
                     
                     if (!cont)
                     {
-                        rl_printf ("FIXME: could not find %ld in %p / fmt %d).\n", uin, cg->serv->contacts, format);
+                        rl_printf ("%s%s%s %s\n", COLERROR, i18n (1619, "Warning:"), COLNONE,
+                                   i18n (2118, "Out of memory.\n"));
                         ERROR;
                     }
+                    ContactCreate (cg->serv, cont);
 
                     if (cg != cg->serv->contacts)
                     {
@@ -1383,7 +1479,7 @@ void PrefReadStat (FILE *stf)
                     for (i = uinconts = 0; (conn = ConnectionNr (i)); i++)
                     {
                         if (conn->type & TYPEF_ANY_SERVER && conn->contacts && uinconts < 20
-                            && (cont = ContactFind (conn->contacts, 0, uin, NULL)))
+                            && (cont = ContactFindUIN (conn->contacts, uin)))
                         {
                             uincont[uinconts++] = cont;
                             ContactAddAlias (cont, cmd);
@@ -1411,23 +1507,19 @@ void PrefReadStat (FILE *stf)
                 PrefParse (cmd);
                 if (!strcasecmp (cmd, "server"))
                 {
-                    UWORD type = 0;
-                    UDWORD uin = 0;
+                    UWORD type;
                     
                     PrefParse (cmd);
-                    if      (!strcasecmp (cmd, "icq5")) type = TYPE_SERVER_OLD;
-                    else if (!strcasecmp (cmd, "icq8")) type = TYPE_SERVER;
-                    else if (!strcasecmp (cmd, "msn"))  type = TYPE_MSN_SERVER;
-                    else
+                    if (!(type = ConnectionServerNType (cmd, 0)))
+                        ERROR;
+
+                    PrefParse (cmd);
+                    if (!(conn = ConnectionFindScreen (type, cmd)))
                         ERROR;
-                    
-                    PrefParseInt (uin);
                     cont = NULL;
-                    if (!(conn = ConnectionFindUIN (type, uin)))
-                        ERROR;
                     if (!conn->contacts)
                     {
-                        conn->contacts = ContactGroupC (conn, 0, s_sprintf ("contacts-%s-%ld", ConnectionServerType (conn), conn->uin));
+                        conn->contacts = ContactGroupC (conn, 0, s_sprintf ("contacts-%s-%s", ConnectionServerType (conn->type), conn->screen));
                         OptSetVal (&conn->contacts->copts, CO_IGNORE, 0);
                         conn->contacts->serv = conn;
                     }
@@ -1435,15 +1527,15 @@ void PrefReadStat (FILE *stf)
                 else if (conn && !strcasecmp (cmd, "entry"))
                 {
                     UDWORD id;
-                    UDWORD uin;
                     
-                    PrefParseInt (id);
-                    PrefParseInt (uin);
+                    if (format < 3)
+                        PrefParseInt (id);
                     PrefParse (cmd);
                     
-                    cont = ContactFindCreate (conn->contacts, uin, cmd);
+                    cont = ContactScreen (conn, cmd);
                     if (!cont)
                         ERROR;
+                    ContactCreate (conn, cont);
                     
                     while ((par = s_parse (&args)))
                         ContactAddAlias (cont, par->txt);
@@ -1468,16 +1560,16 @@ void PrefReadStat (FILE *stf)
     
     for (i = 0; (conn = ConnectionNr (i)); i++)
     {
-        if (conn->type & TYPEF_SERVER || conn->type == TYPE_MSN_SERVER)
+        if (conn->type & TYPEF_ANY_SERVER)
         {
             if (!conn->contacts)
             {
-                conn->contacts = cg = ContactGroupC (conn, 0, s_sprintf ("contacts-%s-%ld", ConnectionServerType (conn), conn->uin));
+                conn->contacts = cg = ContactGroupC (conn, 0, s_sprintf ("contacts-%s-%s", ConnectionServerType (conn->type), conn->screen));
                 OptSetVal (&cg->copts, CO_IGNORE, 0);
                 dep = 21;
             }
             if (!conn->cont)
-                conn->cont = ContactUIN (conn, conn->uin);
+                conn->cont = ContactScreen (conn, conn->screen);
         }
     }
 
@@ -1501,6 +1593,7 @@ int PrefWriteStatusFile (void)
 
     if (!prG->statfile)
         prG->statfile = strdup (s_sprintf ("%sstatus", PrefUserDir (prG)));
+    rl_printf (i18n (2048, "Saving preferences to %s.\n"), prG->statfile);
     stfn = strdup (s_sprintf ("%s.##", prG->statfile));
     stf = fopen (stfn, "w");
     if (!stf)
@@ -1531,18 +1624,20 @@ int PrefWriteStatusFile (void)
     t = time (NULL);
     fprintf (stf, "# This file was generated by mICQ " MICQ_VERSION " of %s %s\n", __TIME__, __DATE__);
     fprintf (stf, "# This file was generated on %s\n", ctime (&t));
-    fprintf (stf, "format 2\n\n");
+    fprintf (stf, "format 3\n\n");
 
     fprintf (stf, "\n# The contact list section.\n");
     for (k = 0; (ss = ConnectionNr (k)); k++)
     {
-        if (!ss->flags || !ss->uin)
+        if (~ss->type & TYPEF_ANY_SERVER)
             continue;
-        if (ss->type != TYPE_SERVER && ss->type != TYPE_SERVER_OLD)
+        if (!ss->flags)
+            continue;
+        if (!ss->uin && (ss->type & TYPEF_HAVEUIN))
             continue;
 
         fprintf (stf, "[Contacts]\n");
-        fprintf (stf, "server %s %ld\n", ConnectionServerType (ss), ss->uin);
+        fprintf (stf, "server %s %s\n", ConnectionServerType (ss->type), ss->screen);
         fprintf (stf, "%s\n", OptString (&ss->contacts->copts));
 
         for (i = 0; (cont = ContactIndex (0, i)); i++)
@@ -1550,7 +1645,8 @@ int PrefWriteStatusFile (void)
             if (cont->group && cont->group->serv == ss)
             {
                 ContactAlias *alias;
-                fprintf (stf, "entry %d %10ld %s", 0, cont->uin, s_quote (cont->nick));
+                fprintf (stf, "entry %s ", s_quote (cont->screen));
+                fprintf (stf, "%s", s_quote (cont->nick));
                 for (alias = cont->alias; alias; alias = alias->more)
                     fprintf (stf, " %s", s_quote (alias->alias));
                 fprintf (stf, "\n%s", OptString (&cont->copts));
@@ -1566,13 +1662,13 @@ int PrefWriteStatusFile (void)
             continue;
 
         fprintf (stf, "\n[Group]\n");
-        fprintf (stf, "server %s %ld\n", ConnectionServerType (cg->serv), cg->serv->uin);
+        fprintf (stf, "server %s %s\n", ConnectionServerType (cg->serv->type), cg->serv->screen);
         fprintf (stf, "label %s\n", s_quote (cg->name));
         fprintf (stf, "id %d\n", cg->id);
         fprintf (stf, "%s", OptString (&cg->copts));
 
         for (i = 0; (cont = ContactIndex (cg, i)); i++)
-            fprintf (stf, "entry 0 %10ld\n", cont->uin);
+            fprintf (stf, "entry %s\n", s_quote (cont->screen));
     }
 
     if (fclose (stf))
@@ -1647,31 +1743,27 @@ int PrefWriteConfFile (void)
     {
         if (!ss->flags)
             continue;
-        if ((!ss->uin && ss->type == TYPE_SERVER)
-            || (ss->type != TYPE_SERVER && ss->type != TYPE_SERVER_OLD
-                && ss->type != TYPE_MSN_SERVER
-                && ss->type != TYPE_MSGLISTEN && ss->type != TYPE_REMOTE)
-            || (ss->type == TYPE_MSGLISTEN && ss->parent && !ss->parent->uin))
+        if (!strcmp (ConnectionServerType (ss->type), "unknown"))
+            continue;
+        if (ss->type == TYPE_MSGLISTEN && (!ss->parent || !ss->parent->uin))
             continue;
 
         fprintf (rcf, "[Connection]\n");
-        fprintf (rcf, "type %s%s\n",  ConnectionServerType (ss),
+        fprintf (rcf, "type %s%s\n",  ConnectionServerType (ss->type),
                                       ss->flags & CONN_AUTOLOGIN ? " auto" : "");
         fprintf (rcf, "version %d\n", ss->version);
         if (ss->pref_server)
             fprintf (rcf, "server %s\n",   s_quote (ss->pref_server));
-        if (ss->pref_port)
+        if (ss->pref_port && ss->pref_port != -1)
             fprintf (rcf, "port %ld\n",    ss->pref_port);
-        if (ss->uin)
-            fprintf (rcf, "uin %ld\n",     ss->uin);
-        if (ss->screen && ss->type == TYPE_MSN_SERVER)
+        if (ss->type & TYPEF_ANY_SERVER)
             fprintf (rcf, "screen %s\n",   s_quote (ss->screen));
         if (ss->pref_passwd)
             fprintf (rcf, "password %s\n", s_quote (ss->pref_passwd));
         else if (!k)
             fprintf (rcf, "# password\n");
         if (ss->pref_status || !k)
-            fprintf (rcf, "status %ld\n",  ss->pref_status);
+            fprintf (rcf, "status %s\n",  ContactStatusStr (ss->pref_status));
         fprintf (rcf, "\n");
     }
 
@@ -1718,8 +1810,9 @@ int PrefWriteConfFile (void)
                     prG->flags & FLAG_COLOR     ? "on " : "off");
     fprintf (rcf, "set auto       %s # if automatic responses are to be sent\n",
                     prG->flags & FLAG_AUTOREPLY ? "on " : "off");
-    fprintf (rcf, "set uinprompt  %s # if the prompt should contain the last uin a message was received from\n",
-                    prG->flags & FLAG_UINPROMPT ? "on " : "off");
+    fprintf (rcf, "set prompt  %s # type of prompt (user, uin, simple)\n",
+                    prG->flags & FLAG_USERPROMPT ? "user " : 
+                    prG->flags & FLAG_UINPROMPT ? "uin " : "simple");
     fprintf (rcf, "set autosave   %s # whether the micqrc should be automatically saved on exit\n",
                     prG->flags & FLAG_AUTOSAVE ? "on " : "off");
     fprintf (rcf, "set autofinger %s # whether new UINs should be fingered automatically\n",
@@ -1747,9 +1840,17 @@ int PrefWriteConfFile (void)
 
     fprintf (rcf, "event %s\n\n", prG->event_cmd && *prG->event_cmd ? s_quote (prG->event_cmd) : "off");
 
+    fprintf (rcf, "prompt_strftime \"%s\"          # format for time prompt option %%T. man strftime\n",
+                   prG->prompt_strftime);
+
     fprintf (rcf, "\n# The strings section - runtime redefinable strings.\n");
-    fprintf (rcf, "# The aliases.\n");
     fprintf (rcf, "[Strings]\n");
+    fprintf (rcf, "# The user prompt.\n");
+
+    fprintf (rcf, "prompt %s          # user prompt. for prompt template see manual\n",
+                   s_quote (prG->prompt));
+
+    fprintf (rcf, "# The aliases.\n");
     {
         const alias_t *node;
         for (node = AliasList (); node; node = node->next)
