@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: im_icq8.c,v 1.23 2005/05/05 17:02:00 kuhlmann Exp $
+ * $Id: im_icq8.c,v 1.29 2007-04-17 19:23:58 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -108,27 +108,27 @@ static void IMRosterDoDelete (Event *event)
     if (roster->delname)
     {
         for (re = roster->groups; re; re = re->next)
-            if (!strcmp (re->name, roster->delname))
+            if (!strcmp (re->name, roster->delname) || (re->nick && !strcmp (re->nick, roster->delname)))
                 SnacCliRosterentrydelete (serv, re);
 
         for (re = roster->ignore; re; re = re->next)
-            if (!strcmp (re->name, roster->delname))
+            if (!strcmp (re->name, roster->delname) || (re->nick && !strcmp (re->nick, roster->delname)))
                 SnacCliRosterentrydelete (serv, re);
 
         for (re = roster->invisible; re; re = re->next)
-            if (!strcmp (re->name, roster->delname))
+            if (!strcmp (re->name, roster->delname) || (re->nick && !strcmp (re->nick, roster->delname)))
                 SnacCliRosterentrydelete (serv, re);
 
         for (re = roster->visible; re; re = re->next)
-            if (!strcmp (re->name, roster->delname))
+            if (!strcmp (re->name, roster->delname) || (re->nick && !strcmp (re->nick, roster->delname)))
                 SnacCliRosterentrydelete (serv, re);
 
         for (re = roster->normal; re; re = re->next)
-            if (!strcmp (re->name, roster->delname))
+            if (!strcmp (re->name, roster->delname) || (re->nick && !strcmp (re->nick, roster->delname)))
                 SnacCliRosterentrydelete (serv, re);
 
         for (re = roster->generic; re; re = re->next)
-            if (!strcmp (re->name, roster->delname))
+            if (!strcmp (re->name, roster->delname) || (re->nick && !strcmp (re->nick, roster->delname)))
                 SnacCliRosterentrydelete (serv, re);
     }
     else
@@ -317,7 +317,7 @@ static void IMRosterDownCont (Connection *serv, Roster *roster, ContactGroup *cg
         mod = 1;
     }
     ContactAddAlias (cont, rc->name);
-    if (mod || (rc->nick && strcmp (rc->nick, rc->name) && cont != ContactFind (serv->contacts, 0, cont->uin, rc->nick)))
+    if (rc->nick && strcmp (rc->nick, rc->name) && !ContactFindAlias (cont, rc->nick))
     {
         ContactAddAlias (cont, rc->nick);
         mod = 1;
@@ -564,7 +564,7 @@ static void IMRosterDiff (Event *event)
                        rg && rg->name ? rg->name : "?", rc->name, rc->nick ? rc->nick : rc->name, rc->id);
             rl_printf (i18n (2483, "Local:  Group %s Contact %s/%s (#%d)\n"),
                        cont->group == serv->contacts ? "(none)" : cont->group->name,
-                       s_sprintf ("%ld", cont->uin), cont->nick, 0);
+                       cont->screen, cont->nick, 0);
         }
     }
     for (rc = roster->normal; rc; rc = rc->next)
@@ -583,7 +583,7 @@ static void IMRosterDiff (Event *event)
                        rg && rg->name ? rg->name : "?", rc->name, rc->nick ? rc->nick : rc->name, rc->id);
             rl_printf (i18n (2486, "Local:  Group %s Contact %s/%s (#%d) <%s>\n"),
                        cont->group == serv->contacts ? "(none)" : cont->group->name,
-                       s_sprintf ("%ld", cont->uin), cont->nick, 0,
+                       cont->screen, cont->nick, 0,
                        ContactPrefVal (cont, CO_HIDEFROM) ? i18n (2487, "hidefrom") : i18n (2488, "intimate"));
         }
     }
@@ -603,32 +603,32 @@ static void IMRosterDiff (Event *event)
                        rg && rg->name ? rg->name : "?", rc->name, rc->nick ? rc->nick : rc->name, rc->id);
             rl_printf (i18n (2483, "Local:  Group %s Contact %s/%s (#%d)\n"),
                        cont->group == serv->contacts ? "(none)" : cont->group->name,
-                       s_sprintf ("%ld", cont->uin), cont->nick, 0);
+                       cont->screen, cont->nick, 0);
         }
     }
 
     for (i = 0; (cont = ContactIndex (serv->contacts, i)); i++)    
         if (ContactPrefVal (cont, CO_WANTSBL))
         {
-            if (ContactPrefVal (cont, CO_HIDEFROM) && !ContactID (cont, ROSTER_TYPE_INVISIBLE)->issbl)
+            if (ContactPrefVal (cont, CO_HIDEFROM) && !ContactID (cont, roster_invisible)->issbl)
                 rl_printf (i18n (2486, "Local:  Group %s Contact %s/%s (#%d) <%s>\n"),
                            cont->group == serv->contacts ? "(none)" : cont->group->name,
-                           s_sprintf ("%ld", cont->uin), cont->nick, ContactIDGet (cont, ROSTER_TYPE_INVISIBLE),
+                           cont->screen, cont->nick, ContactIDGet (cont, roster_invisible),
                            i18n (2487, "hidefrom"));
-            else if (ContactPrefVal (cont, CO_INTIMATE) && !ContactID (cont, ROSTER_TYPE_VISIBLE)->issbl)
+            else if (ContactPrefVal (cont, CO_INTIMATE) && !ContactID (cont, roster_visible)->issbl)
                 rl_printf (i18n (2486, "Local:  Group %s Contact %s/%s (#%d) <%s>\n"),
                            cont->group == serv->contacts ? "(none)" : cont->group->name,
-                           s_sprintf ("%ld", cont->uin), cont->nick, ContactIDGet (cont, ROSTER_TYPE_VISIBLE),
+                           cont->screen, cont->nick, ContactIDGet (cont, roster_visible),
                            i18n (2488, "intimate"));
-            else if (ContactPrefVal (cont, CO_IGNORE) && !ContactID (cont, ROSTER_TYPE_IGNORE)->issbl)
+            else if (ContactPrefVal (cont, CO_IGNORE) && !ContactID (cont, roster_ignore)->issbl)
                 rl_printf (i18n (2486, "Local:  Group %s Contact %s/%s (#%d) <%s>\n"),
                            cont->group == serv->contacts ? "(none)" : cont->group->name,
-                           s_sprintf ("%ld", cont->uin), cont->nick, ContactIDGet (cont, ROSTER_TYPE_IGNORE),
+                           cont->screen, cont->nick, ContactIDGet (cont, roster_ignore),
                            i18n (2580, "ignore"));
-            else if (!ContactID (cont, ROSTER_TYPE_NORMAL)->issbl || !ContactPrefVal (cont, CO_ISSBL))
+            else if (!ContactID (cont, roster_normal)->issbl || !ContactPrefVal (cont, CO_ISSBL))
                 rl_printf (i18n (2486, "Local:  Group %s Contact %s/%s (#%d) <%s>\n"),
                            cont->group == serv->contacts ? "(none)" : cont->group->name,
-                           s_sprintf ("%ld", cont->uin), cont->nick, ContactIDGet (cont, ROSTER_TYPE_IGNORE),
+                           cont->screen, cont->nick, ContactIDGet (cont, roster_ignore),
                            i18n (2490, "normal"));
             else
                 cnt_more--;
