@@ -1,4 +1,4 @@
-/* $Id: micq.c,v 1.154 2005/12/22 16:33:17 kuhlmann Exp $ */
+/* $Id: micq.c,v 1.167 2007-04-07 13:50:30 kuhlmann Exp $ */
 /* Copyright
  * This file may be distributed under version 2 of the GPL licence.
  */
@@ -25,29 +25,23 @@
 #include <conio.h>
 #endif
 
-#include "util_ui.h"
 #include "file_util.h"
-#include "util.h"
 #include "util_rl.h"
 #include "conv.h"
 #include "buildmark.h"
-#include "cmd_user.h"
-#include "icq_response.h"
 #include "preferences.h"
 #include "server.h"
 #include "contact.h"
 #include "connection.h"
 #include "tcp.h"
 #include "tabs.h"
-#include "msg_queue.h"
 #include "util_io.h"
-#include "oscar_snac.h"
-#include "oscar_service.h"
 #include "oldicq_compat.h"
-#include "oldicq_client.h"
 #include "util_tcl.h"
 #include "util_ssl.h"
 #include "util_alias.h"
+#include "cmd_user.h"
+#include "util_parse.h"
 #include "os.h"
 
 #define MICQ_ICON_1 "   " GREEN "_" SGR0 "     "
@@ -75,105 +69,105 @@ static void Idle_Check (Connection *conn)
 {
     int saver = -1;
     time_t now;
-    UDWORD delta, new = 0xffffffffL;
+    UDWORD delta;
+    status_t news = ims_offline;
+    status_noi_t noinv = ContactClearInv (conn->status);
 
     if (~conn->type & TYPEF_ANY_SERVER)
         return;
 
-    if ((conn->status & (STATUSF_ICQDND | STATUSF_ICQOCC | STATUSF_ICQFFC))
-        || !(conn->connect & CONNECT_OK))
+    if ((noinv == imr_dnd || noinv == imr_occ || noinv == imr_ffc)
+        || ~conn->connect & CONNECT_OK)
     {
         uiG.idle_val = 0;
         return;
     }
     
     now = time (NULL);
-    if (!prG->away_time && !uiG.idle_val)
-        uiG.idle_val = now;
-    delta = uiG.idle_val ? now - uiG.idle_val : 0;
-
-    if (!prG->away_time && delta > 10 && uiG.idle_val)
+    
+    if (!prG->away_time)
     {
-        saver = os_DetectLockedWorkstation();
-        
-        if (saver >= 0 && saver <= 3)
+        if (!uiG.idle_val)
+            uiG.idle_val = now;
+
+        delta = now - uiG.idle_val;
+
+        if (delta > 10 && uiG.idle_val)
         {
-            switch (saver)
+            saver = os_DetectLockedWorkstation();
+            
+            if (saver >= 0 && saver <= 3)
             {
-                case 0: /* no screen saver, not locked */
-                    if (!(conn->status & (STATUSF_ICQAWAY | STATUSF_ICQNA)))
-                        return;
-                    new = (conn->status & STATUSF_ICQINV) | STATUS_ICQONLINE;
-                    uiG.idle_flag = 0;
-                    break;
-                case 2: /* locked workstation */
-                case 3:
-                    if (conn->status & STATUS_ICQNA)
-                        return;
-                    new = (conn->status & STATUSF_ICQINV) | STATUS_ICQNA;
-                    uiG.idle_msgs = 0;
-                    uiG.idle_flag = 1;
-                    break;
-                case 1: /* screen saver only */
-                    if ((conn->status & (STATUSF_ICQAWAY | STATUSF_ICQNA)) == STATUS_ICQAWAY)
-                        return;
-                    new = (conn->status & STATUSF_ICQINV) | STATUS_ICQAWAY;
-                    uiG.idle_msgs = 0;
-                    uiG.idle_flag = 1;
-                    break;
+                switch (saver)
+                {
+                    case 0: /* no screen saver, not locked */
+                        if (noinv != imr_dnd && noinv != imr_occ && noinv != imr_na && noinv != imr_away)
+                            return;
+                        news = ContactCopyInv (conn->status, imr_online);
+                        conn->idle_flag = i_idle;
+                        break;
+                    case 2: /* locked workstation */
+                    case 3:
+                        if (noinv == imr_na)
+                            return;
+                        news = ContactCopyInv (conn->status, imr_na);
+                        uiG.idle_msgs = 0;
+                        conn->idle_flag = i_os;
+                        break;
+                    case 1: /* screen saver only */
+                        if (noinv == imr_away || noinv == imr_occ || noinv == imr_dnd)
+                            return;
+                        news = ContactCopyInv (conn->status, imr_away);
+                        uiG.idle_msgs = 0;
+                        conn->idle_flag = i_os;
+                        break;
+                }
+
+                uiG.idle_val = 0;
+                delta = 0;
+                if (news == ims_offline || news == conn->status)
+                    return;
             }
-
-            uiG.idle_val = 0;
-            delta = 0;
-            if (new == 0xffffffffL || new == conn->status)
-                return;
         }
-    }
 
-    if (!prG->away_time && !uiG.idle_flag && new == 0xffffffffL)
-        return;
+        if (conn->idle_flag == i_idle && news == ims_offline)
+            return;
+    }
+    else
+        delta = uiG.idle_val ? now - uiG.idle_val : 0;
 
     if (!uiG.idle_val)
         uiG.idle_val = now;
-
-    if (uiG.idle_flag & 2)
+    
+    if (conn->idle_flag == i_idle)
     {
-        if (conn->status & STATUSF_ICQNA)
+        if (delta >= prG->away_time && noinv != imr_dnd
+            && noinv != imr_occ && noinv != imr_na && noinv != imr_away)
         {
-            if (delta < prG->away_time || !prG->away_time)
-            {
-                new = (conn->status & STATUSF_ICQINV) | STATUS_ICQONLINE;
-                uiG.idle_flag = 0;
-                uiG.idle_val = 0;
-            }
-        }
-        else if (conn->status & STATUSF_ICQAWAY)
-        {
-            if (delta >= 2 * prG->away_time)
-                new = (conn->status & STATUSF_ICQINV) | STATUS_ICQNA;
-            else if (delta < prG->away_time || !prG->away_time)
-            {
-                new = (conn->status & STATUSF_ICQINV) | STATUS_ICQONLINE;
-                uiG.idle_flag = 0;
-                uiG.idle_val = 0;
-            }
+            news = ContactCopyInv (conn->status, imr_away);
+            conn->idle_flag = i_want_away;
+            uiG.idle_msgs = 0;
         }
     }
-    else if (!uiG.idle_flag && delta >= prG->away_time && !(conn->status & (STATUSF_ICQAWAY | STATUSF_ICQNA)))
+    else if (conn->idle_flag != i_os)
     {
-        new = (conn->status & STATUSF_ICQINV) | STATUS_ICQAWAY;
-        uiG.idle_flag = 2;
-        uiG.idle_msgs = 0;
+        if (delta < prG->away_time || !prG->away_time)
+        {
+            news = ContactCopyInv (conn->status, imr_online);
+            conn->idle_flag = i_idle;
+            uiG.idle_val = 0;
+        }
+        else if (conn->idle_flag == i_want_away && delta >= 2 * prG->away_time)
+        {
+            news = ContactCopyInv (conn->status, imr_na);
+            conn->idle_flag = i_want_na;
+        }
     }
-    if (new != 0xffffffffL && new != conn->status)
+    if (news != ims_offline && news != conn->status)
     {
-        if (conn->type == TYPE_SERVER)
-            SnacCliSetstatus (conn, new, 1);
-        else
-            CmdPktCmdStatusChange (conn, new);
-        conn->status = new;
+        IMSetStatus (conn, NULL, news, i18n (2605, "Automatic status change."));
         rl_printf ("%s ", s_now);
-        rl_printf (i18n (1064, "Automatically changed status to %s.\n"), s_status (new));
+        rl_printf (i18n (1064, "Automatically changed status to %s.\n"), s_status (news, 0));
     }
     return;
 }
@@ -189,7 +183,8 @@ static void Init (int argc, char *argv[])
     Connection *conn;
     Event *loginevent = NULL;
     const char *arg_v, *arg_f, *arg_l, *arg_i, *arg_b, *arg_s, *arg_u, *arg_p;
-    UDWORD loaded, uingiven = 0, arg_h = 0, arg_vv = 0, arg_c = 0, arg_ss = 0;
+    UDWORD loaded, uingiven = 0, arg_h = 0, arg_vv = 0, arg_c = 0;
+    status_t arg_ss = ims_offline;
     str_s arg_C = { NULL, 0, 0 };
     val_t res;
     UBYTE save_conv_error;
@@ -234,7 +229,7 @@ static void Init (int argc, char *argv[])
                     i++, arg_vv = n | 0x80000000UL;
             }
         }
-        else if (!strcmp (argv[i], "-u") || !strcmp (argv[i], "--uin"))
+        else if (!strcmp (argv[i], "-u") || !strcmp (argv[i], "--uin") || !strcmp (argv[i], "--user"))
         {
             if (argv[i + 1] && *argv[i + 1])
             {
@@ -361,7 +356,7 @@ static void Init (int argc, char *argv[])
         rl_print  (i18n (2204, "  -i, --i1" "8n     use given locale (default: auto-detected)\n"));
         rl_print  (i18n (2200, "  -v, --verbose  set (or increase) verbosity (mostly for debugging)\n"));
         rl_printf (i18n (2203, "  -l, --logplace use given log file/dir (default: %s)\n"), "BASE" _OS_PATHSEPSTR "history" _OS_PATHSEPSTR);
-        rl_print  (i18n (2494, "  -u, --uin      login with this UIN instead of those configured\n"));
+        rl_print  (i18n (2494, "  -u, --user     login with this user/UIN instead of those configured\n"));
         rl_print  (i18n (2495, "  -p, --passwd   ... and override password\n"));
         rl_print  (i18n (2496, "  -s, --status   ... and override status\n"));
         rl_print  (i18n (2497, "  -C, --cmd      ... and execute mICQ command\n"));
@@ -477,12 +472,25 @@ static void Init (int argc, char *argv[])
     
     for (i = 0; i < j; i++)
     {
-        if      (!strcmp (targv[i], "-u") || !strcmp (targv[i], "--uin"))
+        if      (!strcmp (targv[i], "-u") || !strcmp (targv[i], "--uin") || !strcmp (targv[i], "--user"))
         {
             if (arg_u)
             {
-                if (!(conn = ConnectionFindUIN (TYPEF_ANY_SERVER, atoll (arg_u))) && atoll (arg_u))
-                    conn = PrefNewConnection (atoll (arg_u), arg_p);
+                if (!(conn = ConnectionFindScreen (TYPEF_ANY_SERVER, arg_u)))
+                {
+                    char *u = strdup (arg_u);
+                    if (is_valid_icq_name (u))
+                        conn = PrefNewConnection (TYPE_SERVER, u, arg_p);
+#ifdef ENABLE_XMPP
+                    else if (is_valid_xmpp_name (u))
+                        conn = PrefNewConnection (TYPE_XMPP_SERVER, u, arg_p);
+#endif
+#ifdef ENABLE_MSN
+                    else if (is_valid_msn_name (u))
+                        conn = PrefNewConnection (TYPE_MSN_SERVER, u, arg_p);
+#endif
+                    s_free (u);
+                }
             }
             else if (!*targv[i+1])
                 conn = ConnectionFind (TYPEF_ANY_SERVER, NULL, NULL);
@@ -492,7 +500,7 @@ static void Init (int argc, char *argv[])
                     conn->status = arg_ss;
                 if (arg_p)
                     s_repl (&conn->passwd, arg_p);
-                if ((!arg_s || arg_ss != STATUS_ICQOFFLINE) && (loginevent = conn->open (conn)))
+                if ((!arg_s || arg_ss != ims_offline) && (loginevent = conn->open (conn)))
                     QueueEnqueueDep (conn, QUEUE_MICQ_COMMAND, 0, loginevent, NULL, conn->cont,
                                      OptSetVals (NULL, CO_MICQCOMMAND, arg_C.len ? arg_C.txt : "eg", 0),
                                      &CmdUserCallbackTodo);
@@ -503,7 +511,7 @@ static void Init (int argc, char *argv[])
                 arg_u = NULL;
             arg_p = arg_s = NULL;
             s_init (&arg_C, "", 0);
-            arg_ss = STATUS_ICQONLINE;
+            arg_ss = ims_online;
             conn = NULL;
         }
         else if (!strcmp (targv[i], "-p") || !strcmp (targv[i], "--passwd"))
@@ -512,22 +520,8 @@ static void Init (int argc, char *argv[])
         {
             if ((arg_s = targv[++i]))
             {
-                if (!strncmp (arg_s, "inv", 3))
-                    arg_ss = STATUS_ICQINV;
-                else if (!strcmp (arg_s, "dnd"))
-                    arg_ss = STATUS_ICQDND;
-                else if (!strcmp (arg_s, "occ"))
-                    arg_ss = STATUS_ICQOCC;
-                else if (!strcmp (arg_s, "na"))
-                    arg_ss = STATUS_ICQNA;
-                else if (!strcmp (arg_s, "away"))
-                    arg_ss = STATUS_ICQAWAY;
-                else if (!strcmp (arg_s, "ffc"))
-                    arg_ss = STATUS_ICQFFC;
-                else if (!strncmp (arg_s, "off", 3))
-                    arg_ss = STATUS_ICQOFFLINE;
-                else
-                    arg_ss = atoll (arg_s);
+                if (!ContactStatus (&arg_s, &arg_ss))
+                    arg_ss = IcqToStatus (atoll (arg_s));
             }
         }
         else
@@ -544,7 +538,7 @@ static void Init (int argc, char *argv[])
             {
                 if (conn->type & TYPEF_ANY_SERVER)
                 {
-                    if (conn->status != STATUS_ICQOFFLINE && (loginevent = conn->open (conn)))
+                    if (conn->status != ims_offline && (loginevent = conn->open (conn)))
                          QueueEnqueueDep (conn, QUEUE_MICQ_COMMAND, 0, loginevent, NULL, conn->cont,
                                           OptSetVals (NULL, CO_MICQCOMMAND, arg_C.len ? arg_C.txt : "eg", 0),
                                           &CmdUserCallbackTodo);
