@@ -27,7 +27,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: oldicq_client.c,v 1.6 2005/12/22 16:33:17 kuhlmann Exp $
+ * $Id: oldicq_client.c,v 1.10 2006-09-21 22:03:49 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -69,7 +69,7 @@ void CmdPktCmdSendMessage (Connection *conn, Contact *cont, const char *text, UD
     if (!cont)
         return;
     
-    IMIntMsg (cont, conn, NOW, STATUS_ICQOFFLINE, INT_MSGACK_V5, text, NULL);
+    IMIntMsg (cont, conn, NOW, ims_offline, INT_MSGACK_V5, text, NULL);
                     
     pak = PacketCv5 (conn, CMD_SEND_MESSAGE);
     PacketWrite4    (pak, cont->uin);
@@ -127,7 +127,7 @@ void CmdPktCmdLogin (Connection *conn)
     PacketWrite4 (pak, conn->our_local_ip);
     PacketWrite1 (pak, conn->assoc && conn->assoc->connect & CONNECT_OK ?
                        conn->assoc->status : 0);         /* 1=firewall | 2=proxy | 4=tcp */
-    PacketWrite4 (pak, prG->status);
+    PacketWrite4 (pak, IcqFromStatus (prG->status));
     PacketWrite2 (pak, conn->assoc && conn->assoc->connect & CONNECT_OK ?
                        conn->assoc->version : 0);
     PacketWrite2 (pak, 0);
@@ -306,13 +306,14 @@ void CmdPktCmdExtInfoReq (Connection *conn, Contact *cont)
 /*
  * CMD_STATUS_CHANGE - change status.
  */
-void CmdPktCmdStatusChange (Connection *conn, UDWORD status)
+void CmdPktCmdStatusChange (Connection *conn, status_t status)
 {
     Packet *pak = PacketCv5 (conn, CMD_STATUS_CHANGE);
-    PacketWrite4 (pak, status);
+    PacketWrite4 (pak, IcqFromStatus (status));
     PacketEnqueuev5 (pak, conn);
 
     conn->status = status;
+    conn->nativestatus = 0;
 }
 
 /*
