@@ -1,8 +1,9 @@
 /* Copyright: This file may be distributed under version 2 of the GPL licence.
- * $Id: oldicq_compat.c,v 1.10 2005/12/22 16:33:17 kuhlmann Exp $
+ * $Id: oldicq_compat.c,v 1.17 2006-11-06 21:13:02 kuhlmann Exp $
  */
 
 #include "micq.h"
+#include <assert.h>
 #include "oldicq_compat.h"
 #include "contact.h"
 #include "connection.h"
@@ -16,38 +17,79 @@
 #include "oscar_oldicq.h"
 #include "util_ui.h"
 
+#define META_SRV_GEN_UPDATE     100
+#define META_SRV_OTHER_UPDATE   120
+#define META_SRV_ABOUT_UPDATE   130
+#define META_SRV_SMS_OK         150
+#define META_SRV_PASS_UPDATE    170
+#define META_SRV_GEN            200
+#define META_SRV_WORK           210
+#define META_SRV_MORE           220
+#define META_SRV_ABOUT          230
+#define META_SRV_INTEREST       240
+#define META_SRV_BACKGROUND     250
+#define META_SRV_MOREEMAIL      235
+#define META_SRV_INFO           260
+#define META_SRV_UNKNOWN_270    270
+#define META_SRV_WP_FOUND       420
+#define META_SRV_WP_LAST_USER   430
+#define META_SRV_RANDOM         870
+#define META_SRV_RANDOM_UPDATE  880
+
+/* test in this order */
+#define STATUSF_ICQINV       0x00000100
+#define STATUSF_ICQDND       0x00000002
+#define STATUSF_ICQOCC       0x00000010
+#define STATUSF_ICQNA        0x00000004
+#define STATUSF_ICQAWAY      0x00000001
+#define STATUSF_ICQFFC       0x00000020
+
+#define STATUS_ICQOFFLINE    0xffffffff
+#define STATUS_ICQINV         STATUSF_ICQINV
+#define STATUS_ICQDND        (STATUSF_ICQDND | STATUSF_ICQOCC | STATUSF_ICQAWAY)
+#define STATUS_ICQOCC        (STATUSF_ICQOCC | STATUSF_ICQAWAY)
+#define STATUS_ICQNA         (STATUSF_ICQNA  | STATUSF_ICQAWAY)
+#define STATUS_ICQAWAY        STATUSF_ICQAWAY
+#define STATUS_ICQFFC         STATUSF_ICQFFC
+#define STATUS_ICQONLINE     0x00000000
+
+
 void Auto_Reply (Connection *conn, Contact *cont)
 {
-    const char *temp;
+    const char *temp = NULL;
 
     if (!(prG->flags & FLAG_AUTOREPLY) || !cont)
         return;
 
-          if (conn->status & STATUSF_ICQDND)
-         temp = ContactPrefStr (cont, CO_TAUTODND);
-     else if (conn->status & STATUSF_ICQOCC)
-         temp = ContactPrefStr (cont, CO_TAUTOOCC);
-     else if (conn->status & STATUSF_ICQNA)
-         temp = ContactPrefStr (cont, CO_TAUTONA);
-     else if (conn->status & STATUSF_ICQAWAY)
-         temp = ContactPrefStr (cont, CO_TAUTOAWAY);
-     else if (conn->status & STATUSF_ICQINV)
-         temp = ContactPrefStr (cont, CO_TAUTOINV);
-     else
-         return;
+    switch (ContactClearInv (cont->status))
+    {
+        case imr_dnd:  temp = ContactPrefStr (cont, CO_TAUTODND);  break;
+        case imr_occ:  temp = ContactPrefStr (cont, CO_TAUTOOCC);  break;
+        case imr_na:   temp = ContactPrefStr (cont, CO_TAUTONA);   break;
+        case imr_away: temp = ContactPrefStr (cont, CO_TAUTOAWAY); break;
+        case imr_ffc:  temp = ContactPrefStr (cont, CO_TAUTOFFC);  break;
+        case imr_offline: return;
+        case imr_online:
+            if (ContactIsInv (cont->status))
+                       temp = ContactPrefStr (cont, CO_TAUTOINV);
+            else
+                return;
+    }
     
     if (!temp || !*temp)
     {
-              if (conn->status & STATUSF_ICQDND)
-             temp = ContactPrefStr (cont, CO_AUTODND);
-         else if (conn->status & STATUSF_ICQOCC)
-             temp = ContactPrefStr (cont, CO_AUTOOCC);
-         else if (conn->status & STATUSF_ICQNA)
-             temp = ContactPrefStr (cont, CO_AUTONA);
-         else if (conn->status & STATUSF_ICQAWAY)
-             temp = ContactPrefStr (cont, CO_AUTOAWAY);
-         else if (conn->status & STATUSF_ICQINV)
-             temp = ContactPrefStr (cont, CO_AUTOINV);
+        switch (ContactClearInv (cont->status))
+        {
+            case imr_dnd:  temp = ContactPrefStr (cont, CO_AUTODND);  break;
+            case imr_occ:  temp = ContactPrefStr (cont, CO_AUTOOCC);  break;
+            case imr_na:   temp = ContactPrefStr (cont, CO_AUTONA);   break;
+            case imr_away: temp = ContactPrefStr (cont, CO_AUTOAWAY); break;
+            case imr_ffc:  temp = ContactPrefStr (cont, CO_AUTOFFC);  break;
+            case imr_offline: assert (0);
+            case imr_online:
+                assert (ContactIsInv (cont->status));
+                           temp = ContactPrefStr (cont, CO_AUTOINV);
+        }
     }
 
     IMCliMsg (conn, cont, OptSetVals (NULL, CO_MSGTYPE, MSG_AUTO, CO_MSGTEXT, temp, 0));
@@ -245,6 +287,7 @@ void Meta_User (Connection *conn, Contact *cont, Packet *pak)
             break;
         case META_SRV_MOREEMAIL:
             ContactMetaD (cont->meta_email);
+            cont->meta_email = NULL;
             if ((i = PacketRead1 (pak)))
             {
                 for (j = 0; j < i; j++)
@@ -330,8 +373,8 @@ void Meta_User (Connection *conn, Contact *cont, Packet *pak)
             uin = PacketRead4 (pak);
             event->cont = cont = ContactUIN (event->conn, uin);
             wdata = PacketRead2 (pak);
-            rl_printf (i18n (2009, "Found random chat partner UIN %ld in chat group %d.\n"),
-                      cont->uin, wdata);
+            rl_printf (i18n (2606, "Found random chat partner UIN %s in chat group %d.\n"),
+                      cont->screen, wdata);
             if (!cont || !CONTACT_DC (cont))
                 break;
             if (~cont->updated & UP_INFO)
@@ -455,3 +498,88 @@ void Recv_Message (Connection *conn, Packet *pak)
     IMSrvMsg (cont, conn, timegm (&stamp),
               OptSetVals (NULL, CO_ORIGIN, CV_ORIGIN_v5, CO_MSGTYPE, type, CO_MSGTEXT, text, 0));
 }
+
+status_t IcqToStatus (UDWORD status)
+{
+    status_t tmp;
+    
+    if (status == (UWORD)STATUS_ICQOFFLINE)
+        return ims_offline;
+    tmp = (status & STATUSF_ICQINV) ? ims_inv : ims_online;
+    if (status & STATUSF_ICQDND)
+        return ContactCopyInv (tmp, imr_dnd);
+    if (status & STATUSF_ICQOCC)
+        return ContactCopyInv (tmp, imr_occ);
+    if (status & STATUSF_ICQNA)
+        return ContactCopyInv (tmp, imr_na);
+    if (status & STATUSF_ICQAWAY)
+        return ContactCopyInv (tmp, imr_away);
+    if (status & STATUSF_ICQFFC)
+        return ContactCopyInv (tmp, imr_ffc);
+    return tmp;
+}
+
+UDWORD IcqFromStatus (status_t status)
+{
+    UDWORD isinv = ContactIsInv (status) ? STATUSF_ICQINV : 0;
+    switch (ContactClearInv (status))
+    {
+        case imr_offline:  return STATUS_ICQOFFLINE;
+        default:
+        case imr_online:   return STATUS_ICQONLINE | isinv;
+        case imr_ffc:      return STATUS_ICQFFC    | isinv;
+        case imr_away:     return STATUS_ICQAWAY   | isinv;
+        case imr_na:       return STATUS_ICQNA     | isinv;
+        case imr_occ:      return STATUS_ICQOCC    | isinv;
+        case imr_dnd:      return STATUS_ICQDND    | isinv;
+    }
+}
+
+#define STATUSF_ICQBIRTH     0x00080000
+#define STATUSF_ICQWEBAWARE  0x00010000
+#define STATUSF_ICQIP        0x00020000
+#define STATUSF_ICQDC_AUTH   0x10000000
+#define STATUSF_ICQDC_CONT   0x20000000
+
+statusflag_t IcqToFlags (UDWORD status)
+{
+    statusflag_t flags = imf_none;
+    if (status & STATUSF_ICQBIRTH)
+        flags |= imf_birth;
+    if (status & STATUSF_ICQWEBAWARE)
+        flags |= imf_web;
+    if (status & STATUSF_ICQDC_AUTH)
+        flags |= imf_dcauth;
+    if (status & STATUSF_ICQDC_CONT)
+        flags |= imf_dccont;
+    return flags;
+}
+
+UDWORD IcqFromFlags (statusflag_t flags)
+{
+    UDWORD status = 0;
+    if (flags & imf_birth)
+        status |= STATUSF_ICQBIRTH;
+    if (flags & imf_web)
+        status |= STATUSF_ICQWEBAWARE;
+    if (flags & imf_dcauth)
+        status |= STATUSF_ICQDC_AUTH;
+    if (flags & imf_dccont)
+        status |= STATUSF_ICQDC_CONT;
+    return status;
+}
+
+UDWORD IcqIsUIN (const char *screen)
+{
+    UDWORD uin = 0;
+    while (*screen)
+    {
+        if (*screen < '0' || *screen > '9')
+            return 0;
+        uin *= 10;
+        uin += *screen - '0';
+        screen++;
+    }
+    return uin;
+}
+
