@@ -1,6 +1,6 @@
 
 /* Copyright: This file may be distributed under version 2 of the GPL licence.
- * $Id: oldicq_server.c,v 1.3 2005/12/22 16:33:18 kuhlmann Exp $
+ * $Id: oldicq_server.c,v 1.10 2006-12-05 21:36:32 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -20,6 +20,7 @@
 #include "contact.h"
 #include "util_io.h"
 #include "util_syntax.h"
+#include "oscar_base.h"
 #include "conv.h"
 #include <assert.h>
 
@@ -249,7 +250,7 @@ void CmdPktSrvProcess (Connection *conn, Contact *cont, Packet *pak,
             Meta_User (conn, cont, pak);
             break;
         case SRV_NEW_UIN:
-            rl_printf (i18n (1639, "The new UIN is %ld!\n"), cont->uin);
+            rl_printf (i18n (2607, "The new UIN is %s!\n"), cont->screen);
             break;
         case SRV_UPDATE_FAIL:
             rl_print (i18n (1640, "Failed to update info.\n"));
@@ -258,7 +259,8 @@ void CmdPktSrvProcess (Connection *conn, Contact *cont, Packet *pak,
             rl_print (i18n (1641, "User info successfully updated.\n"));
             break;
         case SRV_LOGIN_REPLY:
-            rl_printf ("%s %s%10lu%s %s\n", s_now, COLCONTACT, cont->uin, COLNONE, i18n (1050, "Login successful!"));
+            rl_log_for (cont->nick, COLCONTACT);
+            rl_printf ("%s\n", i18n (1050, "Login successful!"));
             CmdPktCmdLogin1 (conn);
             CmdPktCmdContactList (conn);
             CmdPktCmdInvisList (conn);
@@ -280,9 +282,9 @@ void CmdPktSrvProcess (Connection *conn, Contact *cont, Packet *pak,
             ip[1] = PacketRead1 (pak);
             ip[2] = PacketRead1 (pak);
             ip[3] = PacketRead1 (pak);
-            rl_printf ("%s %s%*s%s %s %u.%u.%u.%u\n",
-                s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick,
-                COLNONE, i18n (1642, "IP:"), ip[0], ip[1], ip[2], ip[3]);
+            rl_log_for (cont->nick, COLCONTACT);
+            rl_printf ("%s %u.%u.%u.%u\n",
+                i18n (1642, "IP:"), ip[0], ip[1], ip[2], ip[3]);
             QueueEnqueueData (conn, QUEUE_UDP_KEEPALIVE, 0, time (NULL) + 120,
                               NULL, 0, NULL, &CmdPktSrvCallBackKeepAlive);
             break;
@@ -352,12 +354,15 @@ void CmdPktSrvProcess (Connection *conn, Contact *cont, Packet *pak,
             cont->dc->id2     = PacketRead4 (pak);
             cont->dc->id3     = PacketRead4 (pak);
             ContactSetVersion (cont);
-            IMOnline (cont, conn, status);
+            IMOnline (cont, conn, IcqToStatus (status), IcqToFlags (status), status, NULL);
             break;
         case SRV_STATUS_UPDATE:
             uin = PacketRead4 (pak);
             if ((cont = ContactUIN (conn, uin)))
-                IMOnline (cont, conn, PacketRead4 (pak));
+            {
+                status = PacketRead4 (pak);
+                IMOnline (cont, conn, IcqToStatus (status), IcqToFlags (status), status, NULL);
+            }
             break;
         case SRV_GO_AWAY:
         case SRV_NOT_CONNECTED:
@@ -409,16 +414,20 @@ void CmdPktSrvProcess (Connection *conn, Contact *cont, Packet *pak,
             cont->dc->port    = PacketRead4  (pak);
             cont->dc->ip_loc  = PacketReadB4 (pak);
             cont->dc->type    = PacketRead1  (pak);
-            cont->status      = PacketRead4  (pak);
+            status            = PacketRead4  (pak);
             cont->dc->version = PacketRead2  (pak);
+            
+            cont->status = IcqToStatus (status);
+            cont->flags = IcqToFlags (status);
+            cont->nativestatus = status;
 
-            rl_printf ("%-15s %lu\n", i18n (1440, "Random User:"), cont->uin);
+            rl_printf ("%-15s %s\n", i18n (1440, "Random User:"), cont->screen);
             rl_printf ("%-15s %s:%lu\n", i18n (1441, "remote IP:"), 
                       s_ip (cont->dc->ip_rem), cont->dc->port);
             rl_printf ("%-15s %s\n", i18n (1451, "local  IP:"),  s_ip (cont->dc->ip_loc));
             rl_printf ("%-15s %s\n", i18n (1454, "Connection:"), cont->dc->type == 4
                       ? i18n (1493, "Peer-to-Peer") : i18n (1494, "Server Only"));
-            rl_printf ("%-15s %s\n", i18n (1452, "Status:"), s_status (cont->status));
+            rl_printf ("%-15s %s\n", i18n (1452, "Status:"), s_status (cont->status, cont->nativestatus));
             rl_printf ("%-15s %d\n", i18n (1453, "TCP version:"), cont->dc->version);
         
             CmdPktCmdMetaReqInfo (conn, cont);
@@ -540,7 +549,7 @@ static JUMP_SRV_F (CmdPktSrvAck)
         if (!(cont = ContactUIN (conn, PacketReadAt4 (event->pak, CMD_v5_OFF_PARAM))))
             return;
 
-        IMIntMsg (cont, conn, NOW, STATUS_ICQOFFLINE, INT_MSGACK_V5,
+        IMIntMsg (cont, conn, NOW, ims_offline, INT_MSGACK_V5,
                   c_in_to_split (PacketReadAtL2Str (event->pak, 30, NULL), cont), NULL);
     }
     
