@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: oscar_base.c,v 1.11 2005/12/22 20:24:57 kuhlmann Exp $
+ * $Id: oscar_base.c,v 1.24 2006-12-05 21:36:32 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -146,7 +146,7 @@ static void FlapChannel4 (Connection *conn, Packet *pak)
 {
     TLV *tlv;
     
-    tlv = TLVRead (pak, PacketReadLeft (pak));
+    tlv = TLVRead (pak, PacketReadLeft (pak), -1);
     if (!tlv[5].str.len)
     {
         rl_printf ("%s " COLINDENT, s_now);
@@ -155,7 +155,7 @@ static void FlapChannel4 (Connection *conn, Packet *pak)
         else
             rl_print (i18n (1896, "Server closed connection:\n"));
         rl_printf (i18n (1048, "Error code: %ld\n"), tlv[9].nr ? tlv[9].nr : tlv[8].nr);
-        if (tlv[1].str.len && (UDWORD)atoi (tlv[1].str.txt) != conn->uin)
+        if (tlv[1].str.len && strcmp (tlv[1].str.txt, conn->screen))
             rl_printf (i18n (2218, "UIN: %s\n"), tlv[1].str.txt);
         if (tlv[4].str.len)
             rl_printf (i18n (1961, "URL: %s\n"), tlv[4].str.txt);
@@ -220,7 +220,7 @@ void FlapSave (Packet *pak, BOOL in)
 {
     FILE *logf;
     UDWORD oldrpos = pak->rpos;
-    char buf[200];
+    char buf[200], *d;
     
     snprintf (buf, sizeof (buf), "%sdebug", PrefUserDir (prG));
     mkdir (buf, 0700);
@@ -241,7 +241,8 @@ void FlapSave (Packet *pak, BOOL in)
                  PacketReadAtB2 (pak, 10), PacketReadAtB4 (pak, 12));
         pak->rpos = 16;
     }
-    fprintf (logf, PacketDump (pak, "", "", ""));
+    fprintf (logf, "%s", d = PacketDump (pak, "", "", ""));
+    free (d);
     pak->rpos = oldrpos;
     fclose (logf);
 }
@@ -320,9 +321,9 @@ void FlapCliIdent (Connection *conn)
     
     pak = FlapC (1);
     PacketWriteB4 (pak, CLI_HELLO);
-    PacketWriteTLVStr  (pak, 1, s_sprintf ("%ld", conn->uin));
+    PacketWriteTLVStr  (pak, 1, conn->screen);
     PacketWriteTLVData (pak, 2, f = _encryptpw (conn->passwd), strlen (conn->passwd));
-    PacketWriteTLVStr  (pak, 3, "ICQ Inc. - Product of ICQ (TM).2002a.5.37.1.3728.85");
+    PacketWriteTLVStr  (pak, 3, "ICQ Inc. - Product of ICQ (TM).2003b.5.37.1.3728.85");
     PacketWriteTLV2    (pak, 22, 266);
     PacketWriteTLV2    (pak, 23, FLAP_VER_MAJOR);
     PacketWriteTLV2    (pak, 24, FLAP_VER_MINOR);
@@ -330,7 +331,7 @@ void FlapCliIdent (Connection *conn)
     PacketWriteTLV2    (pak, 26, FLAP_VER_BUILD);
     PacketWriteTLV4    (pak, 20, FLAP_VER_SUBBUILD);
     PacketWriteTLVStr  (pak, 15, "de");  /* en */
-    PacketWriteTLVStr  (pak, 14, "de");  /* en */
+    PacketWriteTLVStr  (pak, 14, "DE");  /* en */
     FlapSend (conn, pak);
     free (f);
 }
@@ -387,21 +388,23 @@ Event *ConnectionInitServer (Connection *conn)
     
     if (!conn->server || !*conn->server || !conn->port)
         return NULL;
+    if (!conn->uin && ~conn->flags & CONN_WIZARD)
+        return NULL;
 
     if (conn->sok != -1)
         sockclose (conn->sok);
     conn->sok = -1;
-    conn->cont = cont = ContactUIN (conn, conn->uin);
+    conn->cont = cont = conn->uin ? ContactUIN (conn, conn->uin) : NULL;
     conn->our_seq  = rand () & 0x7fff;
     conn->connect  = 0;
     conn->dispatch = &SrvCallBackReceive;
     conn->reconnect= &SrvCallBackReconn;
     conn->close    = &FlapCliGoodbye;
     s_repl (&conn->server, conn->pref_server);
-    if (conn->status == STATUS_ICQOFFLINE)
+    if (conn->status == ims_offline)
         conn->status = conn->pref_status;
     
-    if ((event = QueueDequeue2 (conn, QUEUE_DEP_OSCARLOGIN, 0, NULL)))
+    if ((event = QueueDequeue2 (conn, QUEUE_DEP_WAITLOGIN, 0, NULL)))
     {
         event->attempts++;
         event->due = time (NULL) + 10 * event->attempts + 10;
@@ -409,13 +412,13 @@ Event *ConnectionInitServer (Connection *conn)
         QueueEnqueue (event);
     }
     else
-        event = QueueEnqueueData (conn, QUEUE_DEP_OSCARLOGIN, 0, time (NULL) + 12,
+        event = QueueEnqueueData (conn, QUEUE_DEP_WAITLOGIN, 0, time (NULL) + 12,
                                   NULL, conn->cont, NULL, &SrvCallBackTimeout);
 
     rl_printf (i18n (2512, "Opening v8 connection to %s:%s%ld%s for %s%s%s... "),
               s_wordquote (conn->server), COLQUOTE, conn->port, COLNONE, COLCONTACT,
               !cont ? i18n (2513, "new UIN") : cont->nick ? cont->nick 
-              : s_sprintf ("%ld", cont->uin), COLNONE);
+              : cont->screen, COLNONE);
 
     UtilIOConnectTCP (conn);
     return event;
@@ -431,14 +434,14 @@ static void SrvCallBackReconn (Connection *conn)
     if (!(cont = conn->cont))
         return;
     
-    if (!(event = QueueDequeue2 (conn, QUEUE_DEP_OSCARLOGIN, 0, NULL)))
+    if (!(event = QueueDequeue2 (conn, QUEUE_DEP_WAITLOGIN, 0, NULL)))
     {
         ConnectionInitServer (conn);
         return;
     }
     
     conn->connect = 0;
-    rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
+    rl_log_for (cont->nick, COLCONTACT);
     if (event->attempts < 5)
     {
         rl_printf (i18n (2032, "Scheduling v8 reconnect in %d seconds.\n"), 10 << event->attempts);
@@ -452,7 +455,7 @@ static void SrvCallBackReconn (Connection *conn)
         EventD (event);
     }
     for (i = 0; (cont = ContactIndex (cg, i)); i++)
-        cont->status = STATUS_ICQOFFLINE;
+        cont->status = ims_offline;
 }
 
 static void SrvCallBackDoReconn (Event *event)
@@ -561,21 +564,21 @@ void SrvCallBackReceive (Connection *conn)
 
 Connection *SrvRegisterUIN (Connection *conn, const char *pass)
 {
-    Connection *new;
+    Connection *news;
 #ifdef ENABLE_PEER2PEER
     Connection *newl;
 #endif
     
-    if (!(new = ConnectionC (TYPE_SERVER)))
+    if (!(news = ConnectionC (TYPE_SERVER)))
         return NULL;
 
 #ifdef ENABLE_PEER2PEER
-    if (!(newl = ConnectionClone (new, TYPE_MSGLISTEN)))
+    if (!(newl = ConnectionClone (news, TYPE_MSGLISTEN)))
     {
-        ConnectionD (new);
+        ConnectionD (news);
         return NULL;
     }
-    new->assoc = newl;
+    news->assoc = newl;
     newl->open = &ConnectionInitPeer;
     if (conn && conn->assoc)
     {
@@ -587,37 +590,36 @@ Connection *SrvRegisterUIN (Connection *conn, const char *pass)
     {
         newl->version = 8;
         newl->status = newl->pref_status = prG->s5Use ? 2 : TCP_OK_FLAG;
-        newl->flags |= CONN_AUTOLOGIN;
     }
+    newl->flags &= ~CONN_AUTOLOGIN;
 #endif
 
     if (conn)
     {
         assert (conn->type == TYPE_SERVER);
         
-        new->flags   = conn->flags & ~CONN_CONFIGURED;
-        new->version = conn->version;
-        new->uin     = 0;
-        new->pref_status  = STATUS_ICQONLINE;
-        new->pref_server  = strdup (conn->pref_server);
-        new->pref_port    = conn->pref_port;
-        new->pref_passwd  = strdup (pass);
+        news->flags   = conn->flags & ~CONN_CONFIGURED;
+        news->version = conn->version;
+        news->pref_status  = ims_online;
+        news->pref_server  = strdup (conn->pref_server);
+        news->pref_port    = conn->pref_port;
+        news->pref_passwd  = strdup (pass);
     }
     else
     {
-        new->version = 8;
-        new->uin     = 0;
-        new->pref_status  = STATUS_ICQONLINE;
-        new->pref_server  = strdup ("login.icq.com");
-        new->pref_port    = 5190;
-        new->pref_passwd  = strdup (pass);
-        new->flags |= CONN_AUTOLOGIN;
+        news->version = 8;
+        news->pref_status  = ims_online;
+        news->pref_server  = strdup ("login.icq.com");
+        news->pref_port    = 5190;
+        news->pref_passwd  = strdup (pass);
+        news->flags |= CONN_AUTOLOGIN;
     }
-    s_repl (&new->screen, "0");
-    new->server = strdup (new->pref_server);
-    new->port = new->pref_port;
-    new->passwd = strdup (pass);
-    new->open = &ConnectionInitServer;
-    return new;
+    news->uin = 0;
+    s_repl (&news->screen, "");
+    news->server = strdup (news->pref_server);
+    news->port = news->pref_port;
+    news->passwd = strdup (pass);
+    news->open = &ConnectionInitServer;
+    return news;
 }
 
