@@ -27,7 +27,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: oscar_icbm.c,v 1.28 2005/12/22 16:33:18 kuhlmann Exp $
+ * $Id: oscar_icbm.c,v 1.42 2007-02-20 21:56:02 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -169,7 +169,7 @@ JUMP_SNAC_F(SnacSrvAckmsg)
         const char *opt_text;
         if (OptGetStr (event->opt, CO_MSGTEXT, &opt_text));
         {
-            IMIntMsg (cont, serv, NOW, STATUS_ICQOFFLINE, INT_MSGACK_TYPE2, opt_text, NULL);
+            IMIntMsg (cont, serv, NOW, ims_offline, INT_MSGACK_TYPE2, opt_text, NULL);
             if ((~cont->oldflags & CONT_SEENAUTO) && strlen (text) && strcmp (text, opt_text))
             {
                 IMSrvMsg (cont, serv, NOW, OptSetVals (NULL, CO_ORIGIN, CV_ORIGIN_v8,
@@ -211,6 +211,8 @@ UBYTE SnacCliSendmsg (Connection *serv, Contact *cont, const char *text, UDWORD
             case MSG_AUTH_GRANT:
             case MSG_AUTH_DENY:
             case MSG_AUTH_ADDED:
+                if (!cont->uin)
+                    return RET_DEFER;
                 format = 4;
                 break;
             case MSG_NORM:
@@ -244,7 +246,7 @@ UBYTE SnacCliSendmsg (Connection *serv, Contact *cont, const char *text, UDWORD
             
             remenc = ContactPrefVal (cont, CO_ENCODING);
             
-            if (cont->status != STATUS_ICQOFFLINE &&
+            if (cont->status != ims_offline &&
                 HAS_CAP (cont->caps, CAP_UTF8) &&
                 !(cont->dc && cont->dc->id1 == (time_t)0xffffff42 &&
                   (cont->dc->id2 & 0x7fffffff) < (time_t)0x00040c00)) /* exclude old mICQ */
@@ -364,7 +366,7 @@ static void SnacCallbackType2 (Event *event)
         if (serv->connect & CONNECT_OK)
         {
             if (event->attempts > 1)
-                IMIntMsg (cont, serv, NOW, STATUS_ICQOFFLINE, INT_MSGTRY_TYPE2,
+                IMIntMsg (cont, serv, NOW, ims_offline, INT_MSGTRY_TYPE2,
                           opt_text, NULL);
             SnacSend (serv, PacketClone (pak));
             event->attempts++;
@@ -519,9 +521,9 @@ JUMP_SNAC_F(SnacSrvRecvmsg)
     Event *newevent;
     Cap *cap1, *cap2;
     Packet *p = NULL, *pp = NULL, *pak;
-    TLV *tlv;
+    TLV *tlv, *tlvs;
     Opt *opt;
-    UDWORD midtim, midrnd, midtime, midrand, unk, tmp, type1enc;
+    UDWORD midtim, midrnd, midtime, midrand, unk, tmp, type1enc, tlvc;
     UWORD seq1, tcpver, len, i, msgtyp, type;
     const char *txt = NULL;
     strc_t ctext;
@@ -534,36 +536,42 @@ JUMP_SNAC_F(SnacSrvRecvmsg)
     type    = PacketReadB2 (pak);
     cont    = PacketReadCont (pak, serv);
               PacketReadB2 (pak); /* WARNING */
-              PacketReadB2 (pak); /* COUNT */
+    tlvc    = PacketReadB2 (pak); /* COUNT */
     
     if (!cont)
         return;
 
-    tlv = TLVRead (pak, PacketReadLeft (pak));
+    tlv = TLVRead (pak, PacketReadLeft (pak), tlvc);
 
 #ifdef WIP
-    if (tlv[6].str.len && tlv[6].nr != cont->status)
-        rl_printf ("FIXMEWIP: status for %ld embedded in message 0x%08lx different from server status 0x%08lx.\n", cont->uin, tlv[6].nr, cont->status);
+    if (tlv[6].str.len && tlv[6].nr != cont->nativestatus)
+        rl_printf ("FIXMEWIP: status for %s embedded in message 0x%08lx different from server status 0x%08lx.\n", cont->screen, tlv[6].nr, cont->nativestatus);
 #endif
 
     if (tlv[6].str.len)
         event->opt = OptSetVals (event->opt, CO_STATUS, tlv[6].nr, 0);
 
-    /* tlv[2] may be there twice - ignore the member since time(NULL). */
-    if (tlv[2].str.len == 4)
-        TLVDone (tlv, 2);
+    TLVD (tlv);
+    tlv = TLVRead (pak, PacketReadLeft (pak), -1);
 
+    if (type == 1 && tlv[2].str.len)
+        tlvs = &tlv[2];
+    else if (type == 2 && tlv[5].str.len)
+        tlvs = &tlv[5];
+    else if (type == 4 && tlv[5].str.len)
+        tlvs = &tlv[5];
+    else
+    {
+        SnacSrvUnknown (event);
+        TLVD (tlv);
+        return;
+    }
+    p = PacketCreate (&tlvs->str);
+    TLVD (tlv);
+    
     switch (type)
     {
         case 1:
-            if (!tlv[2].str.len)
-            {
-                SnacSrvUnknown (event);
-                TLVD (tlv);
-                return;
-            }
-
-            p = PacketCreate (&tlv[2].str);
             PacketReadB2 (p);
             PacketReadData (p, NULL, PacketReadB2 (p));
                        PacketReadB2 (p);
@@ -573,7 +581,6 @@ JUMP_SNAC_F(SnacSrvRecvmsg)
             if (len < 4)
             {
                 SnacSrvUnknown (event);
-                TLVD (tlv);
                 PacketD (p);
                 return;
             }
@@ -606,12 +613,10 @@ JUMP_SNAC_F(SnacSrvRecvmsg)
             s_done (&str);
             break;
         case 2:
-            p = PacketCreate (&tlv[5].str);
             type   = PacketReadB2 (p);
             midtim = PacketReadB4 (p);
             midrnd = PacketReadB4 (p);
             cap1   = PacketReadCap (p);
-            TLVD (tlv);
             
             ContactSetCap (cont, cap1);
             if (midtim != midtime || midrnd != midrand)
@@ -621,7 +626,7 @@ JUMP_SNAC_F(SnacSrvRecvmsg)
                 return;
             }
 
-            tlv = TLVRead (p, PacketReadLeft (p));
+            tlv = TLVRead (p, PacketReadLeft (p), -1);
             PacketD (p);
             
             if ((i = TLVGet (tlv, 0x2711)) == (UWORD)-1)
@@ -631,7 +636,7 @@ JUMP_SNAC_F(SnacSrvRecvmsg)
 #ifdef WIP
                 else
                 {
-                    rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
+                    rl_log_for (cont->nick, COLCONTACT);
                     rl_printf ("FIXMEWIP: tlv(b)-only packet.\n");
                 }
 #endif
@@ -674,8 +679,8 @@ JUMP_SNAC_F(SnacSrvRecvmsg)
                         }
                         
 #ifdef WIP
-                        rl_printf ("%s %*s FIXMEWIP: updates dc to %s:%ld|%ld|%ld v%d %d seq %ld\n",
-                                  s_now, uiG.nick_len + s_delta (cont->nick), cont->nick,
+                        rl_log_for (cont->nick, COLCONTACT);
+                        rl_printf ("FIXMEWIP: updates dc to %s:%ld|%ld|%ld v%d %d seq %ld\n",
                                   s_ip (sip), sp1, sp2, sop, sver, scon, sunk);
 #endif
                         CONTACT_DC (cont)->ip_rem = sip;
@@ -734,7 +739,6 @@ JUMP_SNAC_F(SnacSrvRecvmsg)
             /* TLV 1, 2(!), 3, 4, f ignored */
             break;
         case 4:
-            p = PacketCreate (&tlv[5].str);
             unk    = PacketRead4 (p);
             msgtyp = PacketRead2 (p);
             if (unk != cont->uin)
@@ -754,12 +758,7 @@ JUMP_SNAC_F(SnacSrvRecvmsg)
             IMSrvMsg (cont, serv, NOW, opt);
             Auto_Reply (serv, cont);
             break;
-        default:
-            SnacSrvUnknown (event);
-            TLVD (tlv);
-            return;
     }
-    TLVD (tlv);
 }
 
 /*
@@ -792,12 +791,12 @@ JUMP_SNAC_F(SnacSrvSrvackmsg)
         case 1:
             event2 = QueueDequeue (serv, QUEUE_TYPE1_RESEND_ACK, pak->ref);
             if (event2 && OptGetStr (event2->opt, CO_MSGTEXT, &text))
-                IMIntMsg (cont, serv, NOW, STATUS_ICQOFFLINE, INT_MSGACK_V8, text, NULL);
+                IMIntMsg (cont, serv, NOW, ims_offline, INT_MSGACK_V8, text, NULL);
             break;
         case 4:
             event2 = QueueDequeue (serv, QUEUE_TYPE4_RESEND_ACK, pak->ref);
             if (event2 && OptGetStr (event2->opt, CO_MSGTEXT, &text))
-                IMIntMsg (cont, serv, NOW, STATUS_ICQOFFLINE, INT_MSGACK_V8, text, NULL);
+                IMIntMsg (cont, serv, NOW, ims_offline, INT_MSGACK_V8, text, NULL);
             break;
         case 2: /* msg was received by server */
             event2 = QueueDequeue (serv, QUEUE_TYPE2_RESEND_ACK, pak->ref);
@@ -812,26 +811,25 @@ JUMP_SNAC_F(SnacSrvSrvackmsg)
         EventD (event2);
 }
 
-void SrvMsgAdvanced (Packet *pak, UDWORD seq, UWORD msgtype, UWORD status,
-                     UDWORD deststatus, UWORD flags, const char *msg)
+void SrvMsgAdvanced (Packet *pak, UDWORD seq, UWORD msgtype, status_t status,
+                     status_t deststatus, UWORD flags, const char *msg)
 {
-    if (msgtype == MSG_SSL_OPEN)       status = 0;
-    else if (status == (UWORD)STATUS_ICQOFFLINE) /* keep */ ;
-    else if (status & STATUSF_ICQDND)     status = STATUSF_ICQDND  | (status & STATUSF_ICQINV);
-    else if (status & STATUSF_ICQOCC)     status = STATUSF_ICQOCC  | (status & STATUSF_ICQINV);
-    else if (status & STATUSF_ICQNA)      status = STATUSF_ICQNA   | (status & STATUSF_ICQINV);
-    else if (status & STATUSF_ICQAWAY)    status = STATUSF_ICQAWAY | (status & STATUSF_ICQINV);
-    else if (status & STATUSF_ICQFFC)     status = STATUSF_ICQFFC  | (status & STATUSF_ICQINV);
-    else                                  status &= STATUSF_ICQINV;
+    if (msgtype == MSG_SSL_OPEN)
+        status = ims_online;
     
-    if      (flags != (UWORD)-1)           /* keep */ ;
-    else if (deststatus == (UWORD)STATUS_ICQOFFLINE) /* keep */ ;
-    else if (deststatus & STATUSF_ICQDND)     flags = TCP_MSGF_CLIST;
-    else if (deststatus & STATUSF_ICQOCC)     flags = TCP_MSGF_CLIST;
-    else if (deststatus & STATUSF_ICQNA)      flags = TCP_MSGF_1;
-    else if (deststatus & STATUSF_ICQAWAY)    flags = TCP_MSGF_1;
-    else if (deststatus & STATUSF_ICQFFC)     flags = TCP_MSGF_LIST | TCP_MSGF_1;
-    else                                      flags = TCP_MSGF_LIST | TCP_MSGF_1;
+    if (flags == (UWORD)-1)
+    {
+        switch (ContactClearInv (deststatus))
+        {
+            case imr_offline: /* keep */ break;
+            case imr_dnd:
+            case imr_occ:     flags = TCP_MSGF_CLIST; break;
+            case imr_na:
+            case imr_away:    flags = TCP_MSGF_1; break;
+            case imr_ffc:
+            case imr_online:  flags = TCP_MSGF_LIST | TCP_MSGF_1; break;
+        }
+    }
 
     PacketWriteLen    (pak);
      PacketWrite2      (pak, seq);       /* sequence number */
@@ -840,7 +838,8 @@ void SrvMsgAdvanced (Packet *pak, UDWORD seq, UWORD msgtype, UWORD status,
      PacketWrite4      (pak, 0);
     PacketWriteLenDone (pak);
     PacketWrite2      (pak, msgtype);    /* message type    */
-    PacketWrite2      (pak, status);     /* status          */
+    PacketWrite2      (pak, IcqFromStatus (status));
+                                         /* status          */
     PacketWrite2      (pak, flags);      /* flags           */
     PacketWriteLNTS   (pak, msg);        /* the message     */
 }
@@ -913,7 +912,7 @@ void SrvReceiveAdvanced (Connection *serv, Event *inc_event, Packet *inc_pak, Ev
     Contact *cont = inc_event->cont;
     Opt *opt = inc_event->opt, *opt2;
     Packet *ack_pak = ack_event->pak;
-    const char *txt, *ack_msg, *tauto;
+    const char *txt, *ack_msg = "", *tauto;
     strc_t text, cname, ctext, reason, cctmp;
     char *name;
     UDWORD tmp, cmd, flen;
@@ -942,29 +941,24 @@ void SrvReceiveAdvanced (Connection *serv, Event *inc_event, Packet *inc_pak, Ev
     
     accept = FALSE;
 
-    if      (serv->status & STATUSF_ICQDND)
-        ack_msg = (tauto = ContactPrefStr (cont, CO_TAUTODND))  && *tauto ? tauto : ContactPrefStr (cont, CO_AUTODND);
-    else if (serv->status & STATUSF_ICQOCC)
-        ack_msg = (tauto = ContactPrefStr (cont, CO_TAUTOOCC))  && *tauto ? tauto : ContactPrefStr (cont, CO_AUTOOCC);
-    else if (serv->status & STATUSF_ICQNA)
-        ack_msg = (tauto = ContactPrefStr (cont, CO_TAUTONA))   && *tauto ? tauto : ContactPrefStr (cont, CO_AUTONA);
-    else if (serv->status & STATUSF_ICQAWAY)
-        ack_msg = (tauto = ContactPrefStr (cont, CO_TAUTOAWAY)) && *tauto ? tauto : ContactPrefStr (cont, CO_AUTOAWAY);
-    else
-        ack_msg = "";
-
-/*  if (serv->status & STATUSF_ICQDND)  ack_status  = pri & 4 ? TCP_ACK_ONLINE : TCP_ACK_DND; else
- *
+/*
  * Don't refuse until we have sensible preferences for that
  */
-    if (serv->status & STATUSF_ICQDND)  ack_status  = TCP_ACK_ONLINE; else
-    if (serv->status & STATUSF_ICQOCC)  ack_status  = TCP_ACK_ONLINE; else
-    if (serv->status & STATUSF_ICQNA)   ack_status  = TCP_ACK_NA;     else
-    if (serv->status & STATUSF_ICQAWAY) ack_status  = TCP_ACK_AWAY;
-    else                                ack_status  = TCP_ACK_ONLINE;
+    switch (ContactClearInv (serv->status))
+    {
+        case imr_dnd:  ack_msg = (tauto = ContactPrefStr (cont, CO_TAUTODND))  && *tauto ?
+                       tauto : ContactPrefStr (cont, CO_AUTODND);  ack_status = TCP_ACK_ONLINE; break;
+        case imr_occ:  ack_msg = (tauto = ContactPrefStr (cont, CO_TAUTOOCC))  && *tauto ?
+                       tauto : ContactPrefStr (cont, CO_AUTOOCC);  ack_status = TCP_ACK_ONLINE; break;
+        case imr_na:   ack_msg = (tauto = ContactPrefStr (cont, CO_TAUTONA))   && *tauto ?
+                       tauto : ContactPrefStr (cont, CO_AUTONA);   ack_status = TCP_ACK_NA;     break;
+        case imr_away: ack_msg = (tauto = ContactPrefStr (cont, CO_TAUTOAWAY)) && *tauto ?
+                       tauto : ContactPrefStr (cont, CO_AUTOAWAY); ack_status = TCP_ACK_AWAY;   break;
+        default:       ack_status  = TCP_ACK_ONLINE;
+    }
 
     ack_flags = 0;
-    if (serv->status & STATUSF_ICQINV)  ack_flags |= TCP_MSGF_INV;
+    if (ContactIsInv (serv->status))  ack_flags |= TCP_MSGF_INV;
 
     switch (msgtype & ~MSGF_MASS)
     {
@@ -999,12 +993,14 @@ void SrvReceiveAdvanced (Connection *serv, Event *inc_event, Packet *inc_pak, Ev
             if (msgtype != 1012)
 #endif
             {    
-                rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
                 if (((ack_flags & TCP_MSGF_INV) && !ContactPrefVal (cont, CO_INTIMATE)) || ContactPrefVal (cont, CO_HIDEFROM))
                 {
                     if (ContactPrefVal (cont, CO_SHOWCHANGE))
+                    {
+                        rl_log_for (cont->nick, COLCONTACT);
                         rl_printf (i18n (2568, "Ignored request for auto-response from %s%s%s.\n"),
                                    COLCONTACT, cont->nick, COLNONE);
+                    }
                     ack_event->due = 0;
                     ack_event->callback = NULL;
                     QueueDequeueEvent (ack_event);
@@ -1012,8 +1008,11 @@ void SrvReceiveAdvanced (Connection *serv, Event *inc_event, Packet *inc_pak, Ev
                     return;
                 }
                 if (ContactPrefVal (cont, CO_SHOWCHANGE))
+                {
+                    rl_log_for (cont->nick, COLCONTACT);
                     rl_printf (i18n (1814, "Sent auto-response message to %s%s%s.\n"),
                                COLCONTACT, cont->nick, COLNONE);
+                }
             }
 
             accept = TRUE;
@@ -1064,7 +1063,7 @@ void SrvReceiveAdvanced (Connection *serv, Event *inc_event, Packet *inc_pak, Ev
                 return;
             }
             free (name);
-            if (PeerFileIncAccept (serv->assoc, inc_event))
+            if (serv->assoc && PeerFileIncAccept (serv->assoc, inc_event))
             {
                 PacketWrite2    (ack_pak, ack_status);
                 PacketWrite2    (ack_pak, ack_flags);
@@ -1159,7 +1158,7 @@ void SrvReceiveAdvanced (Connection *serv, Event *inc_event, Packet *inc_pak, Ev
 #endif
                             return;
                         }
-                        if (PeerFileIncAccept (serv->assoc, inc_event))
+                        if (serv->assoc && PeerFileIncAccept (serv->assoc, inc_event))
                         {
                             PacketWrite2    (ack_pak, ack_status);
                             PacketWrite2    (ack_pak, ack_flags);
