@@ -27,7 +27,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: oscar_location.c,v 1.6 2005/12/11 20:57:29 kuhlmann Exp $
+ * $Id: oscar_location.c,v 1.8 2006-12-05 21:36:32 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -85,6 +85,7 @@ void SnacCliSetuserinfo (Connection *serv)
     PacketWriteCapID   (pak, CAP_ISICQ);
     PacketWriteCapID   (pak, CAP_SRVRELAY);
     PacketWriteCapID   (pak, CAP_UTF8);
+    PacketWriteCapID   (pak, CAP_AIM_INTER);
     PacketWriteCapID   (pak, CAP_MICQ);
     PacketWriteTLVDone (pak);
     SnacSend (serv, pak);
@@ -100,7 +101,7 @@ JUMP_SNAC_F(SnacSrvUserinfo)
         
     if (event->pak->ref == PEEK_REFID || event->pak->ref == PEEK_REFID2)
     {
-        rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
+        rl_log_for (cont->nick, COLCONTACT);
         rl_print (i18n (2017, "The user is online, but possibly invisible.\n"));
     }
 }
