@@ -27,7 +27,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: oscar_oldicq.c,v 1.6 2004/12/05 14:55:31 kuhlmann Exp $
+ * $Id: oscar_oldicq.c,v 1.12 2007-04-09 17:29:37 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -35,6 +35,7 @@
 #include "oscar_tlv.h"
 #include "oscar_snac.h"
 #include "oscar_oldicq.h"
+#include "oscar_roster.h"
 #include "packet.h"
 #include "contact.h"
 #include "connection.h"
@@ -46,6 +47,63 @@
 static Packet *SnacMetaC (Connection *serv, UWORD sub, UWORD type, UWORD ref);
 static void SnacMetaSend (Connection *serv, Packet *pak);
 
+#define PacketWriteMetaLNTS(p,t,s) { PacketWrite2 (p, t); PacketWriteLen (p); PacketWriteLNTS (p, s); PacketWriteLenDone (p); }
+#define PacketWriteMeta1(p,t,i)    { PacketWrite2 (p, t); PacketWrite2 (p, 1); PacketWrite1 (p, i); }
+#define PacketWriteMeta2(p,t,i)    { PacketWrite2 (p, t); PacketWrite2 (p, 2); PacketWrite2 (p, i); }
+#define PacketWriteMeta4(p,t,i)    { PacketWrite2 (p, t); PacketWrite2 (p, 4); PacketWrite4 (p, i); }
+
+#define META_TAG_UIN         310 /* LE32 */
+#define META_TAG_FIRST       320
+#define META_TAG_LAST        330
+#define META_TAG_NICK        340
+#define META_TAG_EMAIL       350 /* LNTS B */
+#define META_TAG_AGERANGE    360 /* LE16 LE16 */
+#define META_TAG_AGE         370 /* LE16 */
+#define META_TAG_GENDER      380 /* B */
+#define META_TAG_LANG        390 /* LE16 */
+#define META_TAG_HOME_CITY   400
+#define META_TAG_HOME_STATE  410
+#define META_TAG_HOME_CTRY   420 /* LE16 */
+#define META_TAG_WRK_COMP    430
+#define META_TAG_WRK_DEPT    440
+#define META_TAG_WRK_POS     450
+#define META_TAG_WRK_OCC     460 /* LE16 */
+#define META_TAG_AFFIL       470 /* LE16 LNTS */
+
+#define META_TAG_INTER       490 /* LE16 LNTS */
+
+#define META_TAG_PASTINFO    510 /* LE16 LNTS */
+
+#define META_TAG_HP_CAT      530 /* LE16 LNTS */
+#define META_TAG_HP_URL      531 /* LE16 LNTS */
+
+#define META_TAG_SRCH_STRING 550
+#define META_TAG_SRCH_ONLINE 560 /* B */
+#define META_TAG_BIRTHDAY    570 /* LE16 LE16 LE16 */
+
+#define META_TAG_ABOUT       600
+#define META_TAG_HOME_STREET 610
+#define META_TAG_HOME_ZIP    620 /* LE32 */
+#define META_TAG_HOME_PHONE  630
+#define META_TAG_HOME_FAX    640
+#define META_TAG_HOME_CELL   650
+#define META_TAG_WRK_STREET  660
+#define META_TAG_WRK_CITY    670
+#define META_TAG_WRK_STATE   680
+#define META_TAG_WRK_CTRY    690 /* LE16 */
+#define META_TAG_WRK_ZIP     700 /* LE32 */
+#define META_TAG_WRK_PHONE   710
+#define META_TAG_WRK_FAX     720
+#define META_TAG_WRK_URL     730
+
+#define META_TAG_AUTHREQ     760 /* B (yes, 760, NOT 780) */
+#define META_TAG_WEBAWARE    780 /* B */
+
+#define META_TAG_TZ          790 /* SB */
+#define META_TAG_ORIG_CITY   800
+#define META_TAG_ORIG_STATE  810
+#define META_TAG_ORIG_CTRY   820 /* LE16 */
+
 /*
  * Create meta request package.
  */
@@ -103,7 +161,7 @@ JUMP_SNAC_F(SnacSrvToicqerr)
                 rl_printf (i18n (2517, "I'm out of wisdom about the server's problem. It just didn't work out.\n"));
             EventD (oevent);
         }
-        else
+        else if ((pak->ref & 0xffff) != 0x4242)
         {
             rl_printf (i18n (2207, "Protocol error in command to old ICQ server: %d.\n"), err);
             rl_print (s_dump (pak->data + pak->rpos, pak->len - pak->rpos));
@@ -140,6 +198,7 @@ void SnacCliMetasetgeneral (Connection *serv, Contact *cont)
 {
     Packet *pak;
 
+#if 0
     pak = SnacMetaC (serv, 2000, META_SET_GENERAL_INFO, 0);
     if (cont->meta_general)
     {
@@ -175,18 +234,129 @@ void SnacCliMetasetgeneral (Connection *serv, Contact *cont)
         PacketWrite1    (pak, 0);
         PacketWrite1    (pak, 0);
     }
+#else
+    pak = SnacMetaC (serv, 2000, META_SAVE_INFO, 0);
+    if (cont->meta_general)
+    {
+        ContactMeta *ml;
+
+        PacketWriteMetaLNTS (pak, META_TAG_NICK, c_out (cont->meta_general->nick));
+        PacketWriteMetaLNTS (pak, META_TAG_FIRST, c_out (cont->meta_general->first));
+        PacketWriteMetaLNTS (pak, META_TAG_LAST, c_out (cont->meta_general->last));
+
+        PacketWrite2 (pak, META_TAG_EMAIL);
+        PacketWriteLen (pak);
+        PacketWriteLNTS (pak, c_out (cont->meta_general->email));
+        PacketWrite1 (pak, 0);
+        PacketWriteLenDone  (pak);
+
+        for (ml = cont->meta_email ; ml; ml = ml->next)
+        {
+            if (!ml->text || !*ml->text)
+                continue;
+            PacketWrite2 (pak, META_TAG_EMAIL);
+            PacketWriteLen (pak);
+            PacketWriteLNTS (pak, c_out (ml->text));
+            PacketWrite1 (pak, ml->data);
+            PacketWriteLenDone  (pak);
+        }
+        
+        PacketWriteMetaLNTS (pak, META_TAG_HOME_CITY, c_out (cont->meta_general->city));
+        PacketWriteMetaLNTS (pak, META_TAG_HOME_STATE, c_out (cont->meta_general->state));
+        PacketWriteMetaLNTS (pak, META_TAG_HOME_PHONE, c_out (cont->meta_general->phone));
+        PacketWriteMetaLNTS (pak, META_TAG_HOME_FAX, c_out (cont->meta_general->fax));
+        PacketWriteMetaLNTS (pak, META_TAG_HOME_STREET, c_out (cont->meta_general->street));
+        PacketWriteMetaLNTS (pak, META_TAG_HOME_CELL, c_out (cont->meta_general->cellular));
+        PacketWriteMeta4    (pak, META_TAG_HOME_ZIP, atoi (cont->meta_general->zip));
+
+        PacketWriteMeta2    (pak, META_TAG_HOME_CTRY, cont->meta_general->country);
+        PacketWriteMeta2    (pak, 835, 0);
+        PacketWriteMeta4    (pak, 621, 0);
+        PacketWriteMeta1    (pak, META_TAG_TZ, -cont->meta_general->tz/2);
+
+        PacketWriteMeta1    (pak, META_TAG_WEBAWARE, cont->meta_general->webaware);
+        PacketWriteMeta1    (pak, META_TAG_AUTHREQ, !cont->meta_general->auth);
+    }
+    else
+    {
+        PacketWriteMetaLNTS (pak, META_TAG_NICK, c_out (cont->nick));
+        PacketWriteMetaLNTS (pak, META_TAG_FIRST, "<unset>");
+        PacketWriteMetaLNTS (pak, META_TAG_LAST, "<unset>");
+
+        PacketWrite2 (pak, META_TAG_EMAIL);
+        PacketWriteLen (pak);
+        PacketWriteLNTS (pak, "");
+        PacketWrite1 (pak, 1);
+        PacketWriteLenDone  (pak);
+
+        PacketWriteMetaLNTS (pak, META_TAG_HOME_CITY, "");
+        PacketWriteMetaLNTS (pak, META_TAG_HOME_STATE, "");
+        PacketWriteMetaLNTS (pak, META_TAG_HOME_PHONE, "");
+        PacketWriteMetaLNTS (pak, META_TAG_HOME_FAX, "");
+        PacketWriteMetaLNTS (pak, META_TAG_HOME_STREET, "");
+        PacketWriteMetaLNTS (pak, META_TAG_HOME_CELL, "");
+        PacketWriteMetaLNTS (pak, META_TAG_HOME_ZIP, "");
+        PacketWriteMeta2    (pak, META_TAG_HOME_CTRY, 0);
+        PacketWriteMeta1    (pak, META_TAG_TZ, 0);
+        PacketWriteMeta1    (pak, META_TAG_WEBAWARE, 0);
+        PacketWriteMeta1    (pak, META_TAG_AUTHREQ, 0);
+    }
+
+#if 0    
+    PacketWriteMeta1    (pak, 760, 0);
+    PacketWriteMeta1    (pak, 780, 0);
+    PacketWriteMeta4    (pak, 620, 0);
+    PacketWriteMeta2    (pak, 420, 0);
+    PacketWriteMeta1    (pak, 790, -2);
+    PacketWriteMeta2    (pak, 820, 0);
+    PacketWriteMeta2    (pak, 370, 99);
+    PacketWriteMeta2    (pak, 380, 2);
+    PacketWriteMeta2    (pak, 531, 0);
+
+    PacketWrite2 (pak, 570);
+    PacketWrite2 (pak, 6);
+    PacketWrite2 (pak, 0);
+    PacketWrite2 (pak, 0);
+    PacketWrite2 (pak, 0);
+    
+    PacketWrite2 (pak, 350);
+    PacketWriteLen (pak);
+    PacketWriteLNTS (pak, "email@adresse");
+    PacketWrite1 (pak, 1);
+    PacketWriteLenDone  (pak);
+    
+    PacketWriteMeta4    (pak, 690, 0);
+    PacketWriteMeta2    (pak, 460, 0);
+    PacketWriteMeta4    (pak, 490, 0);
+    PacketWriteMeta4    (pak, 490, 0);
+    PacketWriteMeta4    (pak, 490, 0);
+    PacketWriteMeta4    (pak, 470, 0);
+    PacketWriteMeta4    (pak, 470, 0);
+    PacketWriteMeta4    (pak, 470, 0);
+    PacketWriteMeta4    (pak, 510, 0);
+    PacketWriteMeta4    (pak, 510, 0);
+    PacketWriteMeta4    (pak, 510, 0);
+#endif
+
+#endif
     SnacMetaSend    (serv, pak);
+    SnacCliSetlastupdate (serv);
 }
 
 /*
  * CLI_METASETABOUT - SNAC(15,2) - 2000/1030
  */
+
 void SnacCliMetasetabout (Connection *serv, const char *text)
 {
     Packet *pak;
-
+#if 0
     pak = SnacMetaC (serv, 2000, META_SET_ABOUT_INFO, 0);
     PacketWriteLNTS (pak, c_out (text));
+#else
+    pak = SnacMetaC (serv, 2000, META_SAVE_INFO, 0);
+    PacketWriteMetaLNTS (pak, META_TAG_ABOUT, c_out (text));
+#endif
     SnacMetaSend    (serv, pak);
 }
 
@@ -382,15 +552,15 @@ void SnacCliSearchwp (Connection *serv, const MetaWP *wp)
 void SnacCliSendsms (Connection *serv, const char *target, const char *text)
 {
     Packet *pak;
-    char buf[2000], tbuf[100];
+    char buf[2000];
     time_t t = time (NULL);
-
-    strftime (tbuf, sizeof (tbuf), "%a, %d %b %Y %H:%M:%S GMT", gmtime (&t));
+    
     snprintf (buf, sizeof (buf), "<icq_sms_message><destination>%s</destination>"
-             "<text>%s (%ld www.mICQ.org)</text><codepage>1252</codepage><senders_UIN>%ld</senders_UIN>"
+             "<text>%s (%s www.mICQ.org)</text><codepage>1252</codepage><senders_UIN>%s</senders_UIN>"
              "<senders_name>%s</senders_name><delivery_receipt>Yes</delivery_receipt>"
              "<time>%s</time></icq_sms_message>",
-             target, text, serv->uin, serv->uin, "mICQ", tbuf);
+             target, text, serv->screen, serv->screen, "mICQ",
+             s_strftime (&t, "%a, %d %b %Y %H:%M:%S GMT", 1));
 
     pak = SnacMetaC (serv, 2000, META_SEND_SMS, 0);
     PacketWriteB2      (pak, 1);
@@ -414,7 +584,7 @@ JUMP_SNAC_F(SnacSrvFromicqsrv)
     UDWORD len, uin, type /*, id*/;
     
     pak = event->pak;
-    tlv = TLVRead (pak, PacketReadLeft (pak));
+    tlv = TLVRead (pak, PacketReadLeft (pak), -1);
     if (tlv[1].str.len < 10)
     {
         SnacSrvUnknown (event);
