@@ -27,7 +27,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: oscar_register.c,v 1.8 2005/09/04 22:24:51 kuhlmann Exp $
+ * $Id: oscar_register.c,v 1.12 2006-11-27 19:17:31 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -53,10 +53,13 @@ JUMP_SNAC_F(SnacSrvRegrefused)
         rl_print (i18n (1792, "I'm sorry, AOL doesn't want to give us a new UIN, probably because of too many new UIN requests from this IP. Please try again later.\n"));
         exit (0);
     }
+    if (~serv->flags & CONN_CONFIGURED)
+    {
 #ifdef ENABLE_PEER2PEER
-    ConnectionD (serv->assoc);
+        ConnectionD (serv->assoc);
 #endif
-    ConnectionD (serv);
+        ConnectionD (serv);
+    }
 }
 
 /*
@@ -103,9 +106,9 @@ JUMP_SNAC_F(SnacSrvNewuin)
     uin = PacketReadAt4 (event->pak, 6 + 10 + 46);
     cont = ContactUIN (serv, uin);
     serv->uin = cont->uin;
-    s_repl (&serv->screen, s_sprintf ("%lu", cont->uin));
+    s_repl (&serv->screen, cont->screen);
     rl_print ("\n");
-    rl_printf (i18n (1762, "Your new UIN is: %ld.\n"), cont->uin);
+    rl_printf (i18n (2608, "Your new UIN is: %s.\n"), cont->screen);
     serv->flags |= CONN_CONFIGURED;
     if (serv->flags & CONN_WIZARD)
     {
@@ -118,7 +121,7 @@ JUMP_SNAC_F(SnacSrvNewuin)
         serv->flags |= CONN_AUTOLOGIN;
         serv->flags |= CONN_INITWP;
 
-        s_repl (&serv->contacts->name, s_sprintf ("contacts-icq8-%ld", cont->uin));
+        s_repl (&serv->contacts->name, s_sprintf ("contacts-icq8-%s", cont->screen));
         rl_print (i18n (1790, "Setup wizard finished. Congratulations to your new UIN!\n"));
         rl_printf ("\n%s", COLERROR);
         rl_print (i18n (2381, "I'll add the author of mICQ to your contact list for your convenience. Don't abuse this opportunity - please use the help command and make a serious attempt to read the man pages and the FAQ before asking questions.\n"));
