@@ -27,7 +27,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: oscar_service.c,v 1.8 2005/12/22 16:33:18 kuhlmann Exp $
+ * $Id: oscar_service.c,v 1.19 2006-11-06 21:14:13 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -61,7 +61,7 @@ static SNAC SNACv[] = {
     { 10,  1, NULL, NULL},
     { 11,  1, NULL, NULL},
     { 12,  1, NULL, NULL},
-    { 19,  4, NULL, NULL},
+    { 19,  4, NULL, NULL}, /* 5 */
     { 21,  1, NULL, NULL},
     { 34,  0, NULL, NULL},
     {  0,  0, NULL, NULL}
@@ -92,7 +92,7 @@ JUMP_SNAC_F (SnacSrvServiceerr)
     TLV *tlv;
     
     err = PacketReadB2 (event->pak);
-    tlv = TLVRead (event->pak, PacketReadLeft (event->pak));
+    tlv = TLVRead (event->pak, PacketReadLeft (event->pak), -1);
     
     if (tlv[8].str.len)
         DebugH (DEB_PROTOCOL, "Server returned error code %d, sub code %ld for service family.", err, tlv[8].nr);
@@ -205,7 +205,7 @@ JUMP_SNAC_F(SnacServerpause)
 #endif
     ConnectionInitServer(serv);
     for (i = 0; (cont = ContactIndex (cg, i)); i++)
-        cont->status = STATUS_ICQOFFLINE;
+        cont->status = ims_offline;
 }
 
 /*
@@ -225,17 +225,18 @@ JUMP_SNAC_F(SnacSrvReplyinfo)
     Contact *cont;
     Packet *pak;
     TLV *tlv;
-    UDWORD status;
+    UDWORD ostat, tlvc;
+    status_t status;
     
     pak = event->pak;
     cont = PacketReadCont (pak, serv);
     
-    if (cont->uin != serv->uin)
-        rl_printf (i18n (1907, "Warning: Server thinks our UIN is %ld, when it is %ld.\n"),
-                  cont->uin, serv->uin);
+    if (strcmp (cont->screen, serv->screen))
+        rl_printf (i18n (2609, "Warning: Server thinks our UIN is %s, when it is %s.\n"),
+                  cont->screen, serv->screen);
     PacketReadB2 (pak);
-    PacketReadB2 (pak);
-    tlv = TLVRead (pak, PacketReadLeft (pak));
+    tlvc = PacketReadB2 (pak);
+    tlv = TLVRead (pak, PacketReadLeft (pak), tlvc);
     if (tlv[10].str.len)
     {
         serv->our_outside_ip = tlv[10].nr;
@@ -246,11 +247,13 @@ JUMP_SNAC_F(SnacSrvReplyinfo)
     }
     if (tlv[6].str.len)
     {
-        status = tlv[6].nr;
+        ostat = tlv[6].nr;
+        status = IcqToStatus (ostat);
         if (status != serv->status)
         {
             serv->status = status;
-            rl_printf ("%s %s\n", s_now, s_status (serv->status));
+            serv->nativestatus = ostat;
+            rl_printf ("%s %s\n", s_now, s_status (serv->status, ostat));
         }
     }
     /* TLV 1 c f 2 3 ignored */
@@ -258,11 +261,12 @@ JUMP_SNAC_F(SnacSrvReplyinfo)
     
     if (~serv->connect & CONNECT_OK)
     {
-        SnacCliSetrandom (serv, prG->chat);
+        if (prG->chat > 0)
+            SnacCliSetrandom (serv, prG->chat);
         serv->connect = CONNECT_OK | CONNECT_SELECT_R;
         QueueEnqueueData (serv, QUEUE_SRV_KEEPALIVE, 0, time (NULL) + 30,
                           NULL, event->cont, NULL, &SrvCallBackKeepalive);
-        if ((event = QueueDequeue2 (serv, QUEUE_DEP_OSCARLOGIN, 0, NULL)))
+        if ((event = QueueDequeue2 (serv, QUEUE_DEP_WAITLOGIN, 0, NULL)))
         {
             event->due = time (NULL) + 5;
             QueueEnqueue (event);
@@ -325,22 +329,26 @@ JUMP_SNAC_F(SnacSrvFamilies2)
  *
  * action: 1 = send status 2 = send connection info (3 = both)
  */
-void SnacCliSetstatus (Connection *serv, UDWORD status, UWORD action)
+void SnacCliSetstatus (Connection *serv, status_t status, UWORD action)
 {
     Packet *pak;
+    UDWORD ostat = IcqFromStatus (status);
+    statusflag_t flags = imf_none;
     
     if (ConnectionPrefVal (serv, CO_WEBAWARE))
-        status |= STATUSF_ICQWEBAWARE;
+        flags |= imf_web;
     if (ConnectionPrefVal (serv, CO_DCAUTH))
-        status |= STATUSF_ICQDC_AUTH;
+        flags |= imf_dcauth;
     if (ConnectionPrefVal (serv, CO_DCCONT))
-        status |= STATUSF_ICQDC_CONT;
+        flags |= imf_dccont;
+    
+    ostat |= IcqFromFlags (flags);
     
     pak = SnacC (serv, 1, 0x1e, 0, 0);
-    if ((action & 1) && (status & STATUSF_ICQINV))
+    if ((action & 1) && ContactIsInv (status))
         SnacCliAddvisible (serv, 0);
     if (action & 1)
-        PacketWriteTLV4 (pak, 6, status);
+        PacketWriteTLV4 (pak, 6, ostat);
     if (action & 2)
     {
         PacketWriteB2 (pak, 0x0c); /* TLV 0C */
@@ -371,7 +379,6 @@ void SnacCliSetstatus (Connection *serv, UDWORD status, UWORD action)
         PacketWriteTLV2 (pak, 8, 0);
     }
     SnacSend (serv, pak);
-    if ((action & 1) && (~status & STATUSF_ICQINV))
+    if ((action & 1) && !ContactIsInv (status))
         SnacCliAddinvis (serv, 0);
 }
-
