@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: oscar_snac.c,v 1.7 2005/12/11 14:46:10 kuhlmann Exp $
+ * $Id: oscar_snac.c,v 1.8 2006-09-24 12:13:27 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -207,7 +207,7 @@ void SnacPrint (Packet *pak)
     len  = PacketReadAtB2 (pak, pak->rpos);
 
     rl_printf ("%s %sSNAC     (%x,%x) [%s] flags %x ref %lx",
-             s_dumpnd (pak->data + 6, flag & 0x8000 ? 10 + len : 10),
+             s_dumpnd (pak->data + 6, flag & 0x8000 ? 10 + len + 2 : 10),
              COLDEBUG, fam, cmd, SnacName (fam, cmd), flag, ref);
     if (flag & 0x8000)
     {
