@@ -27,7 +27,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: packet.c,v 1.63 2005/12/05 21:28:38 kuhlmann Exp $
+ * $Id: packet.c,v 1.64 2006-09-23 13:33:36 kuhlmann Exp $
  *
  * Note: alle read strings need to be free()ed.
  */
@@ -335,11 +335,9 @@ void PacketWriteLLNTS (Packet *pak, const char *data)
 
 void PacketWriteCont (Packet *pak, Contact *cont)
 {
-    const char *str;
-    
-    str = cont->screen ? cont->screen : s_sprintf ("%ld", cont->uin);
-    PacketWrite1 (pak, strlen (str));
-    PacketWriteData (pak, str, strlen (str));
+    int len = strlen (cont->screen);
+    PacketWrite1 (pak, len);
+    PacketWriteData (pak, cont->screen, len);
 }
 
 void PacketWriteTLV (Packet *pak, UDWORD type)
