@@ -28,7 +28,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: preferences.c,v 1.51 2005/12/20 21:04:47 kuhlmann Exp $
+ * $Id: preferences.c,v 1.53 2006-09-19 16:54:59 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -86,7 +86,7 @@ const char *PrefDefUserDirReal (Preferences *pref)
     if (!pref->defaultbasedir)
     {
         const char *home;
-#if HAVE_GETPWUID && HAVE_GETUID
+#if HAVE_GETPWUID && HAVE_GETUID && !__AMIGA__
         struct passwd *pwd = getpwuid (getuid ());
 #endif
 #if defined(_WIN32) || (defined(__CYGWIN__) && defined(_X86_))
@@ -238,7 +238,7 @@ BOOL PrefLoad (Preferences *pref)
         case 3:
             for (i = 0; (conn = ConnectionNr (i)); i++)
             {
-                cont = ContactFind (conn->contacts, 0, 82274703, NULL);
+                cont = ContactFindUIN (conn->contacts, 82274703);
                 if (cont)
                     OptSetVal (&cont->copts, CO_WANTSBL, 0);
             }
