@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: remote.c,v 1.22 2005/02/11 23:33:11 kuhlmann Exp $
+ * $Id: remote.c,v 1.23 2006-04-05 20:03:23 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -58,6 +58,8 @@ static void RemoteClose (Connection *remo);
  */
 Event *RemoteOpen (Connection *remo)
 {
+    if (!remo->pref_server)
+        remo->pref_server = strdup ("scripting");
     s_repl (&remo->server, s_realpath (remo->pref_server));
 
     rl_printf (i18n (2223, "Opening scripting FIFO at %s... "), s_wordquote (remo->server));
