@@ -1,4 +1,4 @@
-/* $Id: server.c,v 1.82 2005/12/22 16:33:18 kuhlmann Exp $ */
+/* $Id: server.c,v 1.94 2007-04-17 22:06:20 kuhlmann Exp $ */
 /* Copyright: This file may be distributed under version 2 of the GPL licence. */
 
 #include "micq.h"
@@ -26,6 +26,9 @@
 #include "tcp.h"
 #include "packet.h"
 #include "conv.h"
+#include "oscar_service.h"
+#include "oscar_roster.h"
+#include "jabber_base.h"
 
 static void CallbackMeta (Event *event);
 static void CallbackTogvis (Event *event);
@@ -34,7 +37,8 @@ static void CallbackTogvis (Event *event)
 {
     if (!event)
         return;
-    if (!event->cont || !event->conn || ContactPrefVal (event->cont, CO_INTIMATE) || !(event->conn->status & STATUSF_ICQINV))
+    if (!event->cont || !event->conn || ContactPrefVal (event->cont, CO_INTIMATE)
+        || !ContactIsInv (event->conn->status))
     {
         EventD (event);
         return;
@@ -58,7 +62,7 @@ UBYTE IMCliMsg (Connection *conn, Contact *cont, Opt *opt)
     if (!OptGetVal (opt, CO_MSGTRANS, &opt_trans))
         OptSetVal (opt, CO_MSGTRANS, opt_trans = CV_MSGTRANS_ANY);
     
-    putlog (conn, NOW, cont, STATUS_ICQONLINE, 
+    putlog (conn, NOW, cont, ims_online, 0,
             opt_type == MSG_AUTO ? LOG_AUTO : LOG_SENT, opt_type, opt_text);
 
     return IMCliReMsg (conn, cont, opt);
@@ -81,7 +85,8 @@ UBYTE IMCliReMsg (Connection *conn, Contact *cont, Opt *opt)
     if (!OptGetVal (opt, CO_MSGTRANS, &opt_trans))
         OptSetVal (opt, CO_MSGTRANS, opt_trans = CV_MSGTRANS_ANY);
     
-    if ((conn->status & STATUSF_ICQINV)  && !ContactPrefVal (cont, CO_INTIMATE)
+    if (conn->type == TYPE_SERVER
+        && ContactIsInv (conn->status)  && !ContactPrefVal (cont, CO_INTIMATE)
         && !(opt_type & MSGF_GETAUTO) && !ContactPrefVal (cont, CO_HIDEFROM)
         && (reveal_time = ContactPrefVal (cont, CO_REVEALTIME)))
     {
@@ -108,7 +113,10 @@ UBYTE IMCliReMsg (Connection *conn, Contact *cont, Opt *opt)
     if (opt_trans & CV_MSGTRANS_ICQv8)
         if (conn->connect & CONNECT_OK && conn->type == TYPE_SERVER)
             if (RET_IS_OK (ret = SnacCliSendmsg (conn, cont, opt_text, opt_type, 0)))
+            {
+                OptD (opt);
                 return ret;
+            }
     OptSetVal (opt, CO_MSGTRANS, opt_trans &= ~CV_MSGTRANS_ICQv8);
     if (opt_trans & CV_MSGTRANS_ICQv5)
         if (conn->connect & CONNECT_OK && conn->type == TYPE_SERVER_OLD)
@@ -117,30 +125,48 @@ UBYTE IMCliReMsg (Connection *conn, Contact *cont, Opt *opt)
             OptD (opt);
             return RET_OK;
         }
+#if ENABLE_XMPP
+    if (opt_trans & CV_MSGTRANS_XMPP)
+        if (conn->connect & CONNECT_OK && conn->type == TYPE_XMPP_SERVER)
+            if (RET_IS_OK (ret = XMPPSendmsg (conn, cont, opt_text, opt_type)))
+                return ret;
+#endif
+    ret = RET_OK;
+    if (conn->type == TYPE_SERVER && opt_type == MSG_AUTH_DENY)
+        SnacCliAuthorize (conn, cont, 0, opt_text);
+    else if (conn->type == TYPE_SERVER && opt_type == MSG_AUTH_REQ)
+        SnacCliReqauth (conn, cont, opt_text);
+    else if (conn->type == TYPE_SERVER && opt_type == MSG_AUTH_ADDED)
+        SnacCliGrantauth (conn, cont);
+    else if (conn->type == TYPE_SERVER && opt_type == MSG_AUTH_GRANT)
+        SnacCliAuthorize (conn, cont, 1, NULL);
+    else
+        ret = RET_INPR;
     OptD (opt);
-    return RET_FAIL;
+    return ret;
 }
 
 void IMCliInfo (Connection *conn, Contact *cont, int group)
 {
-    UDWORD ref;
+    UDWORD ref = 0;
     if (cont)
     {
         cont->updated = 0;
         if (conn->type == TYPE_SERVER)
             ref = SnacCliMetareqinfo (conn, cont);
-        else
+        else if (conn->type == TYPE_SERVER_OLD)
             ref = CmdPktCmdMetaReqInfo (conn, cont);
     }
     else
     {
         if (conn->type == TYPE_SERVER)
             ref = SnacCliSearchrandom (conn, group);
-        else
+        else if (conn->type == TYPE_SERVER_OLD)
             ref = CmdPktCmdRandSearch (conn, group);
     }
-    QueueEnqueueData (conn, QUEUE_REQUEST_META, ref, time (NULL) + 60, NULL,
-                      cont, NULL, &CallbackMeta);
+    if (ref)
+        QueueEnqueueData (conn, QUEUE_REQUEST_META, ref, time (NULL) + 60, NULL,
+                          cont, NULL, &CallbackMeta);
 }
 
 static void CallbackMeta (Event *event)
@@ -150,7 +176,9 @@ static void CallbackMeta (Event *event)
     assert (event && event->conn && event->conn->type & TYPEF_ANY_SERVER);
 
     cont = event->cont;
-    if ((cont->updated != UP_INFO) && !(event->flags & QUEUE_FLAG_CONSIDERED))
+    if (!cont)
+        EventD (event);
+    else if ((cont->updated != UP_INFO) && !(event->flags & QUEUE_FLAG_CONSIDERED))
         QueueEnqueue (event);
     else
     {
@@ -162,3 +190,24 @@ static void CallbackMeta (Event *event)
         EventD (event);
     }
 }
+
+void IMSetStatus (Connection *conn, Contact *cont, status_t status, const char *msg)
+{
+    if (~conn->connect & CONNECT_OK)
+    {
+        conn->status = status;
+        conn->nativestatus = 0;
+    }
+    else if (conn->type == TYPE_SERVER)
+        SnacCliSetstatus (conn, status, 1);
+    else if (conn->type == TYPE_SERVER_OLD)
+    {
+        CmdPktCmdStatusChange (conn, status);
+        rl_printf ("%s %s\n", s_now, s_status (conn->status, conn->nativestatus));
+    }
+#if ENABLE_XMPP
+    else if (conn->type == TYPE_XMPP_SERVER)
+        XMPPSetstatus (conn, cont, status, msg);
+#endif
+}
+
