@@ -5,7 +5,7 @@
  *  Lots of changes from Rüdiger Kuhlmann. File transfer by Rüdiger Kuhlmann.
  *  This file may be distributed under version 2 of the GPL licence.
  *
- * $Id: tcp.c,v 1.203 2005/12/22 16:33:18 kuhlmann Exp $
+ * $Id: tcp.c,v 1.209 2007-04-17 19:38:00 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -30,6 +30,7 @@
 #include "oscar_snac.h"
 #include "oscar_service.h"
 #include "oscar_icbm.h"
+#include "oscar_base.h"
 
 #include <unistd.h>
 #include <assert.h>
@@ -208,7 +209,7 @@ void TCPDispatchReconn (Connection *peer)
     {
         Contact *cont = peer->cont;
 
-        rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
+        rl_log_for (cont->nick, COLCONTACT);
         rl_print  (i18n (2023, "Direct connection closed by peer.\n"));
     }
     if (peer->close)
@@ -252,7 +253,7 @@ void TCPDispatchMain (Connection *list)
                 list->connect &= ~CONNECT_SELECT_W; /* & ~CONNECT_SELECT_X; */
                 if (list->type == TYPE_MSGLISTEN && list->parent && list->parent->type == TYPE_SERVER
                     && (list->parent->connect & CONNECT_OK))
-                    SnacCliSetstatus (list->parent, 0, 2);
+                    SnacCliSetstatus (list->parent, ims_online, 2);
                 break;
             case 2:
                 list->connect = 0;
@@ -338,7 +339,7 @@ void TCPDispatchConn (Connection *peer)
             return;
         }
         
-        DebugH (DEB_TCP, "Conn: uin %ld nick %s state %x", cont->uin, cont->nick, peer->connect);
+        DebugH (DEB_TCP, "Conn: uin %s nick %s state %x", cont->screen, cont->nick, peer->connect);
 
         switch (peer->connect & CONNECT_MASK)
         {
@@ -356,7 +357,7 @@ void TCPDispatchConn (Connection *peer)
                 
                 if (prG->verbose)
                 {
-                    rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
+                    rl_log_for (cont->nick, COLCONTACT);
                     rl_printf (i18n (2522, "Opening TCP connection at %s:%s%ld%s... "),
                               s_wordquote (s_ip (peer->ip)), COLQUOTE, peer->port, COLNONE);
                 }
@@ -376,7 +377,7 @@ void TCPDispatchConn (Connection *peer)
                 
                 if (prG->verbose)
                 {
-                    rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
+                    rl_log_for (cont->nick, COLCONTACT);
                     rl_printf (i18n (2522, "Opening TCP connection at %s:%s%ld%s... "),
                               s_wordquote (s_ip (peer->ip)), COLQUOTE, peer->port, COLNONE);
                 }
@@ -401,7 +402,7 @@ void TCPDispatchConn (Connection *peer)
             case 4:
                 if (prG->verbose)
                 {
-                    rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
+                    rl_log_for (cont->nick, COLCONTACT);
                     rl_printf (i18n (2522, "Opening TCP connection at %s:%s%ld%s... "),
                               s_wordquote (s_ip (peer->ip)), COLQUOTE, peer->port, COLNONE);
                     rl_print (i18n (1785, "success.\n"));
@@ -462,8 +463,8 @@ void TCPDispatchShake (Connection *peer)
             return;
         }
         
-        DebugH (DEB_TCP, "HS %d uin %ld nick %s state %d pak %p peer %p",
-                peer->sok, cont ? cont->uin : 0, cont ? cont->nick : "<>", peer->connect, pak, peer);
+        DebugH (DEB_TCP, "HS %d uin %s nick %s state %d pak %p peer %p",
+                peer->sok, cont ? cont->screen : 0, cont ? cont->nick : "<>", peer->connect, pak, peer);
 
         switch (peer->connect & CONNECT_MASK)
         {
@@ -546,7 +547,7 @@ void TCPDispatchShake (Connection *peer)
                 EventD (QueueDequeue (peer, QUEUE_TCP_TIMEOUT, peer->ip));
                 if (prG->verbose)
                 {
-                    rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
+                    rl_log_for (cont->nick, COLCONTACT);
                     rl_print  (i18n (1833, "Peer to peer TCP connection established.\n"));
                 }
                 peer->connect = CONNECT_OK | CONNECT_SELECT_R;
@@ -830,8 +831,8 @@ static void TCPSendInitv6 (Connection *peer)
     PacketWrite4  (pak, peer->parent->port);                   /* our (other) port */
     PacketWrite4  (pak, peer->our_session);                    /* session id       */
 
-    DebugH (DEB_TCP, "HS %d uin %ld CONNECT pak %p peer %p",
-                     peer->sok, peer->cont->uin, pak, peer);
+    DebugH (DEB_TCP, "HS %d uin %s CONNECT pak %p peer %p",
+                     peer->sok, peer->cont->screen, pak, peer);
 
     PeerPacketSend (peer, pak);
     PacketD (pak);
@@ -885,8 +886,8 @@ static void TCPSendInit (Connection *peer)
     PacketWrite4  (pak, 0x00000003);
     PacketWrite4  (pak, 0);
 
-    DebugH (DEB_TCP, "HS %d uin %ld CONNECTv8 pak %p peer %p",
-            peer->sok, peer->cont->uin, pak, peer);
+    DebugH (DEB_TCP, "HS %d uin %s CONNECTv8 pak %p peer %p",
+            peer->sok, peer->cont->screen, pak, peer);
 
     PeerPacketSend (peer, pak);
     PacketD (pak);
@@ -911,8 +912,8 @@ static void TCPSendInitAck (Connection *peer)
     PacketWrite1 (pak, 0);
     PacketWrite2 (pak, 0);
 
-    DebugH (DEB_TCP, "HS %d uin %ld INITACK pak %p peer %p",
-            peer->sok, peer->cont->uin, pak, peer);
+    DebugH (DEB_TCP, "HS %d uin %s INITACK pak %p peer %p",
+            peer->sok, peer->cont->screen, pak, peer);
 
     PeerPacketSend (peer, pak);
     PacketD (pak);
@@ -941,8 +942,8 @@ static void TCPSendInit2 (Connection *peer)
     PacketWrite4 (pak, 0);
     PacketWrite4 (pak, (peer->connect & 16) ? 0 : 0x40001);
 
-    DebugH (DEB_TCP, "HS %d uin %ld INITMSG pak %p peer %p",
-            peer->sok, peer->cont->uin, pak, peer);
+    DebugH (DEB_TCP, "HS %d uin %s INITMSG pak %p peer %p",
+            peer->sok, peer->cont->screen, pak, peer);
 
     PeerPacketSend (peer, pak);
     PacketD (pak);
@@ -1024,8 +1025,8 @@ static Connection *TCPReceiveInit (Connection *peer, Packet *pak)
         if (iip)      cont->dc->ip_loc = iip;
         if (tcpflag)  cont->dc->type = tcpflag;
 
-        DebugH (DEB_TCP, "HS %d uin %ld nick %s init pak %p peer %p: ver %04x:%04x port %ld uin %ld SID %08lx type %x",
-                peer->sok, cont->uin, cont->nick, pak, peer, peer->version, len, port, uin, sid, peer->type);
+        DebugH (DEB_TCP, "HS %d uin %s nick %s init pak %p peer %p: ver %04x:%04x port %ld uin %ld SID %08lx type %x",
+                peer->sok, cont->screen, cont->nick, pak, peer, peer->version, len, port, uin, sid, peer->type);
 
         for (i = 0; (peer2 = ConnectionNr (i)); i++)
             if (     peer2->type == peer->type && peer2->parent == peer->parent
@@ -1268,6 +1269,8 @@ UBYTE PeerSendMsg (Connection *list, Contact *cont, Opt *opt)
 
     if (!cont->dc || !cont->dc->port)
         return RET_DEFER;
+    if (!cont->uin)
+        return RET_DEFER;
     if (cont->uin == list->parent->uin || !(list->connect & CONNECT_MASK))
         return RET_DEFER;
     if (!cont->dc->ip_loc && !cont->dc->ip_rem)
@@ -1341,6 +1344,8 @@ BOOL TCPSendFiles (Connection *list, Contact *cont, const char *description, con
         return TRUE;
     if (count < 0)
         return FALSE;
+    if (!cont->uin)
+        return FALSE;
     if (!cont->dc || !cont->dc->port)
         return FALSE;
     
@@ -1388,8 +1393,7 @@ BOOL TCPSendFiles (Connection *list, Contact *cont, const char *description, con
         }
         else
         {
-            rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick),
-                      cont->nick, COLNONE);
+            rl_log_for (cont->nick, COLCONTACT);
             rl_printf (i18n (2091, "Queueing %s as %s for transfer.\n"), files[i], as[i]);
             if (sum)
                 s_catn (&filenames, ", ", 2);
@@ -1498,7 +1502,7 @@ static void TCPCallBackResend (Event *event)
         if (peer->connect & CONNECT_OK)
         {
             if (event->attempts > 1)
-                IMIntMsg (cont, peer->parent->parent, NOW, STATUS_ICQOFFLINE, INT_MSGTRY_DC, opt_text, NULL);
+                IMIntMsg (cont, peer->parent->parent, NOW, ims_offline, INT_MSGTRY_DC, opt_text, NULL);
 
             if (event->attempts < 2)
                 PeerPacketSend (peer, pak);
@@ -1565,7 +1569,9 @@ static void TCPCallBackReceive (Event *event)
     strc_t ctmp, ctext, cname, creason;
     char *tmp, *text, *name;
     UWORD cmd, type, seq, port /*, unk*/;
-    UDWORD /*len,*/ status /*, flags, xtmp1, xtmp2, xtmp3*/;
+  /*  UDWORD len, flags, xtmp1, xtmp2, xtmp3; */
+    status_t status;
+    UDWORD ostat;
     const char *opt_text;
     UDWORD opt_origin, opt_type;
     
@@ -1596,7 +1602,8 @@ static void TCPCallBackReceive (Event *event)
             /*xtmp2*/PacketRead4 (pak);
             /*xtmp3*/PacketRead4 (pak);
             type   = PacketRead2 (pak);
-            status = PacketRead2 (pak);
+            ostat  = PacketRead2 (pak);
+            status = IcqToStatus (ostat);
             /*flags*/PacketRead2 (pak);
             ctmp   = PacketReadL2Str (pak, NULL);
             
@@ -1622,7 +1629,7 @@ static void TCPCallBackReceive (Event *event)
             {
                 case MSG_NORM:
                 case MSG_URL:
-                    IMIntMsg (cont, peer, NOW, STATUS_ICQOFFLINE, opt_origin == CV_ORIGIN_dc ? INT_MSGACK_DC : INT_MSGACK_SSL, opt_text, NULL);
+                    IMIntMsg (cont, serv, NOW, ims_offline, opt_origin == CV_ORIGIN_dc ? INT_MSGACK_DC : INT_MSGACK_SSL, opt_text, NULL);
                     if (~cont->oldflags & CONT_SEENAUTO && strlen (tmp) && strcmp (tmp, opt_text))
                     {
                         IMSrvMsg (cont, serv, NOW, OptSetVals (NULL, CO_ORIGIN, opt_origin, CO_MSGTYPE, MSG_AUTO, CO_MSGTEXT, tmp, 0));
@@ -1637,12 +1644,12 @@ static void TCPCallBackReceive (Event *event)
                 case MSGF_GETAUTO | MSG_GET_FFC:
                 case MSGF_GETAUTO | MSG_GET_VER:
                     IMSrvMsg (cont, serv, NOW, OptSetVals (NULL, CO_ORIGIN, opt_origin, CO_MSGTYPE, opt_type,
-                              CO_MSGTEXT, tmp, CO_STATUS, status & ~MSGF_GETAUTO, 0));
+                              CO_MSGTEXT, tmp, CO_STATUS, IcqToStatus (ostat & ~MSGF_GETAUTO), 0));
                     break;
 
                 case MSG_FILE:
                     port = PacketReadB2 (pak);
-                    if (PeerFileAccept (peer, status, port))
+                    if (PeerFileAccept (peer, ostat, port))
                         IMIntMsg (cont, serv, NOW, status, INT_FILE_ACKED, tmp,
                           OptSetVals (NULL, CO_PORT, port, CO_MSGTEXT, opt_text, 0));
                     else
@@ -1654,11 +1661,11 @@ static void TCPCallBackReceive (Event *event)
                  * MSG_SSL_CLOSE is not handled here.
                  */
                 case MSG_SSL_OPEN:
-                    if (!status && !strcmp (tmp, "1"))
+                    if (!ostat && !strcmp (tmp, "1"))
                         ssl_connect (peer, 1);
                     else
                     {
-                        DebugH (DEB_SSL, "%s (%ld) is not SSL capable", cont->nick, cont->uin);
+                        DebugH (DEB_SSL, "%s (%s) is not SSL capable", cont->nick, cont->screen);
 #ifdef ENABLE_TCL
                         TCLEvent (cont, "ssl", "incapable");
 #endif
@@ -1692,7 +1699,7 @@ static void TCPCallBackReceive (Event *event)
                     switch (cmd)
                     {
                         case 0x0029:
-                            if (PeerFileAccept (peer, status, port))
+                            if (PeerFileAccept (peer, ostat, port))
                                 IMIntMsg (cont, serv, NOW, status, INT_FILE_ACKED, tmp,
                                     OptSetVals (NULL, CO_PORT, port, CO_MSGTEXT, opt_text, 0));
                             else
@@ -1710,8 +1717,8 @@ static void TCPCallBackReceive (Event *event)
 
                     /* fall through */
                 default:
-                    DebugH (DEB_TCP, "ACK %d uin %ld nick %s pak %p peer %p seq %04x",
-                                     peer->sok, cont->uin, cont->nick, oldevent->pak, peer, seq);
+                    DebugH (DEB_TCP, "ACK %d uin %s nick %s pak %p peer %p seq %04x",
+                                     peer->sok, cont->screen, cont->nick, oldevent->pak, peer, seq);
             }
             free (tmp);
             EventD (oldevent);
