@@ -3,7 +3,7 @@
  *
  * Copyright: This file may be distributed under version 2 of the GPL licence.
  *
- * $Id: util_io.c,v 1.83 2005/02/27 19:27:01 kuhlmann Exp $
+ * $Id: util_io.c,v 1.88 2007-04-17 22:18:57 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -686,7 +686,8 @@ static void UtilIOTOConn (Event *event)
  */
 Packet *UtilIOReceiveTCP (Connection *conn)
 {
-    int off, len, rc;
+    ssl_errno_t ssl_rc = 0;
+    int rc, off, len;
     Packet *pak;
     
     if (!(conn->connect & CONNECT_MASK))
@@ -721,10 +722,11 @@ Packet *UtilIOReceiveTCP (Connection *conn)
 #if defined(SIGPIPE)
         signal (SIGPIPE, SIG_IGN);
 #endif
-        rc = dc_read (conn, pak->data + pak->len, len - pak->len);
-        if (rc <= 0)
+        rc = 0;
+        ssl_rc = dc_read (conn, pak->data + pak->len, len - pak->len);
+        if (ssl_rc <= 0)
         {
-            if (!rc)
+            if (!ssl_rc)
                 rc = UtilIOError (conn);
             else
                 rc = errno;
@@ -738,7 +740,7 @@ Packet *UtilIOReceiveTCP (Connection *conn)
                 rc = ECONNRESET;
             break;
         }
-        pak->len += rc;
+        pak->len += ssl_rc;
         if (pak->len < len)
             return NULL;
         if (len == off)
@@ -765,11 +767,10 @@ Packet *UtilIOReceiveTCP (Connection *conn)
         if (prG->verbose || conn->type & TYPEF_ANY_SERVER)
         {
             Contact *cont;
-            
             if ((cont = conn->cont))
             {
-                rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
-                rl_printf (i18n (1878, "Error while reading from socket: %s (%d)\n"), dc_strerror (rc), rc);
+                rl_log_for (cont->nick, COLCONTACT);
+                rl_printf (i18n (1878, "Error while reading from socket: %s (%d, %d)\n"), dc_strerror (conn, ssl_rc, rc), ssl_rc, rc);
             }
         }
         conn->connect = 0;
@@ -846,8 +847,8 @@ Packet *UtilIOReceiveF (Connection *conn)
             Contact *cont;
             if ((cont = conn->cont))
             {
-                rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
-                rl_printf (i18n (1878, "Error while reading from socket: %s (%d)\n"), strerror (rc), rc);
+                rl_log_for (cont->nick, COLCONTACT);
+                rl_printf (i18n (1878, "Error while reading from socket: %s (%d, %d)\n"), strerror (rc), rc, rc);
             }
         }
         conn->connect = 0;
@@ -864,6 +865,7 @@ Packet *UtilIOReceiveF (Connection *conn)
 BOOL UtilIOSendTCP (Connection *conn, Packet *pak)
 {
     UBYTE *data;
+    ssl_errno_t ssl_rc = 0;
     int rc, bytessend = 0;
 
     data = (void *) &pak->data;
@@ -878,8 +880,8 @@ BOOL UtilIOSendTCP (Connection *conn, Packet *pak)
 
         for ( ; pak->len > pak->rpos; pak->rpos += bytessend)
         {
-            bytessend = dc_write (conn, data + pak->rpos, pak->len - pak->rpos);
-            if (bytessend <= 0)
+            bytessend = ssl_rc = dc_write (conn, data + pak->rpos, pak->len - pak->rpos);
+            if (ssl_rc <= 0)
                 break;
         }
         if (bytessend <= 0)
@@ -909,8 +911,8 @@ BOOL UtilIOSendTCP (Connection *conn, Packet *pak)
             
             if ((cont = conn->cont))
             {
-                rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
-                rl_printf (i18n (1878, "Error while reading from socket: %s (%d)\n"), dc_strerror (rc), rc);
+                rl_log_for (cont->nick, COLCONTACT);
+                rl_printf (i18n (1878, "Error while reading from socket: %s (%d, %d)\n"), dc_strerror (conn, ssl_rc, rc), ssl_rc, rc);
             }
         }
         conn->connect = 0;
@@ -987,7 +989,7 @@ strc_t UtilIOReadline (FILE *fd)
     static str_s str;
     char *p;
     
-    s_init (&str, "", 128);
+    s_init (&str, "", 256);
     while (1)
     {
         str.txt[str.max - 2] = 0;
