@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: util_ssl.c,v 1.25 2005/07/15 21:23:21 kuhlmann Exp $
+ * $Id: util_ssl.c,v 1.29 2006-12-05 21:36:31 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -165,6 +165,18 @@ static void openssl_info_callback (SSL *s, int where, int ret)
 }
 #endif /* ENABLE_GNUTLS */
 
+const char *ssl_strerror (Connection *conn, ssl_errno_t err, int e)
+{
+#if ENABLE_GNUTLS
+    if (conn->ssl_status == SSL_STATUS_OK && err)
+        return gnutls_strerror (err);
+#else
+    if (conn->ssl_status == SSL_STATUS_OK && err)
+        return "OpenSSL error";
+#endif
+    return strerror (e);
+}
+
 /* 
  * Initialize ssl library
  *
@@ -280,13 +292,13 @@ int ssl_supported (Connection *conn DEBUGPARAM)
           || HAS_CAP (cont->caps, CAP_LICQNEW)
           || (cont->dc && (cont->dc->id1 & 0xFFFF0000) == LICQ_WITHSSL)))
     {
-        Debug (DEB_SSL, "%s (%ld) is no SSL candidate", cont->nick, cont->uin);
+        Debug (DEB_SSL, "%s (%s) is no SSL candidate", cont->nick, cont->screen);
         TCLEvent (cont, "ssl", "no_candidate");
         return 0;
     }
 
     conn->ssl_status = status_save;
-    Debug (DEB_SSL, "%s (%ld) is an SSL candidate", cont->nick, cont->uin);
+    Debug (DEB_SSL, "%s (%s) is an SSL candidate", cont->nick, cont->screen);
     TCLEvent (cont, "ssl", "candidate");
     return 1;
 }
@@ -327,7 +339,10 @@ int ssl_connect (Connection *conn, BOOL is_client DEBUGPARAM)
     gnutls_set_default_priority (conn->ssl);
     gnutls_kx_set_priority (conn->ssl, kx_prio);
 
-    ret = gnutls_credentials_set (conn->ssl, GNUTLS_CRD_ANON, is_client ? client_cred : server_cred);
+    if (is_client)
+        ret = gnutls_credentials_set (conn->ssl, GNUTLS_CRD_ANON, client_cred);
+    else
+        ret = gnutls_credentials_set (conn->ssl, GNUTLS_CRD_ANON, server_cred);
     if (ret)
         TCLEvent (cont, "ssl", "failed key");
 
@@ -359,7 +374,7 @@ int ssl_connect (Connection *conn, BOOL is_client DEBUGPARAM)
  *
  * Calls default sockread() for non-SSL connections.
  */
-int ssl_read (Connection *conn, UBYTE *data, UWORD len_p)
+ssl_errno_t ssl_read (Connection *conn, UBYTE *data, UWORD len_p)
 {
     int len, rc;
 #if !ENABLE_GNUTLS
@@ -427,7 +442,7 @@ int ssl_read (Connection *conn, UBYTE *data, UWORD len_p)
                 rl_printf (i18n (2527, "OpenSSL internal error: %s:%i\n"), file, line);
                 return -1;
             case SSL_ERROR_SYSCALL:
-                rl_printf (i18n (2528, "OpenSSL read error: %s\n"), strerror(tmp));
+                rl_printf (i18n (2528, "OpenSSL read error: %s\n"), strerror (errno));
                 ssl_disconnect (conn);
                 return -1;
             case SSL_ERROR_ZERO_RETURN:
@@ -453,7 +468,7 @@ int ssl_read (Connection *conn, UBYTE *data, UWORD len_p)
  *
  * Calls default sockwrite() for non-SSL connections.
  */
-int ssl_write (Connection *conn, UBYTE *data, UWORD len_p)
+ssl_errno_t ssl_write (Connection *conn, UBYTE *data, UWORD len_p)
 {
     int len = 0;
 #if !ENABLE_GNUTLS
@@ -593,7 +608,7 @@ int ssl_handshake (Connection *conn DEBUGPARAM)
     conn->connect = CONNECT_OK | CONNECT_SELECT_R;
     if (prG->verbose)
     {
-        rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
+        rl_log_for (cont->nick, COLCONTACT);
         rl_printf (i18n (2375, "SSL handshake ok.\n"));
     }
     TCLEvent (cont, "ssl", "ok");
