@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: util_str.c,v 1.77 2005/12/22 16:33:19 kuhlmann Exp $
+ * $Id: util_str.c,v 1.86 2007-04-17 21:13:42 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -47,6 +47,7 @@
 #include <wctype.h>
 #endif
 #include <ctype.h>
+#include <assert.h>
 #include "util_str.h"
 #include "conv.h"
 #include "contact.h"
@@ -314,80 +315,176 @@ const char *s_ip (UDWORD ip)
 /*
  * Return a static string describing the status.
  */
-const char *s_status (UDWORD status)
+const char *s_status (status_t status, UDWORD nativestatus)
 {
     static char buf[200];
     
-    if (status == STATUS_ICQOFFLINE)
+    if (status == ims_offline)
         return i18n (1969, "offline");
  
-    if (status & STATUSF_ICQINV)
+    if (ContactIsInv (status))
         snprintf (buf, sizeof (buf), "%s-", i18n (1975, "invisible"));
     else
         buf[0] = '\0';
     
-    if (status & STATUSF_ICQDND)
-        snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), "%s", i18n (1971, "do not disturb"));
-    else if (status & STATUSF_ICQOCC)
-        snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), "%s", i18n (1973, "occupied"));
-    else if (status & STATUSF_ICQNA)
-        snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), "%s", i18n (1974, "not available"));
-    else if (status & STATUSF_ICQAWAY)
-        snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), "%s", i18n (1972, "away"));
-    else if (status & STATUSF_ICQFFC)
-        snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), "%s", i18n (1976, "free for chat"));
-    else if (buf[0])
-        buf[strlen (buf) - 1] = '\0';
-    else
-        snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), "%s", i18n (1970, "online"));
-    
+    switch (ContactClearInv (status))
+    {
+        case imr_dnd:     snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), "%s", i18n (1971, "do not disturb")); break;
+        case imr_occ:     snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), "%s", i18n (1973, "occupied"));       break;
+        case imr_na:      snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), "%s", i18n (1974, "not available"));  break;
+        case imr_away:    snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), "%s", i18n (1972, "away"));           break;
+        case imr_ffc:     snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), "%s", i18n (1976, "free for chat"));  break;
+        case imr_offline: assert (0);
+        case imr_online:
+            if (buf[0])
+                buf[strlen (buf) - 1] = '\0';
+            else
+                snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), "%s", i18n (1970, "online"));
+    }
+
     if (prG->verbose)
-        snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), " %08lx", status);
+        snprintf (buf + strlen (buf), sizeof(buf) - strlen (buf), " %08lx", nativestatus);
     
     return buf;
 }
 
+/*
+ * Return a static string short describing the status.
+ */
+const char *s_status_short (status_t status)
+{
+    static char buf[20];
+    
+    if (status == ims_offline)
+        return i18n (2621, "off");
+    if (status == ims_inv)
+        return i18n (2622, "inv");
+ 
+    if (ContactIsInv (status))
+        snprintf (buf, sizeof (buf), "%s-", i18n (2622, "inv"));
+    else
+        buf[0] = '\0';
+    
+    switch (ContactClearInv (status))
+    {
+        case imr_dnd:     snprintf (buf + strlen (buf), sizeof (buf) - strlen (buf), "%s", i18n (2623, "dnd"));  break;
+        case imr_occ:     snprintf (buf + strlen (buf), sizeof (buf) - strlen (buf), "%s", i18n (2624, "occ"));  break;
+        case imr_na:      snprintf (buf + strlen (buf), sizeof (buf) - strlen (buf), "%s", i18n (2625, "na"));   break;
+        case imr_away:    snprintf (buf + strlen (buf), sizeof (buf) - strlen (buf), "%s", i18n (2626, "away")); break;
+        case imr_ffc:     snprintf (buf + strlen (buf), sizeof (buf) - strlen (buf), "%s", i18n (2627, "ffc"));  break;
+        case imr_offline: assert (0);
+        case imr_online:  snprintf (buf + strlen (buf), sizeof (buf) - strlen (buf), "%s", i18n (2628, "online"));
+    }
+    return buf;
+}
+
 /*
  * Returns static string describing the given time.
  */
 const char *s_time (time_t *stamp)
 {
+    static str_s str = { NULL, 0, 0 };
     struct timeval p = {0L, 0L};
     struct tm now;
     struct tm *thetime;
-    static char tbuf[40];
-    str_s str = { NULL, 0, 0 };
     time_t nowsec;
+    size_t rc = 0;
 
-#ifdef HAVE_GETTIMEOFDAY
-    if (gettimeofday (&p, NULL) == -1)
-#endif
+    if (stamp && *stamp != NOW)
     {
-        p.tv_usec = 0L;
+        thetime = localtime (stamp);
         nowsec = time (NULL);
     }
-#ifdef HAVE_GETTIMEOFDAY
     else
-        nowsec = p.tv_sec;
+    {
+#ifdef HAVE_GETTIMEOFDAY
+        if (gettimeofday (&p, NULL) == -1)
 #endif
-
+        {
+            p.tv_usec = 0L;
+            nowsec = time (NULL);
+        }
+#ifdef HAVE_GETTIMEOFDAY
+        else
+            nowsec = p.tv_sec;
+#endif
+        thetime = &now;
+        stamp = NULL;
+    }
     now = *localtime (&nowsec);
-
-    thetime = (!stamp || *stamp == NOW) ? &now : localtime (stamp);
-
-    strftime (tbuf, sizeof (tbuf), thetime->tm_year == now.tm_year 
-        && thetime->tm_mon == now.tm_mon && thetime->tm_mday == now.tm_mday
-        ? "%H:%M:%S" : "%a %b %d %H:%M:%S %Y", thetime); /*"*/
-
-    if (prG->verbose > 7)
-        snprintf (tbuf + strlen (tbuf), sizeof (tbuf) - strlen (tbuf),
-                  ".%.06ld", !stamp || *stamp == NOW ? p.tv_usec : 0);
-    else if (prG->verbose > 1)
-        snprintf (tbuf + strlen (tbuf), sizeof (tbuf) - strlen (tbuf),
-                  ".%.03ld", !stamp || *stamp == NOW ? p.tv_usec / 1000 : 0);
     
-    str.txt = tbuf;
-    str.len = strlen (tbuf);
+    while (!rc)
+    {
+        rc = strftime (str.txt, str.max,
+                    thetime->tm_year == now.tm_year 
+                 && thetime->tm_mon  == now.tm_mon
+                 && thetime->tm_mday == now.tm_mday
+                 ? "%H:%M:%S" : "%a %b %d %H:%M:%S %Y", thetime); /*"*/
+        if (rc <= 0 || rc >= str.max)
+        {
+            rc = 0;
+            s_blow (&str, 32);
+        }
+        else
+        {
+            str.txt[str.max - 1] = 0;
+            str.len = strlen (str.txt);
+        }
+    }
+    if (prG->verbose > 7)
+        s_catf (&str, ".%.06ld", p.tv_usec);
+    else if (prG->verbose > 1)
+        s_catf (&str, ".%.03ld", p.tv_usec / 1000);
+    
+    return ConvFrom (&str, prG->enc_loc)->txt;
+}
+
+/*
+ * Returns static string describing the given time in strftime format.
+ */
+const char *s_strftime (time_t *stamp, const char *fmt, char as_gmt)
+{
+    static str_s str = { NULL, 0, 0 };
+    struct tm *thetime;
+    size_t rc = 0;
+    char *dotfmt;
+    
+    /* strftime()'s error reporting is incomplete, so make sure a correct result is never empty */
+    dotfmt = malloc (strlen (fmt) + 2);
+    strcpy (dotfmt, fmt);
+    strcat (dotfmt, ".");
+
+    if (stamp && *stamp != NOW)
+        thetime = as_gmt ? gmtime (stamp) : localtime (stamp);
+    else
+    {
+        time_t nowsec = time (NULL);
+        thetime = as_gmt ? gmtime (&nowsec) : localtime (&nowsec);
+    }
+    
+    s_init (&str, "", 32);
+    
+    while (!rc)
+    {
+        rc = strftime (str.txt, str.max, dotfmt, thetime);
+        if (rc <= 0 || rc >= str.max)
+        {
+            rc = 0;
+            s_blow (&str, 32);
+        }
+        else
+        {
+            str.txt[str.max - 1] = 0;
+            str.len = strlen (str.txt);
+            if (str.len && str.txt [str.len - 1] == '.')
+            {
+                str.txt [str.len - 1] = 0;
+                str.len--;
+            }
+        }
+    }
+    s_free (dotfmt);
+    
     return ConvFrom (&str, prG->enc_loc)->txt;
 }
 
